Package: dimRed
Title: A Framework for Dimensionality Reduction
Version: 0.2.7
Authors@R: c(
            person("Guido", "Kraemer", email = "guido.kraemer@uni-leipzig.de", role = c("aut","cre"),
              comment = c(ORCID = "0000-0003-4865-5041")
            )
           )
Description: A collection of dimensionality reduction
    techniques from R packages and a common
    interface for calling the methods.
Depends: R (>= 3.0.0), DRR
Imports: magrittr, methods
Suggests: NMF, MASS, Matrix, RANN, RSpectra, Rtsne, cccd, coRanking,
        diffusionMap, energy, fastICA, ggplot2, graphics, igraph,
        keras, kernlab, knitr, optimx, pcaL1, pcaPP, reticulate, rgl,
        scales, scatterplot3d, stats, tensorflow, testthat, tidyr,
        tinytex, umap, vegan
VignetteBuilder: knitr
License: GPL-3 | file LICENSE
BugReports: https://github.com/gdkrmr/dimRed/issues
URL: https://www.guido-kraemer.com/software/dimred/
Encoding: UTF-8
Collate: 'autoencoder.R' 'misc.R' 'dimRedData-class.R' 'dataSets.R'
        'dimRedMethod-class.R' 'dimRedResult-class.R' 'diffmap.R'
        'dimRed.R' 'drr.R' 'embed.R' 'fastica.R' 'get_info.R'
        'graph_embed.R' 'hlle.R' 'isomap.R' 'kpca.R' 'l1pca.R' 'leim.R'
        'lle.R' 'loe.R' 'mds.R' 'mixColorSpaces.R' 'nmds.R' 'nnmf.R'
        'pca.R' 'plot.R' 'quality.R' 'rotate.R' 'soe.R' 'tsne.R'
        'umap.R'
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2025-04-22 16:49:13 UTC; gkraemer
Author: Guido Kraemer [aut, cre] (<https://orcid.org/0000-0003-4865-5041>)
Maintainer: Guido Kraemer <guido.kraemer@uni-leipzig.de>
Repository: CRAN
Date/Publication: 2025-04-23 15:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 03:21:14 UTC; unix

Package: dimRed
Title: A Framework for Dimensionality Reduction
Version: 0.2.7
Authors@R: c(
            person("Guido", "Kraemer", email = "guido.kraemer@uni-leipzig.de", role = c("aut","cre"),
              comment = c(ORCID = "0000-0003-4865-5041")
            )
           )
Description: A collection of dimensionality reduction
    techniques from R packages and a common
    interface for calling the methods.
Depends: R (>= 3.0.0), DRR
Imports: magrittr, methods
Suggests: NMF, MASS, Matrix, RANN, RSpectra, Rtsne, cccd, coRanking,
        diffusionMap, energy, fastICA, ggplot2, graphics, igraph,
        keras, kernlab, knitr, optimx, pcaL1, pcaPP, reticulate, rgl,
        scales, scatterplot3d, stats, tensorflow, testthat, tidyr,
        tinytex, umap, vegan
VignetteBuilder: knitr
License: GPL-3 | file LICENSE
BugReports: https://github.com/gdkrmr/dimRed/issues
URL: https://www.guido-kraemer.com/software/dimred/
Encoding: UTF-8
Collate: 'autoencoder.R' 'misc.R' 'dimRedData-class.R' 'dataSets.R'
        'dimRedMethod-class.R' 'dimRedResult-class.R' 'diffmap.R'
        'dimRed.R' 'drr.R' 'embed.R' 'fastica.R' 'get_info.R'
        'graph_embed.R' 'hlle.R' 'isomap.R' 'kpca.R' 'l1pca.R' 'leim.R'
        'lle.R' 'loe.R' 'mds.R' 'mixColorSpaces.R' 'nmds.R' 'nnmf.R'
        'pca.R' 'plot.R' 'quality.R' 'rotate.R' 'soe.R' 'tsne.R'
        'umap.R'
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2025-04-22 16:49:13 UTC; gkraemer
Author: Guido Kraemer [aut, cre] (<https://orcid.org/0000-0003-4865-5041>)
Maintainer: Guido Kraemer <guido.kraemer@uni-leipzig.de>
Repository: CRAN
Date/Publication: 2025-04-23 15:00:02 UTC
Built: R 4.6.0; ; 2026-04-27 16:19:23 UTC; unix

# Generated by roxygen2: do not edit by hand

export(AUC_lnK_R_NX)
export(DRR)
export(DiffusionMaps)
export(DrL)
export(FastICA)
export(FruchtermanReingold)
export(HLLE)
export(Isomap)
export(KamadaKawai)
export(LCMC)
export(MDS)
export(NNMF)
export(PCA)
export(PCA_L1)
export(Q_NX)
export(Q_global)
export(Q_local)
export(R_NX)
export(UMAP)
export(dataSetList)
export(dimRedData)
export(dimRedMethodList)
export(dimRedQualityList)
export(dimRedResult)
export(distance_correlation)
export(embed)
export(getRotationMatrix)
export(installSuggests)
export(inverse)
export(kPCA)
export(loadDataSet)
export(mean_R_NX)
export(mixColor1Ramps)
export(mixColor2Ramps)
export(mixColor3Ramps)
export(mixColorRamps)
export(nMDS)
export(plot)
export(plot_R_NX)
export(predict)
export(quality)
export(reconstruction_error)
export(reconstruction_rmse)
export(tSNE)
export(total_correlation)
exportClasses(DRR)
exportClasses(DiffusionMaps)
exportClasses(DrL)
exportClasses(FastICA)
exportClasses(FruchtermanReingold)
exportClasses(HLLE)
exportClasses(Isomap)
exportClasses(KamadaKawai)
exportClasses(MDS)
exportClasses(NNMF)
exportClasses(PCA)
exportClasses(PCA_L1)
exportClasses(UMAP)
exportClasses(dimRedData)
exportClasses(dimRedMethod)
exportClasses(dimRedResult)
exportClasses(kPCA)
exportClasses(nMDS)
exportClasses(tSNE)
exportMethods("[")
exportMethods(AUC_lnK_R_NX)
exportMethods(LCMC)
exportMethods(Q_NX)
exportMethods(Q_global)
exportMethods(Q_local)
exportMethods(R_NX)
exportMethods(as.data.frame)
exportMethods(as.dimRedData)
exportMethods(cophenetic_correlation)
exportMethods(distance_correlation)
exportMethods(embed)
exportMethods(getData)
exportMethods(getDimRedData)
exportMethods(getMeta)
exportMethods(getNDim)
exportMethods(getOrgData)
exportMethods(getOtherData)
exportMethods(getPars)
exportMethods(inverse)
exportMethods(maximize_correlation)
exportMethods(mean_R_NX)
exportMethods(ndims)
exportMethods(nrow)
exportMethods(plot)
exportMethods(predict)
exportMethods(print)
exportMethods(quality)
exportMethods(reconstruction_error)
exportMethods(reconstruction_rmse)
exportMethods(total_correlation)
import(DRR)
import(methods)
import(utils)
importFrom(grDevices,colorRamp)
importFrom(grDevices,rgb)
importFrom(graphics,plot)
importFrom(magrittr,"%>%")
importFrom(stats,predict)

## ----"pca_isomap_example",include=FALSE,fig.width=4,fig.height=4--------------
if(Sys.getenv("BNET_BUILD_VIGNETTE") != "") {
  library(dimRed); library(ggplot2); #library(dplyr); library(tidyr)
  ## define which methods to apply
  embed_methods <- c("Isomap", "PCA")
  ## load test data set
  data_set <- loadDataSet("3D S Curve", n = 1000)
  ## apply dimensionality reduction
  data_emb <- lapply(embed_methods, function(x) embed(data_set, x))
  names(data_emb) <- embed_methods
  ## plot data set, embeddings, and quality analysis
  ## plot(data_set, type = "3vars")
  ## lapply(data_emb, plot, type = "2vars")
  ## plot_R_NX(data_emb)

  add_label <- function(label)
    grid::grid.text(label, 0.2, 1, hjust = 0, vjust = 1,
                    gp = grid::gpar(fontface = "bold",
                                    cex = 1.5))
  ## pdf('~/phd/text/dimRedPackage/plots/plot_example.pdf', width = 4, height = 4)
  ## plot the results
  plot(data_set, type = "3vars", angle = 15, mar = c(3, 3, 0, 0), box = FALSE, grid = FALSE, pch = 16)
  add_label("a")
  par(mar = c(4, 4, 0, 0) + 0.1, bty = "n", las = 1)
  plot(data_emb$Isomap, type = "2vars", pch = 16)
  add_label("b")
  plot(data_emb$PCA,    type = "2vars", pch = 16)
  add_label("d")
  ## calculate quality scores
  print(
  plot_R_NX(data_emb) +
  theme(legend.title = element_blank(),
  legend.position = c(0.5, 0.1),
  legend.justification = c(0.5, 0.1))
  )
  add_label("c")
} else {
  # These cannot all be plot(1:10)!!! It's a mistery to me.
  plot(1:10)
  barplot(1:10)
  hist(1:10)
  plot(1:10)
}

## ----eval=FALSE---------------------------------------------------------------
# ## define which methods to apply
# embed_methods <- c("Isomap", "PCA")
# ## load test data set
# data_set <- loadDataSet("3D S Curve", n = 1000)
# ## apply dimensionality reduction
# data_emb <- lapply(embed_methods, function(x) embed(data_set, x))
# names(data_emb) <- embed_methods
# ## figure \ref{fig:plotexample}a, the data set
# plot(data_set, type = "3vars")
# ## figures \ref{fig:plotexample}b (Isomap) and \ref{fig:plotexample}d (PCA)
# lapply(data_emb, plot, type = "2vars")
# ## figure \ref{fig:plotexample}c, quality analysis
# plot_R_NX(data_emb)

## ----include=FALSE------------------------------------------------------------
if(Sys.getenv("BNET_BUILD_VIGNETTE") != "") {
  library(dimRed)
  library(cccd)
  ## Load data
  ss <- loadDataSet("3D S Curve", n = 500)
  ## Parameter space
  kk <- floor(seq(5, 100, length.out = 40))
  ## Embedding over parameter space
  emb <- lapply(kk, function(x) embed(ss, "Isomap", knn = x))
  ## Quality over embeddings
  qual <- sapply(emb, function(x) quality(x, "Q_local"))
  ## Find best value for K
  ind_max <- which.max(qual)
  k_max <- kk[ind_max]

  add_label <- function(label){
    par(xpd = TRUE)
    b = par("usr")
    text(b[1], b[4], label, adj = c(0, 1), cex = 1.5, font = 2)
    par(xpd = FALSE)
  }

  names(qual) <- kk
}

## ----"select_k",include=FALSE,fig.width=11,fig.height=5-----------------------
if (Sys.getenv("BNET_BUILD_VIGNETTE") != "") {
  par(mfrow = c(1, 2),
      mar = c(5, 4, 0, 0) + 0.1,
      oma = c(0, 0, 0, 0))
  plot(kk, qual, type = "l", xlab = "k", ylab = expression(Q[local]), bty = "n")
  abline(v = k_max, col = "red")
  add_label("a")
  plot(ss, type = "3vars", angle = 15, mar = c(3, 3, 0, 0), box = FALSE, grid = FALSE, pch = 16)
  add_label("b")
} else {
  plot(1:10)
  plot(1:10)
}

## ----"knngraphs",include=FALSE,fig.width=8,fig.height=3-----------------------
if(Sys.getenv("BNET_BUILD_VIGNETTE") != "") {
par(mfrow = c(1, 3),
    mar = c(5, 4, 0, 0) + 0.1,
    oma = c(0, 0, 0, 0))
add_knn_graph <- function(ind) {
    nn1 <- nng(ss@data, k = kk[ind])
    el <- get.edgelist(nn1)
    segments(x0 = emb[[ind]]@data@data[el[, 1], 1],
             y0 = emb[[ind]]@data@data[el[, 1], 2],
             x1 = emb[[ind]]@data@data[el[, 2], 1],
             y1 = emb[[ind]]@data@data[el[, 2], 2],
             col = "#00000010")
}
plot(emb[[2]]@data@data, type = "n", bty = "n")
add_knn_graph(2)
points(emb[[2]]@data@data, col = dimRed:::colorize(ss@meta),
       pch = 16)
add_label("c")
plot(emb[[ind_max]]@data@data, type = "n", bty = "n")
add_knn_graph(ind_max)
points(emb[[ind_max]]@data@data, col = dimRed:::colorize(ss@meta),
       pch = 16)
add_label("d")
plot(emb[[length(emb)]]@data@data, type = "n", bty = "n")
add_knn_graph(length(emb))
points(emb[[length(emb)]]@data@data, col = dimRed:::colorize(ss@meta),
       pch = 16)
add_label("e")
} else {
plot(1:10)
plot(1:10)
plot(1:10)
}

## ----eval=FALSE---------------------------------------------------------------
# ## Load data
# ss <- loadDataSet("3D S Curve", n = 500)
# ## Parameter space
# kk <- floor(seq(5, 100, length.out = 40))
# ## Embedding over parameter space
# emb <- lapply(kk, function(x) embed(ss, "Isomap", knn = x))
# ## Quality over embeddings
# qual <- sapply(emb, function(x) quality(x, "Q_local"))
# ## Find best value for K
# ind_max <- which.max(qual)
# k_max <- kk[ind_max]

## ----"plot_quality",include=FALSE---------------------------------------------
if(Sys.getenv("BNET_BUILD_VIGNETTE") != "") {
  embed_methods <- dimRedMethodList()
  quality_methods <- c("Q_local", "Q_global", "AUC_lnK_R_NX",
                      "cophenetic_correlation")
  iris_data <- loadDataSet("Iris")
  quality_results <- matrix(
      NA, length(embed_methods), length(quality_methods),
      dimnames = list(embed_methods, quality_methods)
)
embedded_data <- list()

for (e in embed_methods) {
  try(embedded_data[[e]] <- embed(iris_data, e))
  for (q in quality_methods)
    try(quality_results[e,q] <- quality(embedded_data[[e]], q))
}

quality_results <- quality_results[order(rowMeans(quality_results)), ]

palette(c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e"))
col_hsv <- rgb2hsv(col2rgb(palette()))
## col_hsv["v", ] <- col_hsv["v", ] * 3 / 1
palette(hsv(col_hsv["h", ], col_hsv["s", ], col_hsv["v", ]))
par(mar = c(2, 8, 0, 0) + 0.1)
barplot(t(quality_results), beside = TRUE, col = 1:4,
        legend.text = quality_methods, horiz = TRUE, las = 1,
        cex.names = 0.85,
        args.legend = list(x = "topleft", bg = "white", cex = 0.8))
} else {
  plot(1:10)
}

## ----eval=FALSE---------------------------------------------------------------
# embed_methods <- dimRedMethodList()
# quality_methods <- c("Q_local", "Q_global", "AUC_lnK_R_NX",
#                      "cophenetic_correlation")
# scurve <- loadDataSet("3D S Curve", n = 2000)
# quality_results <- matrix(
#   NA, length(embed_methods), length(quality_methods),
#   dimnames = list(embed_methods, quality_methods)
# )
# 
# embedded_data <- list()
# for (e in embed_methods) {
#   embedded_data[[e]] <- embed(scurve, e)
#   for (q in quality_methods) {
#     try(quality_results[e, q] <- quality(embedded_data[[e]], q))
#   }
# }

## #' Locally Linear Embedding
## #'
## #' An S4 Class implementing Locally Linear Embedding (LLE)
## #'
## #' LLE approximates the points in the manifold by linear combination
## #' of its neighbors. These linear combinations are the same inside the
## #' manifold and in highdimensional space.
## #'
## #' @template dimRedMethodSlots
## #'
## #' @template dimRedMethodGeneralUsage
## #'
## #' @section Parameters:
## #' LLE can take the following parameters:
## #' \describe{
## #'   \item{knn}{the number of neighbors for the knn graph., defaults to 50.}
## #'   \item{ndim}{the number of embedding dimensions, defaults to 2.}
## #' }
## #'
## #' @section Implementation:
## #' Wraps around \code{\link[lle]{lle}}, only
## #' exposes the parameters \code{k} and \code{m}.
## #'
## #' @references
## #'
## #' Roweis, S.T., Saul, L.K., 2000. Nonlinear Dimensionality Reduction
## #' by Locally Linear Embedding. Science 290,
## #' 2323-2326. doi:10.1126/science.290.5500.2323
## #'
## #' @examples
## #' \dontrun{
## #' if(requireNamespace("lle")) {
## #'
## #' dat <- loadDataSet("3D S Curve", n = 500)
## #' emb <- embed(dat, "LLE", knn = 45)
## #' plot(emb, type = "2vars")
## #'
## #' }
## #' }
## #' @include dimRedResult-class.R
## #' @include dimRedMethod-class.R
## #' @family dimensionality reduction methods
## #' @export LLE
## #' @exportClass LLE
## LLE <- setClass(
##     "LLE",
##     contains = "dimRedMethod",
##     prototype = list(
##         stdpars = list(knn = 50, ndim = 2),
##         fun = function (data, pars,
##                         keep.org.data = TRUE) {
##           chckpkg("lle")
##           meta <- data@meta
##           orgdata <- if (keep.org.data) data@data else NULL
##           indata <- data@data

##           outdata <- lle::lle(indata,
##                               k = pars$knn,
##                               m = pars$ndim)$Y
##           if (is.null(dim(outdata))) {
##             dim(outdata) <- c(length(outdata), 1)
##           }
##           colnames(outdata) <- paste0("LLE", 1:ncol(outdata))

##           return(new(
##             "dimRedResult",
##             data         = new("dimRedData",
##                                data = outdata,
##                                meta = meta),
##             org.data     = orgdata,
##             has.org.data = keep.org.data,
##             method       = "lle",
##             pars         = pars
##           ))
##         },
##         requires = c("lle")
##     )
## )

#' Principal Component Analysis
#'
#' S4 Class implementing PCA.
#'
#' PCA transforms the data in orthogonal components so that the first
#' axis accounts for the larges variance in the data, all the
#' following axes account for the highest variance under the
#' constraint that they are orthogonal to the preceding axes.  PCA is
#' sensitive to the scaling of the variables. PCA is by far the
#' fastest and simples method of dimensionality reduction and should
#' probably always be applied as a baseline if other methods are tested.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' PCA can take the following parameters:
#' \describe{
#'   \item{ndim}{The number of output dimensions.}
#'   \item{center}{logical, should the data be centered, defaults to \code{TRUE}.}
#'   \item{scale.}{logical, should the data be scaled, defaults to \code{FALSE}.}
#' }
#'
#' @section Implementation:
#'
#' Wraps around \code{\link{prcomp}}. Because PCA can be reduced to a
#' simple rotation, forward and backward projection functions are
#' supplied.
#'
#' @references
#'
#' Pearson, K., 1901. On lines and planes of closest fit to systems of points in
#' space. Philosophical Magazine 2, 559-572.
#'
#' @examples
#' dat <- loadDataSet("Iris")
#' emb <- embed(dat, "PCA")
#'
#' plot(emb, type = "2vars")
#' if(requireNamespace("scatterplot3d", quietly = TRUE))
#'   plot(inverse(emb, getDimRedData(emb)), type = "3vars")
#'
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export PCA
#' @exportClass PCA
PCA <- setClass(
    "PCA",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(ndim = 2,
                       center = TRUE,
                       scale. = FALSE),
        fun = function (data, pars,
                        keep.org.data = TRUE) {
        ndim <- pars$ndim
        pars$ndim <- NULL

        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        data <- data@data
        res <- do.call(
            prcomp,
            c(list(x = data), pars)
        )

                                        # evaluate results here for functions
        data <- res$x[, seq_len(ndim), drop = FALSE]
        ce <- res$center
        sc <- res$scale
        rot <- res$rotation[, seq_len(ndim)]
        rerot <- t(rot)


        appl <- function(x) {
            appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
            proj <- if (inherits(x, "dimRedData")) x@data else x

            if (ncol(proj) != ncol(orgdata))
                stop("x must have the same number of dimensions ",
                     "as the original data")


            if (ce[1]  != FALSE) proj <- t(apply(proj, 1, function(x) x - ce))
            if (sc[1]  != FALSE) proj <- t(apply(proj, 1, function(x) x / sc))
            proj <- proj %*% rot

            proj <- new("dimRedData", data = proj, meta = appl.meta)
            return(proj)
        }
        inv  <- function(x) {
            appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
            proj <- if (inherits(x, "dimRedData")) x@data else x
            if (ncol(proj) > ncol(data))
                stop("x must have less or equal number of dimensions ",
                     "as the original data")


            d <- ncol(proj)
            reproj <- proj %*% rerot[seq_len(d), ]

            if (sc[1] != FALSE)
                reproj <- t(apply(reproj, 1, function(x) x * sc))
            if (ce[1] != FALSE)
                reproj <- t(apply(reproj, 1, function(x) x + ce))

            reproj <- new("dimRedData", data = reproj, meta = appl.meta)

            return(reproj)
        }

        res <- new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = data,
                               meta = meta),
            org.data     = orgdata,
            apply        = appl,
            inverse      = inv,
            has.org.data = keep.org.data,
            has.apply    = TRUE,
            has.inverse  = TRUE,
            method       = "PCA",
            pars         = pars
        )

        return(res)
    })
)

#' Non-Metric Dimensional Scaling
#'
#' An S4 Class implementing Non-Metric Dimensional Scaling.
#'
#' A non-linear extension of MDS using monotonic regression
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' nMDS can take the following parameters:
#' \describe{
#'   \item{d}{A distance function.}
#'   \item{ndim}{The number of embedding dimensions.}
#' }
#'
#' @section Implementation:
#' Wraps around the
#' \code{\link[vegan]{monoMDS}}. For parameters that are not
#' available here, the standard configuration is used.
#'
#' @references
#'
#' Kruskal, J.B., 1964. Nonmetric multidimensional scaling: A numerical method.
#' Psychometrika 29, 115-129. https://doi.org/10.1007/BF02289694
#'
#' @examples
#' if(requireNamespace("vegan", quietly = TRUE)) {
#'
#' dat <- loadDataSet("3D S Curve", n = 300)
#' emb <- embed(dat, "nMDS")
#' plot(emb, type = "2vars")
#'
#' }
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export nMDS
#' @exportClass nMDS
nMDS <- setClass(
    "nMDS",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(d = stats::dist, ndim = 2),
        fun = function (data, pars,
                        keep.org.data = TRUE) {
        chckpkg("vegan")

        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        indata <- data@data

        outdata <- vegan::monoMDS(pars$d(indata), k = pars$ndim)$points

        colnames(outdata) <- paste0("NMDS", 1:ncol(outdata))

        return(new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = outdata,
                               meta = meta),
            org.data     = orgdata,
            has.org.data = keep.org.data,
            method       = "nmds",
            pars         = pars
        ))
        },
      requires = c("vegan"))
)

#' Independent Component Analysis
#'
#' An S4 Class implementing the FastICA algorithm for Indepentend
#' Component Analysis.
#'
#' ICA is used for blind signal separation of different sources. It is
#' a linear Projection.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' FastICA can take the following parameters:
#' \describe{
#'   \item{ndim}{The number of output dimensions. Defaults to \code{2}}
#' }
#'
#' @section Implementation:
#' Wraps around \code{\link[fastICA]{fastICA}}. FastICA uses a very
#' fast approximation for negentropy to estimate statistical
#' independences between signals. Because it is a simple
#' rotation/projection, forward and backward functions can be given.
#'
#' @references
#'
#' Hyvarinen, A., 1999. Fast and robust fixed-point algorithms for independent
#' component analysis. IEEE Transactions on Neural Networks 10, 626-634.
#' https://doi.org/10.1109/72.761722
#'
#' @examples
#' if(requireNamespace("fastICA", quietly = TRUE)) {
#'
#' dat <- loadDataSet("3D S Curve")
#' emb <- embed(dat, "FastICA", ndim = 2)
#' plot(getData(getDimRedData(emb)))
#'
#' }
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export FastICA
#' @exportClass FastICA
FastICA <- setClass(
    "FastICA",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(ndim = 2),
        fun = function (data,
                        pars,
                        keep.org.data = TRUE) {
        chckpkg("fastICA")

        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        orgdata.colmeans <- colMeans(orgdata)
        indata <- data@data

        res <- fastICA::fastICA(indata, n.comp = pars$ndim, method = "C")

        outdata <- res$S
        colnames(outdata) <- paste0("ICA", 1:ncol(outdata))

        appl <- function(x){
            appl.meta <- if (inherits(x, "dimRedData"))
                             x@meta
                         else
                             matrix(numeric(0), 0, 0)

            proj <- if (inherits(x, "dimRedData"))
                        x@data
                    else
                        x

            out <- scale(proj, center = orgdata.colmeans, scale = FALSE) %*%
                res$K %*%
                res$W
            colnames(out) <- paste0("ICA", 1:ncol(out))
            return(new("dimRedData", data = out, meta = appl.meta))
        }

        inv <- function(x){
            appl.meta <- if (inherits(x, "dimRedData"))
                             x@meta
                         else
                             matrix(numeric(0), 0, 0)

            proj <- if (inherits(x, "dimRedData"))
                        x@data
                    else
                        x

            out <- scale(proj %*% res$A[1:ncol(proj), ],
                         center = -orgdata.colmeans,
                         scale = FALSE)
            reproj <- new("dimRedData", data = out, meta = appl.meta)
            return(reproj)
        }


        return(new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = outdata,
                               meta = meta),
            org.data     = orgdata,
            has.org.data = keep.org.data,
            apply        = appl,
            inverse      = inv,
            has.apply    = TRUE,
            has.inverse  = TRUE,
            method       = "FastICA",
            pars         = pars
        ))
        },
      requires = c("fastICA"))
)

#' @include dimRedResult-class.R
#' @include dimRedData-class.R

#' @export
setGeneric("quality",
           function (.data, ...) standardGeneric("quality"),
           valueClass = "numeric")

#' Quality Criteria for dimensionality reduction.
#'
#' A collection of functions to compute quality measures on
#' \code{\link{dimRedResult}} objects.
#'
#' @section Implemented methods:
#'
#' Method must be one of \code{"\link{Q_local}", "\link{Q_global}",
#' "\link{mean_R_NX}", "\link{total_correlation}",
#' "\link{cophenetic_correlation}", "\link{distance_correlation}",
#' "\link{reconstruction_rmse}"}
#'
#' @section Rank based criteria:
#'
#' \code{Q_local}, \code{Q_global}, and \code{mean_R_NX} are
#' quality criteria based on the Co-ranking matrix.  \code{Q_local}
#' and \code{Q_global} determine the local/global quality of the
#' embedding, while \code{mean_R_NX} determines the quality of the
#' overall embedding. They are parameter free and return a single
#' number. The object must include the original data.  The number
#' returns is in the range [0, 1], higher values mean a better
#' local/global embedding.
#'
#' @section Correlation based criteria:
#'
#' \code{total_correlation} calculates the sum of the mean squared
#' correlations of the original axes with the axes in reduced
#' dimensions, because some methods do not care about correlations
#' with axes, there is an option to rotate data in reduced space to
#' maximize this criterium. The number may be greater than one if more
#' dimensions are summed up.
#'
#' \code{cophenetic_correlation} calculate the correlation between the
#' lower triangles of distance matrices, the correlation and distance
#' methods may be specified. The result is in range [-1, 1].
#'
#' \code{distance_correlation} measures the independes of samples by
#' calculating the correlation of distances. For details see
#' \code{\link[energy]{dcor}}.
#'
#' @section Reconstruction error:
#'
#' \code{reconstruction_rmse} calculates the root mean squared error
#' of the reconstrucion. \code{object} requires an inverse function.
#'
#'
#' @references
#'
#' Lueks, W., Mokbel, B., Biehl, M., Hammer, B., 2011. How
#'     to Evaluate Dimensionality Reduction? - Improving the
#'     Co-ranking Matrix. arXiv:1110.3917 [cs].
#'
#' Szekely, G.J., Rizzo, M.L., Bakirov, N.K., 2007. Measuring and
#'     testing dependence by correlation of distances. Ann. Statist. 35,
#'     2769-2794. doi:10.1214/009053607000000505
#'
#' Lee, J.A., Peluffo-Ordonez, D.H., Verleysen, M., 2015. Multi-scale
#'     similarities in stochastic neighbour embedding: Reducing
#'     dimensionality while preserving both local and global
#'     structure. Neurocomputing, 169,
#'     246-261. doi:10.1016/j.neucom.2014.12.095
#'
#'
#'
#' @param .data object of class \code{dimRedResult}
#' @param .method character vector naming one of the methods
#' @param .mute what output from the embedding method should be muted.
#' @param ... the pameters, internally passed as a list to the
#'     quality method as \code{pars = list(...)}
#' @return a number
#'
#' @examples
#' \dontrun{
#' embed_methods <- dimRedMethodList()
#' quality_methods <- dimRedQualityList()
#' scurve <- loadDataSet("Iris")
#'
#' quality_results <- matrix(NA, length(embed_methods), length(quality_methods),
#'                               dimnames = list(embed_methods, quality_methods))
#' embedded_data <- list()
#'
#' for (e in embed_methods) {
#'   message("embedding: ", e)
#'   embedded_data[[e]] <- embed(scurve, e, .mute = c("message", "output"))
#'   for (q in quality_methods) {
#'     message("  quality: ", q)
#'     quality_results[e, q] <- tryCatch(
#'       quality(embedded_data[[e]], q),
#'       error = function (e) NA
#'     )
#'   }
#' }
#'
#' print(quality_results)
#' }
#' @author Guido Kraemer
#' @aliases quality quality.dimRedResult
#' @family Quality scores for dimensionality reduction
#' @describeIn quality Calculate a quality index from a dimRedResult object.
#' @export
setMethod(
    "quality",
    "dimRedResult",
    function (.data, .method = dimRedQualityList(),
              .mute = character(0), # c("output", "message"),
              ...) {
        method <-  match.arg(.method)

        methodFunction <- getQualityFunction(method)

        args <- c(list(object = .data), list(...))

        devnull <- if (Sys.info()["sysname"] != "Windows")
                       "/dev/null"
                   else
                       "NUL"
        if ("message" %in% .mute){
            devnull1 <- file(devnull,  "wt")
            sink(devnull1, type = "message")
            on.exit({
                sink(file = NULL, type = "message")
                close(devnull1)
            }, add = TRUE)
        }
        if ("output" %in% .mute) {
            devnull2 <- file(devnull,  "wt")
            sink(devnull2, type = "output")
            on.exit({
                sink()
                close(devnull2)
            }, add = TRUE)
        }

        do.call(methodFunction, args)
    }
)

getQualityFunction <- function (method) {
    switch(
        method,
        Q_local                = Q_local,
        Q_global               = Q_global,
        mean_R_NX              = mean_R_NX,
        AUC_lnK_R_NX           = AUC_lnK_R_NX,
        total_correlation      = total_correlation,
        cophenetic_correlation = cophenetic_correlation,
        distance_correlation   = distance_correlation,
        reconstruction_rmse    = reconstruction_rmse
    )
}


#' @export
setGeneric(
    "Q_local",
    function(object, ...) standardGeneric("Q_local"),
    valueClass = "numeric"
)


#' Method Q_local
#'
#' Calculate the Q_local score to assess the quality of a dimensionality reduction.
#'
#' @param object of class dimRedResult.
#' @param ndim use the first ndim columns of the embedded data for calculation.
#' @family Quality scores for dimensionality reduction
#' @aliases Q_local
#' @export
setMethod(
    "Q_local",
    "dimRedResult",
    function (object, ndim = getNDim(object)) {
        if (!object@has.org.data) stop("object requires original data")
        chckpkg("coRanking")

        Q <- coRanking::coranking(object@org.data,
                                  object@data@data[, seq_len(ndim), drop = FALSE])
        nQ <- nrow(Q)
        N <- nQ + 1

        Qnx <- diag(apply(apply(Q, 2, cumsum), 1, cumsum)) / seq_len(nQ) / N
        lcmc <- Qnx - seq_len(nQ) / nQ

        Kmax <- which.max(lcmc)

        Qlocal <- sum(Qnx[1:Kmax]) / Kmax
        return(as.vector(Qlocal))
    }
)

#' @export
setGeneric(
    "Q_global",
    function(object, ...) standardGeneric("Q_global"),
    valueClass = "numeric"
)

#' Method Q_global
#'
#' Calculate the Q_global score to assess the quality of a dimensionality reduction.
#'
#' @param object of class dimRedResult
#' @family Quality scores for dimensionality reduction
#' @aliases Q_global
#' @export
setMethod(
    "Q_global",
    "dimRedResult",
    function(object){
        if (!object@has.org.data) stop("object requires original data")
        chckpkg("coRanking")

        Q <- coRanking::coranking(object@org.data, object@data@data)
        nQ <- nrow(Q)
        N <- nQ + 1

        Qnx <- diag(apply(apply(Q, 2, cumsum), 1, cumsum)) / seq_len(nQ) / N
        lcmc <- Qnx - seq_len(nQ) / nQ

        Kmax <- which.max(lcmc)

        Qglobal <- sum(Qnx[(Kmax + 1):nQ]) / (N - Kmax)
        return(Qglobal)
    }
)

#' @export
setGeneric(
    "mean_R_NX",
    function(object, ...) standardGeneric("mean_R_NX"),
    valueClass = "numeric"
)

#' Method mean_R_NX
#'
#' Calculate the mean_R_NX score to assess the quality of a dimensionality reduction.
#'
#' @param object of class dimRedResult
#' @family Quality scores for dimensionality reduction
#' @aliases mean_R_NX
#' @export
setMethod(
    "mean_R_NX",
    "dimRedResult",
    function(object) mean(R_NX(object))
)

#' @export
setGeneric(
    "AUC_lnK_R_NX",
    function(object, ...) standardGeneric("AUC_lnK_R_NX"),
    valueClass = "numeric"
)

#' Method AUC_lnK_R_NX
#'
#' Calculate the Area under the R_NX(ln K), used in Lee et. al. (2015). Note
#' that despite the name, this does not weight the mean by the logarithm, but by
#' 1/K. If explicit weighting by the logarithm is desired use \code{weight =
#' "log"} or \code{weight = "log10"}
#'
#' The naming confusion originated from equation 17 in Lee et al (2015) and the
#' name of this method may change in the future to avoid confusion.
#'
#' @references Lee, J.A., Peluffo-Ordonez, D.H., Verleysen, M., 2015.
#'   Multi-scale similarities in stochastic neighbour embedding: Reducing
#'   dimensionality while preserving both local and global structure.
#'   Neurocomputing 169, 246-261. https://doi.org/10.1016/j.neucom.2014.12.095
#'
#' @param object of class dimRedResult
#' @param weight the weight function used, one of \code{c("inv", "log", "log10")}
#' @family Quality scores for dimensionality reduction
#' @aliases AUC_lnK_R_NX
#' @export
setMethod(
    "AUC_lnK_R_NX",
    "dimRedResult",
    function(object, weight = "inv") {
        rnx <- R_NX(object)

        weight <- match.arg(weight, c("inv", "ln", "log", "log10"))
        switch(
          weight,
          inv   = auc_ln_k_inv(rnx),
          log   = auc_log_k(rnx),
          ln    = auc_log_k(rnx),
          log10 = auc_log10_k(rnx),
          stop("wrong parameter for weight")
        )
    }
)

auc_ln_k_inv <- function(rnx) {
    Ks <- seq_along(rnx)
    return (sum(rnx / Ks) / sum(1 / Ks))
}

auc_log_k <- function(rnx) {
    Ks <- seq_along(rnx)
    return (sum(rnx * log(Ks)) / sum(log(Ks)))
}

auc_log10_k <- function(rnx) {
  Ks <- seq_along(rnx)
  return (sum(rnx * log10(Ks)) / sum(log10(Ks)))
}


#' @export
setGeneric(
    "total_correlation",
    function(object, ...) standardGeneric("total_correlation"),
    valueClass = "numeric"
)

#' Method total_correlation
#'
#' Calculate the total correlation of the variables with the axes to
#' assess the quality of a dimensionality reduction.
#'
#' @param object of class dimRedResult
#' @param naxes the number of axes to use for optimization.
#' @param cor_method the correlation method to use.
#' @param is.rotated if FALSE the object is rotated.
#'
#' @family Quality scores for dimensionality reduction
#' @aliases total_correlation
#' @export
setMethod(
    "total_correlation",
    "dimRedResult",
    function(object,
             naxes = ndims(object),
             cor_method = "pearson",
             is.rotated = FALSE){

        if (!object@has.org.data) stop("object requires original data")
        if (length(naxes) != 1 || naxes < 1 || naxes > ncol(object@data@data))
            stop("naxes must specify the numbers of axes to optimize for, ",
                 "i.e. a single integer between 1 and ncol(object@data@data)")
        ## try to partially match cor_method:
        cor_methods <- c("pearson", "kendall", "spearman")
        cor_method <- cor_methods[pmatch(cor_method, cor_methods)]
        if (is.na(cor_method))
            stop("cor_method must match one of ",
                 "'pearson', 'kendall', or 'spearman', ",
                 "at least partially.")

        if (!is.rotated) {
            rotated_result <- maximize_correlation(
                object, naxes, cor_method
            )
        } else {
            rotated_result <- object
        }

        res <- 0
        for (i in 1:naxes)
            res <- res + mean(correlate(
                             rotated_result@data@data,
                             rotated_result@org.data,
                             cor_method
                         )[i, ] ^ 2)

        return(res)
    }
)

setGeneric("cophenetic_correlation",
           function(object, ...) standardGeneric("cophenetic_correlation"),
           valueClass = "numeric")

#' Method cophenetic_correlation
#'
#' Calculate the correlation between the distance matrices in high and
#' low dimensioal space.
#'
#' @param object of class dimRedResult
#' @param d the distance function to use.
#' @param cor_method The correlation method.
#' @aliases cophenetic_correlation
#' @family Quality scores for dimensionality reduction
#' @export
setMethod(
    "cophenetic_correlation",
    "dimRedResult",
    function(object, d = stats::dist, cor_method = "pearson"){
        ## if (missing(d)) d <- stats::dist
        ## if (missing(cor_method)) cor_method <- "pearson"
        if (!object@has.org.data) stop("object requires original data")
        cor_methods <- c("pearson", "kendall", "spearman")
        cor_method <- cor_methods[pmatch(cor_method, cor_methods)]
        if (is.na(cor_method))
            stop("cor_method must match one of ",
                 "'pearson', 'kendall', or 'spearman', ",
                 "at least partially.")

        d.org <- d(object@org.data)
        d.emb <- d(object@data@data)

        if (!inherits(d.org, "dist") || !inherits(d.emb, "dist"))
            stop("d must return a dist object")

        res <- correlate(
            d(object@org.data),
            d(object@data@data),
            cor_method
        )
        return(res)
    }
)

#' @export
setGeneric(
    "distance_correlation",
    function(object) standardGeneric("distance_correlation"),
    valueClass = "numeric"
)

#' Method distance_correlation
#'
#' Calculate the distance correlation between the distance matrices in
#' high and low dimensioal space.
#'
#' @param object of class dimRedResult
#' @aliases distance_correlation
#' @family Quality scores for dimensionality reduction
#' @export
setMethod(
    "distance_correlation",
    "dimRedResult",
    function(object){
        if (!object@has.org.data) stop("object requires original data")
        chckpkg("energy")

        energy::dcor(object@org.data, object@data@data)
    }
)



#' @export
setGeneric(
    "reconstruction_rmse",
    function(object) standardGeneric("reconstruction_rmse"),
    valueClass = "numeric"
)

#' Method reconstruction_rmse
#'
#' Calculate the reconstruction root mean squared error a dimensionality reduction, the method must have an inverse mapping.
#'
#' @param object of class dimRedResult
#' @aliases reconstruction_rmse
#' @family Quality scores for dimensionality reduction
#' @export
setMethod(
    "reconstruction_rmse",
    "dimRedResult",
    function(object){
        if (!object@has.org.data) stop("object requires original data")
        if (!object@has.inverse) stop("object requires an inverse function")

        recon <- object@inverse(object@data)

        rmse(recon@data, object@org.data)
    }
)

#' @rdname quality
#' @param filter filter methods by installed packages
#' @export
dimRedQualityList <- function (filter = FALSE) {
  quality_list <- character()
  if (!filter || requireNamespace("coRanking", quietly = TRUE))
    quality_list <- c(quality_list, c("Q_local", "Q_global",
                                      "mean_R_NX", "AUC_lnK_R_NX"))
  if (!filter || requireNamespace("optimx", quietly = TRUE))
    quality_list <- c(quality_list, "total_correlation")
  quality_list <- c(quality_list, "cophenetic_correlation")
  if (!filter || requireNamespace("energy", quietly = TRUE))
    quality_list <- c(quality_list, "distance_correlation")
  quality_list <- c(quality_list, "reconstruction_rmse")
  return(quality_list)
}

#' @export
setGeneric(
    "R_NX",
    function(object, ...) standardGeneric("R_NX"),
    valueClass = "numeric"
)

#' Method R_NX
#'
#' Calculate the R_NX score from Lee et. al. (2013) which shows the neighborhood
#' preservation for the Kth nearest neighbors, corrected for random point
#' distributions and scaled to range [0, 1].
#' @param object of class dimRedResult
#' @param ndim the number of dimensions to take from the embedded data.
#' @family Quality scores for dimensionality reduction
#' @aliases R_NX
#' @export
setMethod(
    "R_NX",
    "dimRedResult",
    function(object, ndim = getNDim(object)) {
        chckpkg("coRanking")
        if (!object@has.org.data) stop("object requires original data")

        Q <- coRanking::coranking(object@org.data,
                                  object@data@data[, seq_len(ndim),
                                                   drop = FALSE])
        nQ <- nrow(Q)
        N <- nQ + 1

        Qnx <- diag(apply(apply(Q, 2, cumsum), 1, cumsum)) /
            seq_len(nQ) / N

        Rnx <- ((N - 1) * Qnx - seq_len(nQ)) /
            (N - 1 - seq_len(nQ))
        Rnx[-nQ]
    }
)

#' @export
setGeneric(
    "Q_NX",
    function(object, ...) standardGeneric("Q_NX"),
    valueClass = "numeric"
)

#' Method Q_NX
#'
#' Calculate the Q_NX score (Chen & Buja 2006, the notation in the
#' publication is M_k). Which is the fraction of points that remain inside
#' the same K-ary neighborhood in high and low dimensional space.
#'
#' @param object of class dimRedResult
#' @family Quality scores for dimensionality reduction
#' @aliases Q_NX
#' @export
setMethod(
    "Q_NX",
    "dimRedResult",
    function(object) {
        chckpkg("coRanking")

        Q <- coRanking::coranking(object@org.data, object@data@data)
        nQ <- nrow(Q)
        N <- nQ + 1

        Qnx <- diag(apply(apply(Q, 2, cumsum), 1, cumsum)) / seq_len(nQ) / N
        Qnx
    }
)

#'@export
setGeneric(
    "LCMC",
    function(object, ...) standardGeneric("LCMC"),
    valueClass = "numeric"
)

#' Method LCMC
#'
#' Calculates the Local Continuity Meta Criterion, which is
#' \code{\link{Q_NX}} adjusted for random overlap inside the K-ary
#' neighborhood.
#'
#' @param object of class dimRedResult
#' @family Quality scores for dimensionality reduction
#' @aliases LCMC
#' @export
setMethod(
    "LCMC",
    "dimRedResult",
        function(object) {
        chckpkg("coRanking")

        Q <- coRanking::coranking(object@org.data, object@data@data)
        nQ <- nrow(Q)
        N <- nQ + 1

        lcmc <- diag(apply(apply(Q, 2, cumsum), 1, cumsum)) /
            seq_len(nQ) / N -
            seq_len(nQ) / nQ
        lcmc
    }
)

rnx2qnx <- function(rnx, K = seq_along(rnx), N = length(rnx) + 1) {
    (rnx * (N - 1 - K) + K) / (N - 1)
}
qnx2rnx <- function(qnx, K = seq_along(qnx), N = length(qnx) + 1) {
    ((N - 1) * qnx - K) / (N - 1 - K)
}

#' @export
setGeneric(
  "reconstruction_error",
  function(object, ...) standardGeneric("reconstruction_error"),
  valueClass = "numeric"
)

#' Method reconstruction_error
#'
#' Calculate the error using only the first \code{n} dimensions of the embedded
#' data. \code{error_fun} can either be one of \code{c("rmse", "mae")} to
#' calculate the root mean square error or the mean absolute error respectively,
#' or a function that takes to equally sized vectors as input and returns a
#' single number as output.
#'
#' @param object of class dimRedResult
#' @param n a positive integer or vector of integers \code{<= ndims(object)}
#' @param error_fun a function or string indicating an error function, if
#'   indication a function it must take to matrices of the same size and return
#'   a scalar.
#' @return a vector of number with the same length as \code{n} with the
#'
#' @examples
#' \dontrun{
#' ir <- loadDataSet("Iris")
#' ir.drr <- embed(ir, "DRR", ndim = ndims(ir))
#' ir.pca <- embed(ir, "PCA", ndim = ndims(ir))
#'
#' rmse <- data.frame(
#'   rmse_drr = reconstruction_error(ir.drr),
#'   rmse_pca = reconstruction_error(ir.pca)
#' )
#'
#' matplot(rmse, type = "l")
#' plot(ir)
#' plot(ir.drr)
#' plot(ir.pca)
#' }
#' @author Guido Kraemer
#' @family Quality scores for dimensionality reduction
#' @aliases reconstruction_error
#' @export
setMethod(
  "reconstruction_error",
  c("dimRedResult"),
  function (object, n = seq_len(ndims(object)), error_fun = "rmse") {
    if (any(n > ndims(object))) stop("n > ndims(object)")
    if (any(n < 1))             stop("n < 1")

    ef <- if (inherits(error_fun, "character")) {
      switch(
        error_fun,
        rmse = rmse,
        mae  = mae
      )
    } else if (inherits(error_fun, "function")) {
      error_fun
    } else {
      stop("error_fun must be a string or function, see documentation for details")
    }

    res <- numeric(length(n))
    org <- getData(getOrgData(object))
    for (i in seq_along(n)) {
      rec <- getData(inverse(
        object, getData(getDimRedData(object))[, seq_len(n[i]), drop = FALSE]
      ))
      res[i] <- ef(org, rec)
    }
    res
  }
)

rmse <- function (x1, x2) sqrt(mean((x1 - x2) ^ 2))
mae  <- function (x1, x2) mean(abs(x1 - x2))

test_that("high level functions working?", {
    embed_methods <- dimRedMethodList(filter = TRUE)
    quality_methods <- dimRedQualityList(filter = TRUE)
    scurve <- loadDataSet("3D S Curve", n = 300)
    for(i in 1:ncol(scurve@data)){
      scurve@data[, i] <- scurve@data[, i] - min(scurve@data[, i])
    }

    quality_results <- matrix(NA, length(embed_methods),
                              length(quality_methods),
                              dimnames = list(embed_methods, quality_methods))
    embedded_data <- list()

    for (e in embed_methods) {
        message("embedding: ", e)

        if (  (e != "AutoEncoder" || reticulate::py_module_available("tensorflow")) &&
              (e != "UMAP"        || reticulate::py_module_available("umap-learn"))    ) {

          suppressWarnings(
            embedded_data[[e]] <- embed(
              scurve, e,
              .mute = c("message", "output")))

          for (q in quality_methods) {
            message("  quality: ", q)
            quality_results[e, q] <- tryCatch(
              suppressWarnings(quality(embedded_data[[e]], q,
                                       .mute = c("message", "output"))),
              error = function (e) NA
            )
          }

        }
    }

    lapply(embedded_data, function(x) expect_equal(2, getNDim(x)))

    expect(inherits(quality_results, "matrix"), "should be matrix")
    expect(storage.mode(quality_results) == "double",
           'storage should be "double"')
})

#' Umap embedding
#'
#' An S4 Class implementing the UMAP algorithm
#'
#' Uniform Manifold Approximation is a gradient descend based algorithm that
#' gives results similar to t-SNE, but scales better with the number of points.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#'
#' UMAP can take the follwing parameters:
#' \describe{
#'   \item{ndim}{The number of embedding dimensions.}
#'   \item{knn}{The number of neighbors to be used.}
#'   \item{d}{The distance metric to use.}
#'   \item{method}{\code{"naive"} for an R implementation, \code{"python"}
#'     for the reference implementation.}
#' }
#'
#' Other method parameters can also be passed, see
#' \code{\link[umap]{umap.defaults}} for details. The ones above have been
#' standardized for the use with \code{dimRed} and will get automatically
#' translated for \code{\link[umap]{umap}}.
#'
#' @section Implementation:
#'
#' The dimRed package wraps the \code{\link[umap]{umap}} packages which provides
#' an implementation in pure R and also a wrapper around the original python
#' package \code{umap-learn} (https://github.com/lmcinnes/umap/). This requires
#' \code{umap-learn} version 0.4 installed, at the time of writing, there is
#' already \code{umap-learn} 0.5 but it is not supported by the R package
#' \code{\link[umap]{umap}}.
#'
#' The \code{"naive"} implementation is a pure R implementation and considered
#' experimental at the point of writing this, it is also much slower than the
#' python implementation.
#'
#' The \code{"python"} implementation is the reference implementation used by
#' McInees et. al. (2018). It requires the \code{\link[reticulate]{reticulate}}
#' package for the interaction with python and the python package
#' \code{umap-learn} installed (use \code{pip install umap-learn}).
#'
#' @references
#'
#' McInnes, Leland, and John Healy.
#' "UMAP: Uniform Manifold Approximation and Projection for Dimension Reduction."
#' https://arxiv.org/abs/1802.03426
#'
#' @examples
#' \dontrun{
#' dat <- loadDataSet("3D S Curve", n = 300)
#' emb <- embed(dat, "UMAP", .mute = NULL, knn = 10)
#' plot(emb, type = "2vars")
#' }
#'
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export UMAP
#' @exportClass UMAP
UMAP <- setClass(
  "UMAP",
  contains = "dimRedMethod",
  prototype = list(
    stdpars = list(
      knn = 15,
      ndim = 2,
      d = "euclidean",
      method = "umap-learn"
    ),
    fun = function (data, pars,
                    keep.org.data = TRUE) {
      chckpkg("umap")
      if (pars$method == "python") {
        chckpkg("reticulate")
        if (!reticulate::py_module_available("umap"))
          stop("cannot find python umap, install with `pip install umap-learn`")
      }

      meta <- data@meta
      orgdata <- if (keep.org.data) data@data else NULL
      indata <- data@data

      ## Create config
      umap_call_pars <- umap::umap.defaults
      umap_call_pars$n_neighbors  <- pars$knn
      umap_call_pars$n_components <- pars$ndim
      umap_call_pars$metric       <- pars$d
      umap_call_pars$method <- pars$method
      umap_call_pars$d      <- indata

      pars_2 <- pars
      pars_2$knn <- NULL
      pars_2$ndim <- NULL
      pars_2$d <- NULL
      pars_2$method <- NULL

      for (n in names(pars_2))
        umap_call_pars[[n]] <- pars_2[[n]]

      ## Do the embedding
      outdata <- do.call(umap::umap, umap_call_pars)

      ## Post processing
      colnames(outdata$layout) <- paste0("UMAP", 1:ncol(outdata$layout))

      appl <- function(x) {
        appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
        proj <- if (inherits(x, "dimRedData")) x@data else x

        if (ncol(proj) != ncol(orgdata))
          stop("x must have the same number of dimensions ",
               "as the original data")

        new_proj <- umap:::predict.umap(outdata, as.matrix(proj))

        colnames(new_proj) <- paste0("UMAP", 1:ncol(new_proj))
        rownames(new_proj) <- NULL

        out_data <- new("dimRedData", data = new_proj, meta = appl.meta)
        return(out_data)
      }

      return(new(
        "dimRedResult",
        data = new("dimRedData",
                   data = outdata$layout,
                   meta = meta),
        org.data = orgdata,
        apply        = appl,
        has.org.data = keep.org.data,
        has.apply    = TRUE,
        method = "UMAP",
        pars = pars
      ))
    },
    requires = c("umap", "reticulate")
  )
)

#' @include misc.R
#' @include dimRedData-class.R
NULL

#' Class "dimRedResult"
#'
#' A class to hold the results of of a dimensionality reduction.
#'
#' @slot data Output data of class dimRedData.
#' @slot org.data original data, a matrix.
#' @slot apply a function to apply the method to out-of-sampledata,
#'     may not exist.
#' @slot inverse a function to calculate the original coordinates from
#'     reduced space, may not exist.
#' @slot has.org.data logical, if the original data is included in the object.
#' @slot has.apply logical, if a forward method is exists.
#' @slot has.inverse logical if an inverse method exists.
#' @slot method saves the method used.
#' @slot pars saves the parameters used.
#' @slot other.data other data produced by the method, e.g. a distance matrix.
#'
#' @examples
#' ## Create object by embedding data
#' iris.pca <- embed(loadDataSet("Iris"), "PCA")
#'
#' ## Convert the result to a data.frame
#' head(as(iris.pca, "data.frame"))
#' head(as.data.frame(iris.pca))
#'
#' ## There are no nameclashes to avoid here:
#' head(as.data.frame(iris.pca,
#'                    org.data.prefix = "",
#'                    meta.prefix     = "",
#'                    data.prefix     = ""))
#'
#' ## Print it more or less nicely:
#' print(iris.pca)
#'
#' ## Get the embedded data as a dimRedData object:
#' getDimRedData(iris.pca)
#'
#' ## Get the original data including meta information:
#' getOrgData(iris.pca)
#'
#' @family dimRedResult
#' @export dimRedResult
#' @exportClass dimRedResult
dimRedResult <- setClass(
    "dimRedResult",
    slots = c(
        data         = "dimRedData",
        org.data     = "matrix",
        apply        = "function",
        inverse      = "function",
        has.org.data = "logical",
        has.apply    = "logical",
        has.inverse  = "logical",
        method       = "character",
        pars         = "list",
        other.data   = "list"
    ),
    prototype = list(
        data         = new("dimRedData"),
        org.data     = matrix(numeric(0), 0, 0),
        apply        = function(x) NA,
        inverse      = function(x) NA,
        has.org.data = FALSE,
        has.apply    = FALSE,
        has.inverse  = FALSE,
        method       = "",
        pars         = list(),
        other.data   = list()
    )
)

setAs(
    from = "dimRedResult",
    to = "data.frame",
    def = function(from){
        if (from@has.org.data) {
            org.data <- from@org.data
            names(org.data) <- paste("org", names(org.data), sep = ".")
            cbind(as(from@data, "data.frame"), as.data.frame(org.data))
        } else {
            as(from@data, "data.frame")
        }
    }
)

#' @importFrom stats predict
#' @export
setGeneric(
    "predict", function(object, ...) standardGeneric("predict"),
    useAsDefault = stats::predict
)

#' @describeIn dimRedResult apply a trained method to new data, does not work
#'     with all methods, will give an error if there is no \code{apply}.
#'     In some cases the apply function may only be an approximation.
#' @param xnew new data, of type \code{\link{dimRedData}}
#'
#' @export
setMethod(f = "predict",
          signature = "dimRedResult",
          definition = function(object, xnew) {
    if (object@has.apply) object@apply(xnew)
    else                  stop("object does not have an apply function")
})

#' @export
setGeneric(
    "inverse",
    function(object, ...) standardGeneric("inverse")
)

#' @describeIn dimRedResult inverse transformation of embedded data, does not
#'     work with all methods, will give an error if there is no \code{inverse}.
#'     In some cases the apply function may only be an approximation.
#' @param ynew embedded data, of type \code{\link{dimRedData}}
#'
#' @aliases inverse
#' @export
setMethod(f = "inverse",
          signature = c("dimRedResult"),
          definition = function(object, ynew) {
    if (object@has.inverse) object@inverse(ynew)
    else                    stop("object does not have an inverse function")
})


#' @param x Of class \code{dimRedResult}
#' @param org.data.prefix Prefix for the columns of the org.data slot.
#' @param meta.prefix Prefix for the columns of \code{x@@data@@meta}.
#' @param data.prefix Prefix for the columns of \code{x@@data@@data}.
#'
#' @describeIn dimRedResult convert to \code{data.frame}
#' @export
setMethod(f = "as.data.frame",
          signature = c("dimRedResult"),
          definition = function(x, org.data.prefix = "org.",
                                meta.prefix = "meta.",
                                data.prefix = "") {
    tmp <- list()

    if (nrow(x@data@meta) > 0){
        tmp$meta <- as.data.frame(x@data@meta)
        names(tmp$meta) <- paste0(meta.prefix,
                                  colnames(x@data@meta))
    }
    tmp$data <- as.data.frame(x@data@data)
    names(tmp$data) <- paste0(data.prefix,   colnames(x@data@data))
    if (x@has.org.data){
        tmp$org.data <- as.data.frame(x@org.data)
        names(tmp$org.data) <- paste0(org.data.prefix, colnames(x@org.data))
    }
    names(tmp) <- NULL
    data.frame(tmp, stringsAsFactors = FALSE)
})



#' @param object Of class \code{dimRedResult}
#' @describeIn dimRedResult Get the parameters with which the method
#'     was called.
#' @export
setMethod(
    f = "getPars",
    signature = "dimRedResult",
    definition = function (object) {
        object@pars
    }
)

#' @describeIn dimRedResult Get the number of embedding dimensions.
#' @export
setMethod(
    f = "getNDim",
    signature = "dimRedResult",
    definition = function (object) {
        result <- getPars(object)$ndim
        if(is.null(result)) dim(object@data@data)[2] else result
    }
)

#' @describeIn dimRedResult Method for printing.
#' @import utils
#' @export
setMethod(
    f = "print",
    signature = "dimRedResult",
    definition = function(x) {
        cat("Method:\n")
        cat(x@method, "\n")
        cat("Parameters:\n")
        utils::str(x@pars)
    }
)

#' @describeIn dimRedResult Get the original data and meta.data
#' @export
setMethod(
    f = "getOrgData",
    signature = "dimRedResult",
    definition = function(object) {
        return(new("dimRedData",
                   data = object@org.data,
                   meta = object@data@meta))
    }
)

#' @describeIn dimRedResult Get the embedded data
#' @export
setMethod(
    f = "getDimRedData",
    signature = "dimRedResult",
    definition = function(object) {
        return(object@data)
    }
)

#' @describeIn dimRedResult Extract the number of embedding dimensions.
#'
#' @examples
#' ## Get the number of variables:
#' ndims(iris.pca)
#'
#' @export
setMethod(
    "ndims",
    "dimRedResult",
    function(object) ncol(object@data@data)
)

#' @describeIn dimRedResult Get other data produced by the method
#' @export
setMethod(
    f = "getOtherData",
    signature = "dimRedResult",
    definition = function(object) object@other.data
)

test_that("general data conversions", {
    irisData <- as(iris[, 1:4], "dimRedData")
    expect_equal(class(irisData)[1], "dimRedData")

    irisParsCS <- list(center = TRUE,  scale. = TRUE)
    irisParsC  <- list(center = TRUE,  scale. = FALSE)
    irisParsS  <- list(center = FALSE, scale. = TRUE)
    irisPars   <- list(center = FALSE, scale. = FALSE)

    irisResCS <- do.call(function(...) embed(irisData, "PCA", ...), irisParsCS)
    irisResS  <- do.call(function(...) embed(irisData, "PCA", ...), irisParsS)
    irisResC  <- do.call(function(...) embed(irisData, "PCA", ...), irisParsC)
    irisRes   <- do.call(function(...) embed(irisData, "PCA", ...), irisPars)

    expect_equal(2, getNDim(irisResCS))
    expect_equal(2, getNDim(irisResS))
    expect_equal(2, getNDim(irisResC))
    expect_equal(2, getNDim(irisRes))

    expect_equal(class(irisResCS)[1], "dimRedResult")
    expect_equal(class(irisResS)[1],  "dimRedResult")
    expect_equal(class(irisResC)[1],  "dimRedResult")
    expect_equal(class(irisRes)[1],   "dimRedResult")

    expect_equal(irisResCS@apply(irisData), irisResCS@data)
    expect_equal(irisResS@apply(irisData),  irisResS@data)
    expect_equal(irisResC@apply(irisData),  irisResC@data)
    expect_equal(irisRes@apply(irisData),   irisRes@data)

    expect(sqrt(mean(
            (irisResCS@inverse(irisResCS@data)@data - irisData@data) ^ 2
        )) < 0.3,
        "error too large"
    )
    expect(sqrt(mean(
            (irisResS@inverse(irisResS@data)@data   - irisData@data) ^ 2
        )) < 0.3,
        "error too large"
    )
    expect(sqrt(mean(
            (irisResC@inverse(irisResC@data)@data   - irisData@data) ^ 2
        )) < 0.3,
        "error too large"
    )
    expect(sqrt(mean(
            (irisRes@inverse(irisRes@data)@data     - irisData@data) ^ 2
        )) < 0.3,
        "error too large"
    )

    scale2 <- function(x, center, scale.) scale(x, center, scale.)
    expect_equal(
      do.call(function(...) scale2(iris[1:4], ...) %*% getRotationMatrix(irisResCS), irisParsCS),
      getData( getDimRedData(irisResCS) )
    )
    expect_equal(
      do.call(function(...) scale2(iris[1:4], ...) %*% getRotationMatrix(irisResS), irisParsS),
      getData( getDimRedData(irisResS) )
    )
    expect_equal(
      do.call(function(...) scale2(iris[1:4], ...) %*% getRotationMatrix(irisResC), irisParsC),
      getData( getDimRedData(irisResC) )
    )
    expect_equal(
      do.call(function(...) scale2(iris[1:4], ...) %*% getRotationMatrix(irisRes), irisPars),
      getData( getDimRedData(irisRes) )
    )
})

test_that("drr forward and backward passes", {
    spiral <- loadDataSet("Helix", n = 200)

    drr_spiral <- embed(spiral, "DRR", ndim = 3, .mute = c("message", "output"))

    expect_equal(3, getNDim(drr_spiral))
    dsa <- drr_spiral@apply(spiral)
    dsi <- drr_spiral@inverse(dsa)

    expect_equal(dsi, spiral)
})

context("FastICA")

test_that("general data conversions", {
    irisData <- as(iris[, 1:4], "dimRedData")
    expect_equal(class(irisData)[1], "dimRedData")

    irisRes <- embed(irisData, "FastICA")
    expect_equal(class(irisRes)[1],   "dimRedResult")

    expect_equal(2, getNDim(irisRes))

    expect_equal(irisRes@apply(irisData), irisRes@data)

    expect(sqrt(mean(
            (irisRes@inverse(irisRes@data)@data - irisData@data) ^ 2
        )) < 0.3,
        "error too large"
    )

    expect_equal(
      scale(iris[1:4], TRUE, FALSE) %*% getRotationMatrix(irisRes),
      unname(as.matrix(getData( getDimRedData(irisRes) )) )
    )
})

test_that("general data conversions", {
    skip_if_not_installed("pcaL1")
    irisData <- as(iris[, 1:4], "dimRedData")
    expect_equal(class(irisData)[1], "dimRedData")

    irisParsCS <- list(center = TRUE,  .mute = c("message", "output"), ndim = 4, scale. = TRUE,  projections = "l1", fun = "l1pca")
    irisParsC  <- list(center = TRUE,  .mute = c("message", "output"), ndim = 4, scale. = FALSE, projections = "l1", fun = "l1pca")
    irisParsS  <- list(center = TRUE,  .mute = c("message", "output"), ndim = 4, scale. = TRUE,  projections = "l1", fun = "l1pcahp")
    irisPars   <- list(center = FALSE, .mute = c("message", "output"), ndim = 4, scale. = FALSE, projections = "l1", fun = "l1pcastar")

    irisResCS <- do.call(function(...) embed(irisData, "PCA_L1", ...), irisParsCS)
    irisResS  <- do.call(function(...) embed(irisData, "PCA_L1", ...), irisParsS)
    irisResC  <- do.call(function(...) embed(irisData, "PCA_L1", ...), irisParsC)
    irisRes   <- do.call(function(...) embed(irisData, "PCA_L1", ...), irisPars)

    expect_equal(4, getNDim(irisResCS))
    expect_equal(4, getNDim(irisResS))
    expect_equal(4, getNDim(irisResC))
    expect_equal(4, getNDim(irisRes))

    expect_equal(class(irisResCS)[1], "dimRedResult")
    expect_equal(class(irisResS)[1],  "dimRedResult")
    expect_equal(class(irisResC)[1],  "dimRedResult")
    expect_equal(class(irisRes)[1],   "dimRedResult")

    expect_equal(irisResCS@apply(irisData), irisResCS@data)
    expect_equal(irisResS@apply(irisData),  irisResS@data)
    expect_equal(irisResC@apply(irisData),  irisResC@data)
    expect_equal(irisRes@apply(irisData),   irisRes@data)

    expect(sqrt(mean(
            (irisResCS@inverse(irisResCS@data)@data - irisData@data) ^ 2
        )) < 0.3,
        "error too large"
    )
    expect(sqrt(mean(
            (irisResS@inverse(irisResS@data)@data   - irisData@data) ^ 2
        )) < 0.3,
        "error too large"
    )
    expect(sqrt(mean(
            (irisResC@inverse(irisResC@data)@data   - irisData@data) ^ 2
        )) < 0.3,
        "error too large"
    )
    expect(sqrt(mean(
            (irisRes@inverse(irisRes@data)@data     - irisData@data) ^ 2
        )) < 0.3,
        "error too large"
    )

    scale2 <- function(x, center, scale.) scale(x, center, scale.)
    expect_equal(
      do.call(function(...) scale2(iris[1:4], ...) %*% getRotationMatrix(irisResCS),
              irisParsCS[c("center", "scale.")]),
      getData( getDimRedData(irisResCS) ),
      tolerance = 1e-2
    )

    expect_equal(
      do.call(function(...) scale2(iris[1:4], ...) %*% getRotationMatrix(irisResS),
              irisParsS[c("center", "scale.")]),
      getData( getDimRedData(irisResS) ),
      tolerance = 1e-2
    )

    expect_equal(
      do.call(function(...) scale2(iris[1:4], ...) %*% getRotationMatrix(irisResC),
              irisParsC[c("center", "scale.")]),
      getData( getDimRedData(irisResC) ),
      tolerance = 1e-2
    )
    expect_equal(
      do.call(function(...) scale2(iris[1:4], ...) %*% getRotationMatrix(irisRes),
              irisPars[c("center", "scale.")]),
      getData( getDimRedData(irisRes) ),
      tolerance = 1e-2
    )

    expect_s4_class({ embed(iris[1:4], "PCA_L1", ndim = 1,
                            .mute = c("message", "output")) },
                    "dimRedResult")
})

test_that("general data conversions", {
  if(!requireNamespace("FastICA", quietly = TRUE))
    skip("FastICA not available")
  irisData <- as(iris[, 1:4], "dimRedData")
  expect_equal(class(irisData)[1], "dimRedData")

  irisRes <- embed(irisData, "FastICA")
  expect_equal(class(irisRes)[1],   "dimRedResult")

  expect_equal(2, getNDim(irisRes))

  expect_equal(irisRes@apply(irisData), irisRes@data)

  expect(
    sqrt(
      mean(
      (irisRes@inverse(irisRes@data)@data - irisData@data) ^ 2
      )
    ) < 0.3,
    "error too large"
  )

  expect_equal(
    scale(iris[1:4], TRUE, FALSE) %*% getRotationMatrix(irisRes),
    unname(as.matrix(getData( getDimRedData(irisRes) )) )
  )
})

skip_if_no_NMF <- function() {
  if (!requireNamespace("NMF", quietly = TRUE) &&
      Sys.getenv("BNET_FORCE_NNMF_TESTS") != "1")
    skip("NMF not available for testing")
}

## if we don't load the library explicitly, the predict function does not work
## (sometimes...).
## library(NMF)

ints_trn <- matrix(seq(0, 98, by = 2), ncol = 5)
input_trn <- dimRedData(as.data.frame(ints_trn))
input_tst <- dimRedData(ints_trn[1:3,] + 1)

test_that("2D projection", {
  skip_if_no_NMF()

  dim_2_defaults <- embed(input_trn, "NNMF", seed = 13, nrun = 1)

  expect_equal(dim_2_defaults@method, "NNMF")

  ## Expected results from
  ## tmp <- NMF::nmf(t(ints_trn), rank = 2, nrun = 1, seed = 13)
  ## coefs <- basis(tmp)
  ## rownames(coefs) <- paste0("V", 1:5)
  ## colnames(coefs) <- paste0("NNMF", 1:2)
  ## coefs
  ## dput(coefs)

  dim_2_coef <- structure(
    c(18.807241710186, 30.2191667888959,
      32.1069052462692, 9.53490906878683,
      164.109205703974, 0.00064246562138093,
      24.3924277525021, 56.4301459918642,
      108.103923297376, 17.566220349863),
    .Dim = c(5L, 2L),
    .Dimnames = list(c("V1", "V2", "V3", "V4", "V5"),
                     c("NNMF1", "NNMF2")))

  expect_equal(dim_2_defaults@other.data$w, dim_2_coef, ignore_attr = TRUE)

  dim_2_apply <- dim_2_defaults@apply(input_tst)@data
  dim_2_pred <- predict(dim_2_defaults, input_tst)@data

  ## Expected results from
  ## t(solve(crossprod(basis(tmp)), t(input_tst@data %*% basis(tmp))))
  ## preds <- getData(input_tst) %*% t(MASS::ginv(basis(tmp)))
  ## getData(getDimRedData(dim_2_defaults))
  ## colnames(preds) <- paste0("NNMF", 1:2)
  ## dput(preds)

  dim_2_exp <- structure(
    c(0.427476458116875, 0.440237021147746, 0.452997584178617,
      0.512256378881175, 0.5332094651398, 0.554162551398426),
    .Dim = c(3L, 2L),
    .Dimnames = list(NULL, c("NNMF1", "NNMF2"))
  )

  expect_equal(dim_2_apply, dim_2_exp, tolerance = 0.01, ignore_attr = TRUE)
  expect_equal(dim_2_pred,  dim_2_exp, tolerance = 0.01, ignore_attr = TRUE)
})

test_that("other arguments", {
  skip_if_no_NMF()

  dim_3_args <- embed(input_trn, "NNMF", seed = 13, nrun = 10,
                      ndim = 3, method = "KL",
                      options = list(.pbackend = NULL))

  ## Expected results from
  ## tmp <- NMF::nmf(t(ints_trn), rank = 3, nrun = 10, seed = 13,
  ##                 method = "KL", .pbackend = NULL)
  ## coefs <- t(NMF::coef(tmp))
  ## colnames(coefs) <- paste0("NNMF", 1:ncol(coefs))
  ## coefs
  ## dput(coefs)
  ## rot <- NMF::basis(tmp)
  ## rownames(rot) <- paste0("V", 1:nrow(rot))
  ## dput(rot)

  dim_3_rot <- structure(
    c(11.624951277152, 31.2554213278975, 50.8858913786408,
      70.5163614293837, 90.1468314801264, 2.22044604925031e-16,
      36.4357899711133, 72.8715799422292, 109.307369913346,
      145.743159884462, 22.4019808842378, 42.1081005773292,
      61.8142202704197, 81.52033996351, 101.2264596566),
    .Dim = c(5L, 3L),
    .Dimnames = list(c("V1", "V2", "V3", "V4", "V5"), NULL)
  )
  dim_3_pred <- structure(
    c(2.22044604925031e-16, 0.0731742704517501, 0.194863499580201,
      0.50224638618713, 0.557517908619563, 0.197219538171418,
      0.0860784848917408, 0.159094934700865, 0.10366866301249,
      0.216483929440989, 0.54891083782883, 0.481738298195276, 0.40204352636632,
      0.274419226004639, 0.211867578024856, 0.256578985276104,
      0.236980211423017, 0.16984840699324, 0.135869049278152,
      0.0584647425861749, 2.22044604925031e-16, 0.0513058500137363,
      0.0774360678481537, 0.00720517673339281, 0.0678012129377125,
      0.344046917890136, 0.49099862480747, 0.542386371921862, 0.660426277478513,
      0.691161417731563),
    .Dim = c(10L, 3L),
    .Dimnames = list(NULL, c("NNMF1", "NNMF2", "NNMF3"))
  )

  expect_equal(dim_3_args@other.data$w, dim_3_rot, ignore_attr = TRUE)
  expect_equal(getData(getDimRedData(dim_3_args)), dim_3_pred, ignore_attr = TRUE)

  dim_3_apply <- dim_3_args@apply(input_tst)@data
  dim_3_pred <- predict(dim_3_args, input_tst)@data

  ## Expected results from
  ## crossprod(basis(tmp)) does not have full rank!!! This needs to be considered
  ## w <- getOtherData(dim_3_args)$w
  ## preds <- t(solve(crossprod(w), t(input_trn@data %*% w)))
  ## preds <- t(qr.solve(crossprod(w), t(input_trn@data %*% w)))
  ## preds <- getData(input_tst) %*% t(MASS::ginv(w))
  ## preds
  ## dput(preds)
  ## getData(getDimRedData(dim_3_args))
  ## preds - getData(getDimRedData(dim_3_args))
  ## input_trn@data
  ## input_tst@data %*% basis(tmp)
  ## colnames(preds) <- paste0("NNMF", 1:3)

  dim_3_exp <- structure(
    c(0.118730450278164, 0.144080695556738, 0.169430940835312,
      0.494122495652466, 0.439293850852014, 0.384465206051563,
      -0.0169733070286198, 0.0591496323928872, 0.135272571814394),
    .Dim = c(3L, 3L)
  )

  expect_equal(dim_3_apply, dim_3_exp, tolerance = 0.01, ignore_attr = TRUE)
  expect_equal(dim_3_pred,  dim_3_exp, tolerance = 0.01, ignore_attr = TRUE)
})


test_that("Bad args", {
  skip_if_no_NMF()

  expect_error(embed(iris, "NNMF"))
  expect_error(embed(iris[, 1], "NNMF"),
               "`ndim` should be less than the number of columns")
  expect_error(embed(iris[1:4], "NNMF", method = c("a", "b")),
               "only supply one `method`")
  expect_error(embed(scale(iris[1:4]), "NNMF"), "negative entries")

})


test_that("Full_rank", {
  skip_if_no_NMF()

  dim_2_full_rank_example <- embed(input_trn, "NNMF", ndim = ncol(input_trn@data))
  dim_2_recon  <- inverse(dim_2_full_rank_example, dim_2_full_rank_example@data@data)

  expect_equal(dim_2_recon, input_trn, ignore_attr = TRUE, tolerance = 1e-2)
})

test_that("predict/inverse methods", {
  dat <- loadDataSet("Iris")
  emb <- embed(dat, "PCA", ndim = 4)
  pred <- predict(emb, dat)
  inv <- inverse(emb, pred)
  expect_equal(getDimRedData(emb), pred)
  expect_equal(dat, inv)

  if(requireNamespace("Rtsne", quietly = TRUE)) {
    emb2 <- embed(dat, "tSNE")
    expect_error(predict(emb2, dat))
    expect_error(inverse(emb2, dat))
  }
})

test_that("conversion", {
  iris_data_frame_as <- as(embed(loadDataSet("Iris"), "PCA"), "data.frame")
  expect_equal(colnames(iris_data_frame_as), c("meta.Species", "PC1", "PC2", colnames(iris)[-5]))
})

test_that("general data conversions", {
  data(iris)

    irisData <- loadDataSet("Iris")
    expect_equal(class(irisData)[1], "dimRedData")

    irisPars <- list()
    irisPars[[length(irisPars) + 1]] <-
        list(kernel = "rbfdot",
             kpar = list(sigma = 0.1))
    irisPars[[length(irisPars) + 1]] <-
        list(kernel = "rbfdot",
             kpar = list(sigma = 1))
    irisPars[[length(irisPars) + 1]] <-
        list(kernel = "polydot",
             kpar = list(degree = 3))
    irisPars[[length(irisPars) + 1]] <-
        list(kernel = "vanilladot",
             kpar = list())
    irisPars[[length(irisPars) + 1]] <-
        list(kernel = "laplacedot",
             kpar = list(sigma = 1))
    irisPars[[length(irisPars) + 1]] <-
        list(kernel = "laplacedot",
             kpar = list(sigma = 0.1))
    irisPars[[length(irisPars) + 1]] <-
        list(kernel = "besseldot",
             kpar = list(sigma = 0.1,
                         order = 1,
                         degree = 1))
    irisPars[[length(irisPars) + 1]] <-
        list(kernel = "besseldot",
             kpar = list(sigma = 1,
                         order = 2,
                         degree = 3))
    irisPars[[length(irisPars) + 1]] <-
        list(kernel = "splinedot",
             kpar = list())

    irisRes <- lapply(irisPars, function(x)
        do.call(
          function(...) tryCatch(embed(.data = irisData,
                                       .method = "kPCA", ...),
                                   error = function(e) as.character(e)),
            x
        ) )

    for (i in 1:length(irisRes)) {
        if (inherits(irisRes[[i]], "character")){
            expect(grepl("singular", irisRes[[i]]),
                   "singular")
        } else {
            expect(inherits(irisRes[[i]], "dimRedResult"),
                   'should be of class "dimRedResult"')
        }
    }

    ## This test fails with multithreaded blas
    ## for (i in 1:length(irisRes)){
    ##     if (inherits(irisRes[[i]], "dimRedResult")){
    ##       expect_equal(irisRes[[i]]@apply(irisData)@data[, 1:2],
    ##                    irisRes[[i]]@data@data)

    ##       expect_equal(2, getNDim(irisRes[[i]]))

    ##         ## the reverse is an approximate:

    ##       expect_less_than(
    ##           max(
    ##               irisRes[[i]]@inverse(irisRes[[i]]@data)@data - irisData@data
    ##           ), 300,
    ##           ## paste0("inverse of kpca is an approximate, ",
    ##           ##        "so this may fail due to numerical inaccuracy")
    ##         )
    ##     }
    ## }

    ## This one cannot calculate an inverse:
    kpca.fit <- embed(loadDataSet("3D S", n = 200),
                      "kPCA", kernel = "splinedot", kpar = list())
    expect( is.na(kpca.fit@inverse(1)), "The inverse should return NA" )
})

#' Graph Embedding via the Kamada Kawai Algorithm
#'
#' An S4 Class implementing the Kamada Kawai Algorithm for graph embedding.
#'
#' Graph embedding algorithms se the data as a graph. Between the
#' nodes of the graph exist attracting and repelling forces which can
#' be modeled as electrical fields or springs connecting the
#' nodes. The graph is then forced into a lower dimensional
#' representation that tries to represent the forces betweent he nodes
#' accurately by minimizing the total energy of the attracting and
#' repelling forces.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' KamadaKawai can take the following parameters:
#' \describe{
#'   \item{ndim}{The number of dimensions, defaults to 2. Can only be 2 or 3}
#'   \item{knn}{Reduce the graph to keep only the neares neighbors. Defaults to 100.}
#'   \item{d}{The distance function to determine the weights of the graph edges. Defaults to euclidean distances.}
#' }
#'
#' @section Implementation:
#' Wraps around \code{\link[igraph]{layout_with_kk}}. The parameters
#' maxiter, epsilon and kkconst are set to the default values and
#' cannot be set, this may change in a future release. The DimRed
#' Package adds an extra sparsity parameter by constructing a knn
#' graph which also may improve visualization quality.
#'
#' @references
#'
#' Kamada, T., Kawai, S., 1989. An algorithm for drawing general undirected
#' graphs. Information Processing Letters 31, 7-15.
#' https://doi.org/10.1016/0020-0190(89)90102-6
#'
#' @examples
#' if(requireNamespace(c("igraph", "coRanking"), quietly = TRUE)) {
#'
#' dat <- loadDataSet("Swiss Roll", n = 200)
#' emb <- embed(dat, "KamadaKawai")
#' plot(emb, type = "2vars")
#'
#' }
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export KamadaKawai
#' @exportClass KamadaKawai
KamadaKawai <- setClass(
    "KamadaKawai",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(ndim         = 2,
                       knn          = 100,
                       d            = stats::dist),
        fun = function (data, pars,
                        keep.org.data = TRUE) {
        chckpkg("igraph")

        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        indata <- data@data

        outdata <- em_graph_layout(
            indata,
            graph_em_method = igraph::layout_with_kk,
            knn             = pars$knn,
            d               = pars$d,
            ndim            = pars$ndim,
            weight.trans    = I #pars$weight.trans
        )

        colnames(outdata) <- paste0("KK", 1:ncol(outdata))

        return(new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = outdata,
                               meta = meta),
            org.data     = orgdata,
            has.org.data = keep.org.data,
            method       = "graph_kk",
            pars         = pars
        ))
        },
      requires = c("igraph", "coRanking"))
)


#' Distributed Recursive Graph Layout
#'
#' An S4 Class implementing Distributed recursive Graph Layout.
#'
#' DrL uses a complex algorithm to avoid local minima in the graph
#' embedding which uses several steps.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' DrL can take the following parameters:
#' \describe{
#'   \item{ndim}{The number of dimensions, defaults to 2. Can only be 2 or 3}
#'   \item{knn}{Reduce the graph to keep only the neares neighbors. Defaults to 100.}
#'   \item{d}{The distance function to determine the weights of the graph edges. Defaults to euclidean distances.}
#' }
#'
#' @section Implementation:
#' Wraps around \code{\link[igraph]{layout_with_drl}}. The parameters
#' maxiter, epsilon and kkconst are set to the default values and
#' cannot be set, this may change in a future release. The DimRed
#' Package adds an extra sparsity parameter by constructing a knn
#' graph which also may improve visualization quality.
#'
#' @references
#'
#' Martin, S., Brown, W.M., Wylie, B.N., 2007. Dr.l: Distributed Recursive
#' (graph) Layout (No. dRl; 002182MLTPL00). Sandia National Laboratories.
#'
#' @examples
#' \dontrun{
#' if(requireNamespace(c("igraph", "coRanking"), quietly = TRUE)) {
#'
#' dat <- loadDataSet("Swiss Roll", n = 200)
#' emb <- embed(dat, "DrL")
#' plot(emb, type = "2vars")
#' }
#'
#' }
#'
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export DrL
#' @exportClass DrL
DrL <- setClass(
    "DrL",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(ndim         = 2,
                       knn          = 100,
                       d            = stats::dist),
        fun = function (data, pars,
                        keep.org.data = TRUE) {
        chckpkg("igraph")

        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        indata <- data@data

        outdata <- em_graph_layout(
            indata,
            graph_em_method = igraph::layout_with_drl,
            knn = pars$knn,
            d = pars$d,
            ndim = pars$ndim,
            weight.trans = I #pars$weight.trans
        )

        colnames(outdata) <- paste0("DrL", 1:ncol(outdata))

        return(new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = outdata,
                               meta = meta),
            org.data     = orgdata,
            has.org.data = keep.org.data,
            method       = "graph_drl",
            pars         = pars
        ))
        },
        requires = c("igraph", "coRanking")
    )
)

#' Fruchterman Reingold Graph Layout
#'
#' An S4 Class implementing the Fruchterman Reingold Graph Layout
#' algorithm.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' \describe{
#'   \item{ndim}{The number of dimensions, defaults to 2. Can only be 2 or 3}
#'   \item{knn}{Reduce the graph to keep only the neares neighbors. Defaults to 100.}
#'   \item{d}{The distance function to determine the weights of the graph edges. Defaults to euclidean distances.}
#' }
#'
#' @section Implementation:
#' Wraps around \code{\link[igraph]{layout_with_fr}}, see there for
#' details. The Fruchterman Reingold algorithm puts the data into
#' a circle and puts connected points close to each other.
#'
#' @references
#'
#' Fruchterman, T.M.J., Reingold, E.M., 1991. Graph drawing by force-directed
#' placement. Softw: Pract. Exper. 21, 1129-1164.
#' https://doi.org/10.1002/spe.4380211102
#'
#' @examples
#' if(requireNamespace(c("igraph", "coRanking"), quietly = TRUE)) {
#'
#' dat <- loadDataSet("Swiss Roll", n = 100)
#' emb <- embed(dat, "FruchtermanReingold")
#' plot(emb, type = "2vars")
#'
#' }
#'
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export FruchtermanReingold
#' @exportClass FruchtermanReingold
FruchtermanReingold <- setClass(
    "FruchtermanReingold",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(ndim         = 2,
                       knn          = 100,
                       d            = stats::dist),
        fun = function (data, pars,
                        keep.org.data = TRUE) {
        chckpkg("igraph")

        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        indata <- data@data

        outdata <- em_graph_layout(
            indata,
            graph_em_method = igraph::layout_with_fr,
            knn = pars$knn,
            d = pars$d,
            ndim = pars$ndim,
            weight.trans = I #pars$weight.trans
        )

        colnames(outdata) <- paste0("FR", 1:ncol(outdata))

        return(new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = outdata,
                               meta = meta),
            org.data     = orgdata,
            has.org.data = keep.org.data,
            method       = "graph_fr",
            pars         = pars
        ))
        },
        requires = c("igraph", "coRanking")
    )
)

em_graph_layout <- function(data, graph_em_method,
                            knn = 50, d = stats::dist,
                            ndim = 2, weight.trans = I){
  chckpkg("igraph")

  data.dist <- as.matrix(d(data))
  data.graph <- construct_knn_graph(data.dist, knn)

  embed_graph(data.graph, graph_em_method, ndim = ndim)
}

embed_graph <- function(graph, f, weight.trans = I, ndim = 2){
  f(graph, weights = weight.trans(igraph::E(graph)$weight), dim = ndim)
}


construct_knn_graph <- function (data.dist, knn) {
  chckpkg("igraph")
  chckpkg("coRanking")

  data.graph <- igraph::graph_from_adjacency_matrix(
    adjmatrix = data.dist,
    mode = "undirected",
    weighted = TRUE
  )

  if (is.infinite(knn) || is.na(knn))
    return(data.graph)
  ## else: remove all unnecessary edges
  data.rankm <- coRanking::rankmatrix(data.dist, input = "dist")
  data.rankm.ind <- data.rankm <= knn + 1
  inds <- which(
    !(data.rankm.ind | t(data.rankm.ind)),
    arr.ind = TRUE
  )

  data.graph[ from = inds[, 1], to = inds[, 2] ] <- FALSE

  return(data.graph)
}

## if (!isClassUnion("missingORnumeric"))   setClassUnion("missingORnumeric",   c("numeric", "missing"))
## if (!isClassUnion("missingORcharacter")) setClassUnion("missingORcharacter", c("character", "missing"))
## if (!isClassUnion("missingORlogical"))   setClassUnion("missingORlogical",   c("logical", "missing"))
## if (!isClassUnion("missingORfunction"))  setClassUnion("missingORfunction",  c("function", "missing"))

# Squared euclidean distance between points in A and B
# taken from http://blog.felixriedel.com/2013/05/pairwise-distances-in-r/

pdist2 <- function (A, B) {
    an <- rowSums(A ^ 2) # apply(A, 1, function(rvec) crossprod(rvec, rvec))
    bn <- rowSums(B ^ 2) # apply(B, 1, function(rvec) crossprod(rvec, rvec))

    m <- nrow(A)
    n <- nrow(B)

    matrix(rep(an, n), nrow = m) +
        matrix(rep(bn, m), nrow = m, byrow = TRUE) -
        2 * tcrossprod(A, B)
}


## a + b ~ c + d
## becomes
## ~ c + d + 0
rhs <- function (formula) {
    fs <- as.character(formula)[3]
    stats::as.formula(paste("~", fs, "+ 0"))
}

## a + b ~ c + d
## becomes
## ~ a + b + 0
lhs <- function (formula) {
    fs <- as.character(formula)[2]
    stats::as.formula(paste("~", fs, "+ 0"))
}

## check if a package is installed
chckpkg <- function (pkg) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
        stop(paste0("require '", pkg,
                    "' package, install it using install.packages('",
                    pkg, "')"))
    }
}

## create generics that appear in several different places

#' Converts to data.frame
#'
#' General conversions of objects created by \code{dimRed} to \code{data.frame}.
#' See class documentations for details (\code{\link{dimRedData}},
#' \code{\link{dimRedResult}}). For the documentation of this function in base
#' package, see here: \code{\link[base]{as.data.frame.default}}.
#'
#' @param x The object to be converted
#' @param row.names unused in \code{dimRed}
#' @param optional unused in \code{dimRed}
#' @param ... other arguments.
setGeneric(
    "as.data.frame",
    function(x, row.names, optional, ...) standardGeneric("as.data.frame"),
    useAsDefault = base::as.data.frame,
    valueClass = "data.frame"
)

#' Converts to dimRedData
#'
#' Conversion functions to dimRedData.
#'
#' @param formula The formula, left hand side is assigned to the meta slot right
#'   hand side is assigned to the data slot.
#' @param ... other arguments.
setGeneric(
    "as.dimRedData",
    function(formula, ...) standardGeneric("as.dimRedData"),
    valueClass = "dimRedData"
)

#' Method getData
#'
#' Extracts the data slot.
#'
#' @param object The object to be converted.
setGeneric("getData", function(object) standardGeneric("getData"))

#' Method getMeta
#'
#' Extracts the meta slot.
#'
#' @param object The object to be converted.
#' @param ... other arguments.
setGeneric("getMeta", function(object, ...) standardGeneric("getMeta"))

#' Method getPars
#'
#' Extracts the pars slot.
#'
#' @param object The object to be converted.
#' @param ... other arguments.
setGeneric("getPars", function (object, ...) standardGeneric("getPars"))

#' Method getNDim
#'
#' Extract the number of embedding dimensions.
#'
#' @param object The object to get the dimensions from.
#' @param ... other arguments.
setGeneric("getNDim", function (object, ...) standardGeneric("getNDim"))

#' Method getOrgData
#'
#' Extract the Original data.
#'
#' @param object The object to extract data from.
#' @param ... other arguments.
setGeneric("getOrgData", function (object, ...) standardGeneric("getOrgData"))

#' Method getDimRedData
#'
#' Extract dimRedData.
#' @param object The object to extract data from.
#' @param ... other arguments.
setGeneric("getDimRedData",
           function (object, ...) standardGeneric("getDimRedData"))

#' Method getOtherData
#'
#' Extract other data produced by a dimRedMethod
#'
#' @param object The object to extract data from.
#' @param ... other arguments.
setGeneric("getOtherData",
           function (object, ...) standardGeneric("getOtherData"),
           valueClass = "list")

#' Method print
#'
#' Imports the print method into the package namespace.
#'
#' @param x The object to be printed.
#' @param ... Other arguments for printing.
setGeneric("print", function(x, ...) standardGeneric("print"))


#' Method ndims
#'
#' Extract the number of dimensions.
#'
#' @param object To extract the number of dimensions from.
#' @param ... Arguments for further methods
setGeneric("ndims",
           function (object, ...) standardGeneric("ndims"),
           valueClass = "integer")


#' getSuggests
#'
#' Install packages wich are suggested by dimRed.
#'
#' By default dimRed will not install all the dependencies, because
#' there are quite a lot and in case some of them are not available
#' for your platform you will not be able to install dimRed without
#' problems.
#'
#' To solve this I provide a function which automatically installes
#' all the suggested packages.
#'
#' @param ... additional options passed to install.packages.
#'
#' @examples
#' \dontrun{
#' installSuggests()
#' }
#' @export
installSuggests <- function (...) {
    "%w/o%" <- function(x, y) x[!x %in% y]
    pkgString <- installed.packages()["dimRed", "Suggests"]
    deps <- strsplit(pkgString, ", |,\n")[[1]]
    deps <- gsub("\n", "", deps)        # Windows needs this

    installedPkgs <- rownames(installed.packages())
    missingPkgs <- deps %w/o% installedPkgs

    if (length(missingPkgs) > 0) {
        message("The following packages are missing: ")
        cat(missingPkgs, "\n")
        message("installing ...")
        install.packages(missingPkgs, ...)
        pkgString <- installed.packages()["dimRed", "Suggests"]
        installedPkgs <- rownames(installed.packages())
        missingPkgs <- deps %w/o% installedPkgs
        if (length(missingPkgs) > 0) {
            message("Could not install the following packages:")
            cat(missingPkgs, "\n")
            message("please install manually or some methods will not work.")
        } else {
            message("All necessary packages installed")
            message("If things still don't work try 'update.packages()'")
            message("If it still does not work file a bugreport!!")
        }
    } else {
        message("All necessary packages installed")
        message("If things still don't work try 'update.packages()'")
        message("If it still does not work file a bugreport!!")
    }

}


## input data(matrix or data frame) return knn graph implements
## "smart" choices on RANN::nn2 parameters we ignore radius search
## TODO: find out a good limit to switch from kd to bd trees COMMENT:
## bd trees are buggy, they dont work if there are duplicated data
## points and checking would neutralize the performance gain, so bd
## trees are not really usable.

#' makeKNNgraph
#'
#' Create a K-nearest neighbor graph from data x. Uses
#' \code{\link[RANN]{nn2}} as a fast way to find the neares neighbors.
#'
#' @param x data, a matrix, observations in rows, dimensions in
#'     columns
#' @param k the number of nearest neighbors.
#' @param eps number, if \code{eps > 0} the KNN search is approximate,
#'     see \code{\link[RANN]{nn2}}
#' @param diag logical, if \code{TRUE} every edge of the returned
#'     graph will have an edge with weight \code{0} to itself.
#'
#' @return an object of type \code{\link[igraph]{igraph}} with edge
#'     weight being the distances.
#' 
#' 
#' 
makeKNNgraph <- function(x, k, eps = 0, diag = FALSE){
    ## requireNamespace("RANN")
    ## requireNamespace("igraph")

    ## consts
    INF_VAL <- 1.340781e+15
    NA_IDX  <- 0
    BDKD_LIM <- 1000000                 #todo: figure out a good value here

    ## select parameters
    M <- nrow(x)
    treetype <- "kd"                # if (M < BDKD_LIM) "kd" else "bd"
                                    # see:
                                    # https://github.com/jefferis/RANN/issues/19
    searchtype <- if (eps == 0) "standard" else "priority"

    ## RANN::nn2 returns the points in data with respect to query
    ## e.g. the rows in the output are the points in query and the
    ## columns the points in data.
    nn2res <- RANN::nn2(data = x, query = x, k = k + 1, treetype = treetype,
                        searchtype = searchtype, eps = eps)

    ## create graph: the first ny nodes will be y, the last nx nodes
    ## will be x, if x != y
    g <- igraph::make_empty_graph(M, directed = FALSE)
    g[from = if (diag) rep(seq_len(M), times = k + 1)
             else      rep(seq_len(M), times = k),
      to   = if (diag) as.vector(nn2res$nn.idx)
             else      as.vector(nn2res$nn.idx[, -1]),
      attr = "weight"] <- if (diag) as.vector(nn2res$nn.dists)
                          else      as.vector(nn2res$nn.dists[, -1])

    return(g)
}


makeEpsSparseMatrix <- function(x, eps) {
    chckpkg("Matrix")
    n <- nrow(x)
    dd <- stats::dist(x)
    ddind <- dd < eps
    rows <- unlist(lapply(2:n, function(x) x:n), use.names = FALSE)
    cols <- rep(seq_len(n - 1), times = (n - 1):1)
    Matrix::sparseMatrix(i         = rows[ddind],
                         j         = cols[ddind],
                         x         = dd[ddind],
                         dims      = c(n, n),
                         symmetric = TRUE)
}

test_that("squared euclidean distance", {
    a <- matrix(rnorm(25), 5, 5)
    b <- matrix(rnorm(25), 5, 5)

    expect_equal(
        t(as.matrix(dist(rbind(a, b)))[6:10, 1:5] ^ 2),
        pdist2(a, b),
        ignore_attr = TRUE
    )
})

test_that("formula functions", {
    a <- matrix(rnorm(25), 5, 5)
    b <- matrix(rnorm(25), 5, 5)

    expect_true(rhs(a + b ~ c + d) == ~ c + d + 0)
    expect_true(lhs(a + b ~ c + d) == ~ a + b + 0)
    ## expect_equal(rhs(a + b ~ c + d), ~ c + d + 0)
    ## expect_equal(lhs(a + b ~ c + d), ~ a + b + 0)
})



test_that("makeEpsGraph", {
  if(!requireNamespace("Matrix", quietly = TRUE))
    skip("Matrix required")
  check_makeEpsGraph <- function(x, eps){
    naive <- as.matrix(dist(x))
    naive[naive >= eps] <- 0
    epsSp <- as.matrix(makeEpsSparseMatrix(x, eps))
    all(naive == epsSp)
  }
  expect_true(check_makeEpsGraph(iris[1:4], 1000))
  expect_true(check_makeEpsGraph(iris[1:4], 1))
  expect_true(check_makeEpsGraph(iris[1:4], 0.5))
})


test_that("getRotationMatrixFail", {
  if(!requireNamespace("Rtsne", quietly = TRUE))
    skip("Rtsne not available")

  irisData <- as(iris[, 1:4], "dimRedData")
  expect_equal(class(irisData)[1], "dimRedData")

  irisRes <- embed(irisData, "tSNE")

  expect_error(getRotationMatrix(irisRes), "Not implemented for")
})

#' Kernel PCA
#'
#' An S4 Class implementing Kernel PCA
#'
#' Kernel PCA is a nonlinear extension of PCA using kernel methods.
#'
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' Kernel PCA can take the following parameters:
#' \describe{
#'   \item{ndim}{the number of output dimensions, defaults to 2}
#'   \item{kernel}{The kernel function, either as a function or a
#'   character vector with the name of the kernel. Defaults to
#'   \code{"rbfdot"}}
#'   \item{kpar}{A list with the parameters for the kernel function,
#'     defaults to \code{list(sigma = 0.1)}}
#' }
#'
#' The most comprehensive collection of kernel functions can be found in
#' \code{\link[kernlab]{kpca}}. In case the function does not take any
#' parameters \code{kpar} has to be an empty list.
#'
#' @section Implementation:
#'
#' Wraps around \code{\link[kernlab]{kpca}}, but provides additionally
#' forward and backward projections.
#'
#' @references
#'
#' Sch\"olkopf, B., Smola, A., M\"uller, K.-R., 1998. Nonlinear Component Analysis
#' as a Kernel Eigenvalue Problem. Neural Computation 10, 1299-1319.
#' https://doi.org/10.1162/089976698300017467
#'
#' @examples
#' \dontrun{
#' if(requireNamespace("kernlab", quietly = TRUE)) {
#'
#' dat <- loadDataSet("3D S Curve")
#' emb <- embed(dat, "kPCA")
#' plot(emb, type = "2vars")
#' }
#'
#' }
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export kPCA
#' @exportClass kPCA
kPCA <- setClass(
    "kPCA",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(kernel = "rbfdot",
                       kpar = list(sigma = 0.1),
                       ndim = 2),
        fun = function (data, pars,
                        keep.org.data = TRUE) {

        chckpkg("kernlab")

        if (is.null(pars$ndim)) pars$ndim <- 2

        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        indata <- data@data

        message(Sys.time(), ": Calculating kernel PCA")
        res <- do.call(kernlab::kpca, c(list(x = indata), pars))

        kernel <- get_kernel_fun(pars$kernel, pars$kpar)

        message(Sys.time(), ": Trying to calculate reverse")
        K_rev <- kernlab::kernelMatrix(kernel, res@rotated)
        diag(K_rev) <- 0.1 + diag(K_rev)
        dual_coef <- try(solve(K_rev, indata), silent = TRUE)

        appl <- function (x) {
            appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
            proj <- if (inherits(x, "dimRedData")) x@data else x

            proj <- kernlab::predict(res, proj)[, 1:pars$ndim, drop = FALSE]
            colnames(proj) <- paste0("kPCA", 1:ncol(proj))

            new("dimRedData", data = proj, meta = appl.meta)
        }

        inv <-
          if (inherits(dual_coef, "try-error")) {
            message("No inverse function.")
            function(x) NA
          } else {
            function (x) {
              appl.meta <-
                if (inherits(x, "dimRedData")) x@meta else data.frame()
              proj <- if (inherits(x, "dimRedData")) x@data else x

              resrot <- res@rotated[, 1:ncol(proj)]
              rot <- kernlab::kernelMatrix(kernel, proj, resrot)
              proj <- rot %*% dual_coef

              new("dimRedData", data = proj, meta = appl.meta)
            }
          }


        outdata <- res@rotated[, 1:pars$ndim, drop = FALSE]
        colnames(outdata) <- paste0("kPCA", 1:ncol(outdata))

        message(Sys.time(), ": DONE")
        return(
            new(
                "dimRedResult",
                data         = new("dimRedData",
                                   data = outdata,
                                   meta = meta),
                org.data     = orgdata,
                apply        = appl,
                inverse      = inv,
                has.org.data = keep.org.data,
                has.apply    = TRUE,
                has.inverse  = TRUE,
                method       = "kpca",
                pars         = pars
            )
        )
        },
      requires = c("kernlab"))
)


## get the kernel function out of the kernlab namespace:
get_kernel_fun <- function (kernel, pars) {
    if (!is(kernel, "kernel")) {
        if (is(kernel, "function")) {
            kernel <- deparse(substitute(kernel))
        } else {
           kernel <- get(kernel, asNamespace("kernlab"))
        }
        kernel <- do.call(kernel, pars)
    }
    return(kernel)
}

#' Diffusion Maps
#'
#' An S4 Class implementing Diffusion Maps
#'
#' Diffusion Maps uses a diffusion probability matrix to robustly
#' approximate a manifold.
#'
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' Diffusion Maps can take the following parameters:
#' \describe{
#'   \item{d}{a function transforming a matrix row wise into a
#'     distance matrix or \code{dist} object,
#'     e.g. \code{\link[stats]{dist}}.}
#'   \item{ndim}{The number of dimensions}
#'   \item{eps}{The epsilon parameter that determines the
#'      diffusion weight matrix from a distance matrix \code{d},
#'      \eqn{exp(-d^2/eps)}, if set to \code{"auto"} it will
#'      be set to the median distance to the 0.01*n nearest
#'      neighbor.}
#'   \item{t}{Time-scale parameter. The recommended value, 0,
#'      uses multiscale geometry.}
#'   \item{delta}{Sparsity cut-off for the symmetric graph Laplacian,
#'     a higher value results in more sparsity and faster calculation.
#'     The predefined value is 10^-5.}
#' }
#'
#' @section Implementation:
#' Wraps around \code{\link[diffusionMap]{diffuse}}, see there for
#' details. It uses the notation of Richards et al. (2009) which is
#' slightly different from the one in the original paper (Coifman and
#' Lafon, 2006) and there is no \eqn{\alpha} parameter.
#' There is also an out-of-sample extension, see examples.
#'
#'
#' @references
#' Richards, J.W., Freeman, P.E., Lee, A.B., Schafer,
#'     C.M., 2009. Exploiting Low-Dimensional Structure in
#'     Astronomical Spectra. ApJ 691,
#'     32. doi:10.1088/0004-637X/691/1/32
#'
#' Coifman, R.R., Lafon, S., 2006. Diffusion maps. Applied and
#'     Computational Harmonic Analysis 21,
#'     5-30. doi:10.1016/j.acha.2006.04.006
#'
#' @examples
#' if(requireNamespace("diffusionMap", quietly = TRUE)) {

#' dat <- loadDataSet("3D S Curve", n = 300)
#' emb <- embed(dat, "DiffusionMaps")
#'
#' plot(emb, type = "2vars")
#'
#' # predicting is possible:
#' samp <- sample(floor(nrow(dat) / 10))
#' emb2 <- embed(dat[samp])
#' emb3 <- predict(emb2, dat[-samp])
#'
#' plot(emb2, type = "2vars")
#' points(getData(emb3))

#' }
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export DiffusionMaps
#' @exportClass DiffusionMaps
DiffusionMaps <- setClass(
    "DiffusionMaps",
    contains  = "dimRedMethod",
    prototype = list(
        stdpars = list(d     = stats::dist,
                       ndim  = 2,
                       eps   = "auto",
                       t     = 0,
                       delta = 1e-5),
        fun     = function (data, pars,
                            keep.org.data = TRUE) {
        chckpkg("diffusionMap")

        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        indata <- data@data

        distmat <- pars$d(indata)
        if (pars$eps == "auto")
            pars$eps <- diffusionMap::epsilonCompute(distmat)
        diffres <- diffusionMap::diffuse(
                                     D = distmat,
                                     t = pars$t,
                                     eps.val = pars$eps,
                                     neigen = pars$ndim,
                                     maxdim = pars$ndim,
                                     delta = pars$delta
                                 )
        outdata <- as.matrix(diffres$X)


        appl <- function(x) {
            appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
            proj <- if (inherits(x, "dimRedData")) x@data else x

            if (ncol(proj) != ncol(data@data))
                stop("x must have the same number of dimensions ",
                     "as the original data")

            dd <- sqrt(pdist2(proj, indata))

            appl.res <-
                diffusionMap::nystrom(diffres, dd, sigma = diffres$epsilon)
            dimnames(appl.res) <- list(
                rownames(x), paste0("diffMap", seq_len(ncol(outdata)))
            )

            new("dimRedData", data = appl.res, meta = appl.meta)
        }

        colnames(outdata) <- paste0("diffMap", seq_len(ncol(outdata)))

        return(new(
            "dimRedResult",
            data = new("dimRedData",
                       data = outdata,
                       meta = meta),
            org.data     = orgdata,
            apply        = appl,
            has.apply    = TRUE,
            has.org.data = keep.org.data,
            method       = "diffmap",
            pars         = pars
        ))
        },
      requires = c("diffusionMap"))
)

## loe got removed from CRAN



## ' Laplacian Eigenmaps
## #'
## #' An S4 Class implementing Laplacian Eigenmaps
## #'
## #' Laplacian Eigenmaps use a kernel and were originally developed to
## #' separate non-convex clusters under the name spectral clustering.
## #'
## #' @template dimRedMethodSlots
## #'
## #' @template dimRedMethodGeneralUsage
## #'
## #' @section Parameters:
## #' \code{LaplacianEigenmaps} can take the following parameters:
## #' \describe{
## #'   \item{ndim}{the number of output dimensions.}
## #'
## #'   \item{sparse}{A character vector specifying hot to make the graph
## #'    sparse, \code{"knn"} means that a K-nearest neighbor graph is
## #'    constructed, \code{"eps"} an epsilon neighborhood graph is
## #'    constructed, else a dense distance matrix is used.}
## #'
## #'   \item{knn}{The number of nearest neighbors to use for the knn graph.}
## #'   \item{eps}{The distance for the epsilon neighborhood graph.}
## #'
## #'   \item{t}{Parameter for the transformation of the distance matrix
## #'   by \eqn{w=exp(-d^2/t)}, larger values give less weight to
## #'   differences in distance, \code{t == Inf} treats all distances != 0 equally.}
## #'   \item{norm}{logical, should the normed laplacian be used?}
## #' }
## #'
## #' @section Implementation:
## #' Wraps around \code{\link[loe]{spec.emb}}.
## #'
## #' @references
## #'
## #' Belkin, M., Niyogi, P., 2003. Laplacian Eigenmaps for
## #' Dimensionality Reduction and Data Representation. Neural
## #' Computation 15, 1373.
## #'
## #' @examples
## #' if(requireNamespace(c("loe", "RSpectra", "Matrix"), quietly = TRUE)) {
## #'
## #' dat <- loadDataSet("3D S Curve")
## #' emb <- embed(dat, "LaplacianEigenmaps")
## #' plot(emb@data@data)
## #'
## #' }
## #' @include dimRedResult-class.R
## #' @include dimRedMethod-class.R
## #' @export LaplacianEigenmaps
## #' @exportClass LaplacianEigenmaps
## LaplacianEigenmaps <- setClass(
##     "LaplacianEigenmaps",
##     contains = "dimRedMethod",
##     prototype = list(
##         stdpars = list(ndim = 2, sparse = "knn", knn = 50, eps = 0.1,
##                        t = Inf, norm = TRUE),
##         fun = function (data, pars,
##                         keep.org.data = TRUE) {
##         chckpkg("loe")
##         chckpkg("RSpectra")
##         chckpkg("Matrix")

##         meta <- data@meta
##         orgdata <- if (keep.org.data) data@data else NULL
##         indata <- data@data

##         if (is.null(pars$d))     pars$d    <- dist
##         if (is.null(pars$knn))   pars$knn  <- 50
##         if (is.null(pars$ndim))  pars$ndim <- 2
##         if (is.null(pars$t))     pars$t    <- Inf
##         if (is.null(pars$norm))  pars$norm <- TRUE


##         message(Sys.time(), ": Creating weight matrix")
##         W <- if (pars$sparse == "knn") {
##                  knng <- makeKNNgraph(indata, k = pars$knn, eps = 0,
##                                       diag = TRUE)
##                  if (is.infinite(pars$t)){
##                      knng <- igraph::set_edge_attr(knng, name = "weight", value = 1)
##                  } else {
##                    ea <- igraph::edge_attr(knng, name = "weight")
##                    knng <- igraph::set_edge_attr(
##                      knng, name = "weight", value = exp( -(ea ^ 2) / pars$t ))
##                  }
##                  igraph::as_adj(knng, sparse = TRUE,
##                                 attr = "weight", type = "both")
##              } else if (pars$sparse == "eps") {
##                  tmp <- makeEpsSparseMatrix(indata, pars$eps)
##                  tmp@x <- if (is.infinite(pars$t)) rep(1, length(tmp@i))
##                           else exp(- (tmp@x ^ 2) / pars$t)
##                  ## diag(tmp) <- 1
##                  as(tmp, "dgCMatrix")
##              } else {                        # dense case
##                  tmp <- dist(indata)
##                  tmp[] <- if (is.infinite(pars$t)) 1
##                           else exp( -(tmp ^ 2) / pars$t)
##                  tmp <- as.matrix(tmp)
##                  diag(tmp) <- 1
##                  tmp
##              }

##         ## we don't need to test for symmetry, because we know the
##         ## matrix is symmetric
##         D <- Matrix::Diagonal(x = Matrix::rowSums(W))
##         L <- D - W
##         ## for the generalized eigenvalue problem, we do not have a solver
##         ## use A u = \lambda B u
##         ## Lgen <- Matrix::Diagonal(x = 1 / Matrix::diag(D) ) %*% L
##         ## but then we get negative eigenvalues and complex eigenvalues
##         Lgen <- L
##         message(Sys.time(), ": Eigenvalue decomposition")
##         outdata <- if (pars$norm) {
##                        DS <- Matrix::Diagonal(x = 1 / sqrt(Matrix::diag(D)))
##                        RSpectra::eigs_sym(DS %*% Lgen %*% DS,
##                                           k = pars$ndim + 1,
##                                           sigma = -1e-5)
##                    } else {
##                        RSpectra::eigs_sym(Lgen,
##                                           k = pars$ndim + 1,
##                                           sigma = -1e-5)
##                    }
##         message("Eigenvalues: ", paste(format(outdata$values),
##                                        collapse = " "))
##         ## The eigenvalues are in decreasing order and we remove the
##         ## smallest, which should be approx 0:
##         outdata <- outdata$vectors[, order(outdata$values)[-1],
##                                    drop = FALSE]

##         if(ncol(outdata) > 0) {
##           colnames(outdata) <- paste0("LEIM", seq_len(ncol(outdata)))
##         } else {
##           warning("no dimensions left, this is probably due to a badly conditioned eigenvalue decomposition.")
##         }

##         message(Sys.time(), ": DONE")
##         return(new(
##             "dimRedResult",
##             data         = new("dimRedData",
##                                data = outdata,
##                                meta = meta),
##             org.data     = orgdata,
##             has.org.data = keep.org.data,
##             method       = "leim",
##             pars         = pars
##         ))
##         },
##         requires = c("loe", "RSpectra", "Matrix"))
## )

#' Principal Component Analysis with L1 error.
#'
#' S4 Class implementing PCA with L1 error.
#'
#' PCA transforms the data so that the L2 reconstruction error is minimized or
#' the variance of the projected data is maximized. This is sensitive to
#' outliers, L1 PCA minimizes the L1 reconstruction error or maximizes the sum
#' of the L1 norm of the projected observations.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' PCA can take the following parameters:
#' \describe{
#'   \item{ndim}{The number of output dimensions.}
#'   \item{center}{logical, should the data be centered, defaults to \code{TRUE}.}
#'   \item{scale.}{logical, should the data be scaled, defaults to \code{FALSE}.}
#'   \item{fun}{character or function, the method to apply, see the \code{pcaL1} package}
#'   \item{\ldots}{other parameters for \code{fun}}
#' }
#'
#' @section Implementation:
#'
#' Wraps around the different methods is the \code{pcaL1} package. Because PCA
#' can be reduced to a simple rotation, forward and backward projection
#' functions are supplied.
#'
#' @references
#'
#' Park, Y.W., Klabjan, D., 2016. Iteratively Reweighted Least Squares
#' Algorithms for L1-Norm Principal Component Analysis, in: Data Mining (ICDM),
#' 2016 IEEE 16th International Conference On. IEEE, pp. 430-438.
#'
#' @examples
#' if(requireNamespace("pcaL1", quietly = TRUE)) {
#'
#' dat <- loadDataSet("Iris")
#' emb <- embed(dat, "PCA_L1")
#'
#' plot(emb, type = "2vars")
#' plot(inverse(emb, getData(getDimRedData((emb)))), type = "3vars")
#'
#' }
#'
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export PCA_L1
#' @exportClass PCA_L1
PCA_L1 <- setClass(
    "PCA_L1",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(ndim = 2,
                       center = TRUE,
                       scale. = FALSE,
                       fun = "awl1pca",
                       projections = "l1"),
        fun = function (data, pars,
                        keep.org.data = TRUE) {
        chckpkg("pcaL1")

        ndim <- pars$ndim

        orgnames <- colnames(data@data)
        newnames <- paste0("PC", seq_len(ndim))

        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        data <- data@data

        fun2 <- if(!is.function(pars$fun)) {
                    get(pars$fun, asNamespace("pcaL1"))
                } else {
                    pars$fun
                }

        ce <- if (is.numeric(pars$center)) {
                  if (length(pars$center) != dim(data)[2])
                      error("center must be logical or have the same length as the data dimensions")
                  pars$center
              } else if (is.logical(pars$center)) {
                  if (pars$center) colMeans(data) else FALSE
              }

        sc <- if (is.numeric(pars$scale.)) {
                  if (length(pars$scale.) != dim(data)[2])
                      stop("center must be logical or have the same length as the data dimensions")
                  pars$scale.
              } else if (is.logical(pars$scale.)) {
                  if (pars$scale.) apply(data, 2, sd) else FALSE
              }

        if(!(pars$center == FALSE && pars$scale. == FALSE))
            data <- scale(data, ce, sc)


        pars$center <- NULL
        pars$scale. <- NULL
        pars$ndim <- NULL
        pars$fun <- NULL

        res <- do.call(
            fun2,
            c(list(X = data, projDim = ndim, center = FALSE), pars)
        )

        ## evaluate results here for functions
        data <- res$scores
        colnames(data) <- paste0("PC", seq_len(ndim))
        rot <- res$loadings[, seq_len(ndim), drop = FALSE]
        dimnames(rot) <- list(orgnames, newnames)
        rerot <- t(rot)


        appl <- function(x) {
            appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
            proj <- if (inherits(x, "dimRedData")) x@data else x

            if (ncol(proj) != ncol(orgdata))
                stop("x must have the same number of dimensions ",
                     "as the original data")

            if (ce[1]  != FALSE) proj <- t(apply(proj, 1, function(x) x - ce))
            if (sc[1]  != FALSE) proj <- t(apply(proj, 1, function(x) x / sc))

            proj <- if (pars$projections == "l1") {
                        tmp <- pcaL1::l1projection(proj, rot)$scores
                        colnames(tmp) <- paste0("PC", seq_len(ndim))
                        tmp
                    } else if (pars$projections == "l2") {
                        proj %*% rot
                    } else {
                        stop("projections must be eiter 'l1' or 'l2'")
                    }

            proj <- new("dimRedData", data = proj, meta = appl.meta)
            return(proj)
        }
        inv  <- function(x) {
            appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
            proj <- if (inherits(x, "dimRedData")) x@data else x
            if (ncol(proj) > ncol(data))
                stop("x must have less or equal number of dimensions ",
                     "as the original data")


            d <- ncol(proj)
            reproj <- proj %*% rerot[seq_len(d), ]

            if (sc[1] != FALSE)
                reproj <- t(apply(reproj, 1, function(x) x * sc))
            if (ce[1] != FALSE)
                reproj <- t(apply(reproj, 1, function(x) x + ce))

            colnames(reproj) <- colnames(orgdata)

            reproj <- new("dimRedData", data = reproj, meta = appl.meta)

            return(reproj)
        }

        res <- new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = data,
                               meta = meta),
            org.data     = orgdata,
            apply        = appl,
            inverse      = inv,
            has.org.data = keep.org.data,
            has.apply    = TRUE,
            has.inverse  = TRUE,
            method       = "PCA_L1",
            pars         = pars
        )

        return(res)
        },
      requires = c("pcaL1"))
)

#' t-Distributed Stochastic Neighborhood Embedding
#'
#' An S4 Class for t-SNE.
#'
#' t-SNE is a method that uses Kullback-Leibler divergence between the
#' distance matrices in high and low-dimensional space to embed the
#' data. The method is very well suited to visualize complex
#' structures in low dimensions.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' t-SNE can take the following parameters:
#' \describe{
#'   \item{d}{A distance function, defaults to euclidean distances}
#'   \item{perplexity}{The perplexity parameter, roughly equivalent to neighborhood size.}
#'   \item{theta}{Approximation for the nearest neighbour search, large values are more inaccurate.}
#'   \item{ndim}{The number of embedding dimensions.}
#' }
#'
#' @section Implementation:
#'
#' Wraps around \code{\link[Rtsne]{Rtsne}}, which is very well
#' documented. Setting \code{theta = 0} does a normal t-SNE, larger
#' values for \code{theta < 1} use the Barnes-Hut algorithm which
#' scales much nicer with data size. Larger values for perplexity take
#' larger neighborhoods into account.
#'
#' @references
#' Maaten, L. van der, 2014. Accelerating t-SNE using Tree-Based
#' Algorithms. Journal of Machine Learning Research 15, 3221-3245.
#'
#' van der Maaten, L., Hinton, G., 2008. Visualizing Data using
#' t-SNE. J. Mach. Learn. Res. 9, 2579-2605.
#'
#' @examples
#' \dontrun{
#' dat <- loadDataSet("3D S Curve", n = 300)
#' emb <- embed(dat, "tSNE", perplexity = 80)
#' plot(emb, type = "2vars")
#' }
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export tSNE
#' @exportClass tSNE
tSNE <- setClass(
    "tSNE",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(d = stats::dist,
                       perplexity = 30,
                       theta = 0.5,
                       ndim = 2),
        fun = function (data, pars,
                        keep.org.data = TRUE) {
        chckpkg("Rtsne")

        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        indata <- data@data

        outdata <- Rtsne::Rtsne(pars$d(indata),
                                perplexity = pars$perplexity,
                                theta = pars$theta,
                                dims = pars$ndim)$Y

        colnames(outdata) <- paste0("tSNE", 1:ncol(outdata))

        return(new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = outdata,
                               meta = meta),
            org.data     = orgdata,
            has.org.data = keep.org.data,
            method       = "tsne",
            pars         = pars
        ))
        },
        requires = c("Rtsne"))
)

## #' AutoEncoder
## #'
## #' An S4 Class implementing an Autoencoder
## #'
## #' Autoencoders are neural networks that try to reproduce their input. Consider
## #' this method unstable, as the internals may still be changed.
## #'
## #' @template dimRedMethodSlots
## #'
## #' @template dimRedMethodGeneralUsage
## #'
## #' @section Parameters:
## #' Autoencoder can take the following parameters:
## #' \describe{
## #'   \item{ndim}{The number of dimensions for reduction.}
## #'   \item{n_hidden}{The number of neurons in the hidden
## #'       layers, the length specifies the number of layers,
## #'       the length must be impair, the middle number must
## #'       be the same as ndim.}
## #'   \item{activation}{The activation functions for the layers,
## #'       one of "tanh", "sigmoid", "relu", "elu", everything
## #'       else will silently be ignored and there will be no
## #'       activation function for the layer.}
## #'   \item{weight_decay}{the coefficient for weight decay,
## #'       set to 0 if no weight decay desired.}
## #'   \item{learning_rate}{The learning rate for gradient descend}
## #'   \item{graph}{Optional: A list of bits and pieces that define the
## #'      autoencoder in tensorflow, see details.}
## #'   \item{keras_graph}{Optional: A list of keras layers that define
## #'      the encoder and decoder, specifying this, will ignore all
## #'      other topology related variables, see details.}
## #'   \item{batchsize}{If NA, all data will be used for training,
## #'       else only a random subset of size batchsize will be used}
## #'   \item{n_steps}{the number of training steps.}
## #' }
## #'
## #' @section Details:
## #' There are several ways to specify an autoencoder, the simplest is to pass the
## #' number of neurons per layer in \code{n_hidden}, this must be a vector of
## #' integers of impair length and it must be symmetric and the middle number must
## #' be equal to \code{ndim}, For every layer an activation function can be
## #' specified with \code{activation}.
## #'
## #' For regularization weight decay can be specified by setting
## #' \code{weight_decay} > 0.
## #'
## #' Currently only a gradient descent optimizer is used, the learning rate can be
## #' specified by setting \code{learning_rate}.
## #' The learner can operate on batches if \code{batchsize} is not \code{NA}.
## #' The number of steps the learner uses is specified using \code{n_steps}.
## #'
## #' @section Further training a model:
## #' If the model did not converge in the first training phase or training with
## #' different data is desired, the \code{\link{dimRedResult}} object may be
## #' passed as \code{autoencoder} parameter; In this case all topology related
## #' parameters will be ignored.
## #'
## #' @section Using Keras layers:
## #' The encoder and decoder part can be specified using a list of \pkg{keras}
## #' layers. This requires a list with two entries, \code{encoder} should contain
## #' a LIST of keras layers WITHOUT the \code{\link[keras]{layer_input}}
## #' that will be concatenated in order to form the encoder part.
## #' \code{decoder} should be
## #' defined accordingly, the output of \code{decoder} must have the same number
## #' of dimensions as the input data.
## #'
## #' @section Using Tensorflow:
## #' The model can be entirely defined in \pkg{tensorflow}, it must contain a
## #' list with the following entries:
## #' \describe{
## #'   \item{encoder}{A tensor that defines the encoder.}
## #'   \item{decoder}{A tensor that defines the decoder.}
## #'   \item{network}{A tensor that defines the reconstruction (encoder + decoder).}
## #'   \item{loss}{A tensor that calculates the loss (network + loss function).}
## #'   \item{in_data}{A \code{placeholder} that points to the data input of
## #'     the network AND the encoder.}
## #'   \item{in_decoder}{A \code{placeholder} that points to the input of
## #'     the decoder.}
## #'   \item{session}{A \pkg{tensorflow} \code{Session} object that holds
## #'     the values of the tensors.}
## #' }
## #'
## #' @section Implementation:
## #' Uses \pkg{tensorflow} as a backend, for details an
## #'   problems relating tensorflow, see \url{https://tensorflow.rstudio.com}.
## #'
## #' @examples
## #' \dontrun{
## #' dat <- loadDataSet("3D S Curve")
## #'
## #' emb <- embed(dat, "AutoEncoder")
## #'
## #' # predicting is possible:
## #' samp <- sample(floor(nrow(dat) / 10))
## #' emb2 <- embed(dat[samp])
## #' emb3 <- predict(emb2, dat[-samp])
## #'
## #' plot(emb, type = "2vars")
## #' plot(emb2, type = "2vars")
## #' points(getData(emb3))
## #' }
## #'
## #' @include dimRedResult-class.R
## #' @include dimRedMethod-class.R
## #' @family dimensionality reduction methods
## #' @export AutoEncoder
## #' @exportClass AutoEncoder
## AutoEncoder <- setClass(
##   "AutoEncoder",
##   contains  = "dimRedMethod",
##   prototype = list(
##     stdpars = list(ndim          = 2,
##                    n_hidden      = c(10, 2, 10),
##                    activation    = c("tanh", "lin", "tanh"),
##                    learning_rate = 0.15,
##                    loss          = "mse",
##                    optimizer     = "adam",
##                    encoder       = NULL,
##                    decoder       = NULL,
##                    ## is.na() of an S4 class gives a warning
##                    batchsize     = 20,
##                    epochs        = 5,
##                    validation_split = 0.2),

##     fun = function (data, pars,
##                     keep.org.data = TRUE) {
##       chckpkg("keras3")
##       indata <- data@data
##       indims <- ncol(indata)
##       ndim <- pars$ndim

##       input_data <- keras3::layer_input(shape = indims)
##       input_hidden <- keras3::layer_input(shape = ndim)

##       ## if the user did not specify encoder and decoder, we build them from the
##       ## parameters
##       if (is.null(pars$encoder) || is.null(pars$decoder)) {
##         depth <- length(pars$n_hidden)
##         if (depth %% 2 == 0) {
##           stop("the number of layers must be impair")
##         }

##         if (ndim != pars$n_hidden[ceiling(depth / 2)]) {
##           stop("the middle of n_hidden must be equal to ndim")
##         }

##         if (depth != length(pars$activation)) {
##           stop("declare an activation function for each layer")
##         }


##         in_depth <- ceiling(depth / 2)
##         out_depth <- depth - in_depth


##         layers <- mapply(
##           function(s, n) keras3::layer_dense(units = n, activation = s),
##           pars$activation, pars$n_hidden,
##           SIMPLIFY = FALSE
##         )

##         encoder <- c(input_data, layers[1:in_depth])
##         decoder <- c(input_hidden, layers[(in_depth + 1):depth])
##       } else {
##         encoder <- c(input_data, pars$encoder)
##         decoder <- c(input_hidden, pars$decoder)
##       }

##       ## now build the actual model, compile it and fit it
##       autoencoder <- c(input_data, encoder, decoder)
##       encoder <- encoder %>%
##         chain_list() %>%
##         keras3::keras_model()
##       decoder <- decoder %>%
##         chain_list() %>%
##         keras3::keras_model()
##       autoencoder <- autoencoder %>%
##         chain_list() %>%
##         keras3::keras_model()

##       autoencoder %>%
##         keras3::compile(
##           optimizer = pars$optimizer,
##           loss = pars$loss
##         )

##       history <- autoencoder %>%
##         keras3::fit(
##                  indata, indata,
##                  epochs           = pars$epochs,
##                  batch_size       = pars$batchsize,
##                  validation_split = pars$validation_split,
##                  verbose          = 0
##                )

##       meta <- data@meta
##       orgdata <- if (keep.org.data) data@data else NULL
##       indata <- data@data

##       appl <- function(x) {
##         appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
##         proj <- if (inherits(x, "dimRedData")) x@data else x

##         if (ncol(proj) != ncol(data@data))
##           stop("x must have the same number of dimensions ",
##                "as the original data")

##         res <- encoder(proj)
##         colnames(res) <- paste0("AE", seq_len(ncol(res)))
##         new("dimRedData", data = res, meta = appl.meta)
##       }

##       inv <- function(x) {
##         appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
##         proj <- if (inherits(x, "dimRedData")) x@data else x

##         if (ncol(proj) != pars$ndim)
##           stop("x must have the same number of dimensions ",
##                "as ndim data")

##         res <- decoder(proj)
##         colnames(res) <- colnames(indata)
##         new("dimRedData", data = res, meta = appl.meta)
##       }

##       colnames(outdata) <- paste0("AE", seq_len(ncol(outdata)))

##       return(new(
##         "dimRedResult",
##         data = new("dimRedData",
##                    data = outdata,
##                    meta = meta),
##         org.data     = orgdata,
##         apply        = appl,
##         inverse      = inv,
##         has.apply    = TRUE,
##         has.inverse  = TRUE,
##         has.org.data = keep.org.data,
##         method       = "AutoEncoder",
##         pars         = pars
##       ))
##     },
##     requires = c("tensorflow", "keras3"))
## )

## ## no idea why these and variants do not work:
## ## chain_list <- function(x1, x2) Reduce(`%>%`, x2, init = x1)
## ## chain_list <- function(x) Reduce(`%>%`, x)
## chain_list <- function(x1, x2 = NULL) {

##   if (is.null(x2)) {
##     stopifnot(is.list(x1))
##     result <- x1[[1]]
##     if (length(x1) > 1) for (i in 2:length(x1)) {
##                           result <- result %>% (x1[[i]])
##                         }
##   } else {
##     stopifnot(is.list(x2))
##     result <- x1
##     for (i in 1:length(x2)) {
##       result <- result %>% (x2[[i]])
##     }
##   }

##   return(result)
## }

#' Example Data Sets for dimensionality reduction
#'
#' A compilation of standard data sets that are often being used to
#' showcase dimensionality reduction techniques.
#'
#' The argument \code{name} should be one of
#' \code{dataSetList()}. Partial matching is possible, see
#' \code{\link{match.arg}}. Generated data sets contain the internal
#' coordinates of the manifold in the \code{meta} slot.  Call
#' \code{dataSetList()} to see what data sets are available.
#'
#'
#'
#' @param name A character vector that specifies the name of the data
#'     set.
#' @param n In generated data sets the number of points to be
#'     generated, else ignored.
#' @param sigma In generated data sets the standard deviation of the
#'     noise added, else ignored.
#' @return \code{loadDataSet} an object of class
#'     \code{\link{dimRedData}}. \code{dataSetList()} return a
#'     character string with the implemented data sets
#'
#' @examples
#' ## a list of available data sets:
#' dataSetList()
#'
#' ## Load a data set:
#' swissRoll <- loadDataSet("Swiss Roll")
#' \donttest{
#' if(requireNamespace("scatterplot3d", quietly = TRUE))
#'   plot(swissRoll, type = "3vars")
#' }
#'
#' ## Load Iris data set, partial matching:
#' loadDataSet("I")
#'
#' @name dataSets
NULL

#' @include dimRedData-class.R
#' @rdname dataSets
#' @export
loadDataSet <- function (name = dataSetList(), n = 2000, sigma = 0.05) {
    name <- match.arg(name)
    switch(
        name,
        "Swiss Roll"           = swissRoll(n, sigma),
        "Broken Swiss Roll"    = brokenSwissRoll(n, sigma),
        "Helix"                = helix(n, sigma),
        "Twin Peaks"           = twinPeaks(n, sigma),
        "Sphere"               = sphere(n, sigma),
        "FishBowl"             = fishbowl(n, sigma),
        "Ball"                 = ball(n, sigma),
        "3D S Curve"           = sCurve(n, sigma),
        "variable Noise Helix" = noisyHelix(n, sigma),
        "Cube"                 = cube(n, sigma),
        "Iris"                 = irisdata()
    )
}

#' @rdname dataSets
#' @export
dataSetList <- function () {
    return(c(
        "Swiss Roll",
        "Broken Swiss Roll",
        "Helix",
        "Twin Peaks",
        "Sphere",
        "Ball",
        "FishBowl",
        "3D S Curve",
        "variable Noise Helix",
        "Iris",
        "Cube"
    ))
}

irisdata <- function() {
    dd <- as.matrix(datasets::iris[, 1:4])
    new("dimRedData",
        data = dd,
        meta = datasets::iris[, 5, drop = FALSE])
}

swissRoll <- function (n = 2000, sigma = 0.05) {
    x <- stats::runif(n, 1.5 * pi, 4.5 * pi)
    y <- stats::runif(n, 0, 30)

    new("dimRedData",
        data = swissRollMapping(x, y) + stats::rnorm(3 * n, sd = sigma),
        meta = data.frame(x = x, y = y))
}

brokenSwissRoll <- function (n = 2000, sigma = 0.05) {
    x <- c(
        stats::runif(floor(n / 2),   1.5 * pi, 2.7 * pi),
        stats::runif(ceiling(n / 2), 3.3 * pi, 4.5 * pi)
    )
    y <- stats::runif(n, 0, 30)

    new("dimRedData",
        data = swissRollMapping(x, y) + stats::rnorm(3 * n, sd = sigma),
        meta = data.frame(x = x, y = y))
}

swissRollMapping <- function (x, y) {
    cbind(x = x * cos(x),
          y = y,
          z = x * sin(x))
}


helix <- function (n = 2000, sigma = 0.05) {
    t <- stats::runif(n, 0, 2 * pi)
    new("dimRedData",
        data = helixMapping(t) + stats::rnorm(3 * n, sd = sigma),
        meta = data.frame(t = t))
}

helixMapping <- function (x) {
    cbind(x = (2 + cos(8 * x)) * cos(x),
          y = (2 + cos(8 * x)) * sin(x),
          z = (sin(8 * x)))
}

twinPeaks <- function (n = 2000, sigma = 0.05) {
    x <- stats::runif(n, -1, 1)
    y <- stats::runif(n, -1, 1)

    new("dimRedData",
        data = twinPeaksMapping(x, y) + stats::rnorm(3 * n, sd = sigma),
        meta = data.frame(x = x, y = y))
}

twinPeaksMapping <- function (x, y) {
    cbind(x = x,
          y = y,
          z = sin(pi * x) * tanh(3 * y))
}


sphere <- function (n = 2000, sigma = 0.05) {
    phi <- stats::runif(n, 0, 2 * pi)
    psi <- acos(stats::runif(n, -1, 1))

    new("dimRedData",
        data = sphereMapping(phi, psi) + stats::rnorm(3 * n, sd = sigma),
        meta = data.frame(phi = phi, psi = psi))
}

fishbowl <- function (n = 2000, sigma = 0.05) {
    phi <- stats::runif(n, 0, 2 * pi)
    psi <- acos(stats::runif(n, -1, 0.8))

    new("dimRedData",
        data = sphereMapping(phi, psi) + stats::rnorm(3 * n, sd = sigma),
        meta = data.frame(psi = psi))
}

sphereMapping <- function (phi, psi) {
    cbind(x = cos(phi) * sin(psi),
          y = sin(phi) * sin(psi),
          z = cos(psi))
}

ball <- function (n = 2000, sigma = 0.05) {
    phi <- stats::runif(n, 0, 2 * pi)
    psi <- acos(stats::runif(n, -1, 1))
    ## make it uniformly distributed inside the sphere
    r <-  stats::runif(n) ^ (1 / 3)

    new("dimRedData",
        data = ballMapping(phi, psi, r) + stats::rnorm(3 * n, sd = sigma),
        meta = data.frame(phi = phi, psi = psi, r = r))
}

ballMapping <- function (phi, psi, r) {
    cbind(x = r * cos(phi) * sin(psi),
          y = r * sin(phi) * sin(psi),
          z = r * cos(psi))
}

sCurve <- function (n = 2000, sigma = 0.05) {
    t <- stats::runif(n, -1.5 * pi, 1.5 * pi)
    y <- stats::runif(n, 0, 2)

    new("dimRedData",
        data = sCurveMapping(t, y) + stats::rnorm(3 * n, sd = sigma),
        meta = data.frame(x = t, y = y))
}

sCurveMapping <- function (t, y) {
    cbind(x = sin(t),
          y = y,
          z = sign(t) * (cos(t) - 1))
}

noisyHelix <- function (n = 2000, sigma = 0.05) {
    t <- stats::runif(n, 0, 4 * pi)
    min_noise <- 0.1
    max_noise <- 1.4

    new("dimRedData",
        data = noisyHelixMapping(t, min_noise, max_noise) +
            stats::rnorm(3 * n, sd = sigma),
        meta = data.frame(t = t))
}

noisyHelixMapping <- function(t, min_noise, max_noise) {
    make_noise <- function (t){
        stats::rnorm(length(t), sd = t * max_noise / max(t) + min_noise)
    }

    cbind(x = 3 * cos(t) + make_noise(t),
          y = 3 * sin(t) + make_noise(t),
          z = 2 * t + make_noise(t))
}

cube <- function(n = 2000, sigma = 0.05){
    tmp <- cbind(x = stats::runif(n) + stats::rnorm(n, sd = sigma),
                 y = stats::runif(n) + stats::rnorm(n, sd = sigma),
                 z = stats::runif(n) + stats::rnorm(n, sd = sigma))

    new("dimRedData", data = tmp, meta = tmp)
}

test_that("quality", {

    irisData <- loadDataSet("Iris")

    parsPCA <- list(center = TRUE, scale. = TRUE)
    resPCA <- do.call(function(...) embed(irisData, "PCA", ...), parsPCA)

    if(requireNamespace("coRanking", quietly = TRUE))
      expect_true(is.numeric(Q_local(resPCA)))
    if(requireNamespace("coRanking", quietly = TRUE))
      expect_true(is.numeric(Q_global(resPCA)))
    if(requireNamespace("coRanking", quietly = TRUE))
      expect_true(is.numeric(mean_R_NX(resPCA)))
    if(requireNamespace("optimx", quietly = TRUE))
      expect_true(is.numeric(total_correlation(resPCA)))
    expect_true(is.numeric(cophenetic_correlation(resPCA)))
    if (requireNamespace("energy", quietly = TRUE))
      expect_true(is.numeric(distance_correlation(resPCA)))
    expect_true(is.numeric(reconstruction_rmse(resPCA)))

    ## suppressWarnings(
    ##     resQual <- list(
    ##         Q_local(resPCA),
    ##         Q_global(resPCA),
    ##         mean_R_NX(resPCA),
    ##         total_correlation(resPCA),
    ##         cophenetic_correlation(resPCA),
    ##         distance_correlation(resPCA),
    ##         reconstruction_rmse(resPCA)
    ##     )
    ## )

    ## lapply(resQual, function(x) expect_true(is.numeric(x)))
})

test_that("Q_local ndim", {
  if(!requireNamespace("coRanking", quietly = TRUE)) {
    skip("coRanking not available")
  }

  irisData <- loadDataSet("Iris")
  irisData <- irisData[!duplicated(irisData@data)]

  parsPCA <- list(center = TRUE, scale. = FALSE, ndim = 4)
  resPCA <- do.call(function(...) embed(irisData, "PCA", ...), parsPCA)

  tmp <- sapply(1:4, function(x) quality(resPCA, "Q_local", ndim = x))
  expect_equal(rank(tmp), 1:4)
})

test_that("rmse_by_ndim", {
  if(!requireNamespace("DRR", quietly = TRUE)) {
    skip("DRR not available")
  }

  set.seed(1)

  ir <- loadDataSet("Iris")
  ir.drr <- embed(ir, "DRR", .mute = c("message", "output"), ndim = ndims(ir))
  ir.pca <- embed(ir, "PCA", ndim = ndims(ir))

  rmse_res <- data.frame(
    drr = reconstruction_error(ir.drr),
    pca = reconstruction_error(ir.pca)
  )

  for (i in 1:length(rmse_res$pca)) {
    expect_true(rmse_res$pca[i] - rmse_res$drr[i] + 1e-12 > 0, info = paste0(
                                                           "ndim = ", i,
                                                           ", rmse pca = ", rmse_res$pca[i],
                                                           ", rmse drr = ", rmse_res$drr[i]
                                                         ))
  }
  # expect_true(all((rmse_res$pca - rmse_res$drr) + 1e-12 > 0))

  expect_error(reconstruction_error(ir.pca, 5))
  expect_error(reconstruction_error(ir.pca, 0))
})

test_that("AUC_lnK_R_NX", {
  if(!requireNamespace("coRanking", quietly = TRUE)) {
    skip("coRanking not available")
  }

  irisData <- loadDataSet("Iris")
  irisData <- irisData[!duplicated(irisData@data)]

  parsPCA <- list(center = TRUE, scale. = TRUE, ndim = 4)
  resPCA <- do.call(function(...) embed(irisData, "PCA", ...), parsPCA)

  expect_true(length(AUC_lnK_R_NX(resPCA, weight = "inv"))   == 1)
  expect_true(length(AUC_lnK_R_NX(resPCA, weight = "log"))   == 1)
  expect_true(length(AUC_lnK_R_NX(resPCA, weight = "ln"))    == 1)
  expect_true(length(AUC_lnK_R_NX(resPCA, weight = "log10")) == 1)

  expect_true(AUC_lnK_R_NX(resPCA, weight = "log") == AUC_lnK_R_NX(resPCA, weight = "ln"))
  expect_error(AUC_lnK_R_NX(resPCA, weight = "asdf"))

})

test_that("constructor", {
    expect_equal(dimRedData(), new("dimRedData",
                                   data = matrix(numeric(0),
                                                 nrow = 0, ncol = 0),
                                   meta = data.frame()))
    expect_error(dimRedData(iris))
    expect_s4_class(dimRedData(iris[, 1:4], iris[, 5]), "dimRedData")
    expect_s4_class(dimRedData(iris[, 1:4]), "dimRedData")
    expect_error(dimRedData(iris))
})

test_that("conversion functions", {
    expect_equal(as(iris[, 1:4], "dimRedData"), dimRedData(iris[, 1:4]))
    expect_error(as(iris, "dimRedData"))
    expect_equal(as(loadDataSet("Iris"), "data.frame"),
                 as.data.frame(loadDataSet("Iris")))
    expect_equal(as.dimRedData(
      Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width, iris),
      loadDataSet("Iris"),
      ignore_attr = TRUE
    )
})

test_that("misc functions", {
    Iris <- loadDataSet("Iris")
    expect_equal(getData(Iris), Iris@data)
    expect_equal(getMeta(Iris), Iris@meta)
    ## No idea why this one is broken with --run-donttest --run-dontrun --timings
    ## Also broken for devtools::test("dimRed")
    expect_equal(nrow(Iris), 150)
    expect_equal(Iris[1:4], Iris[1:4, ])
    expect_equal(Iris[1:4], Iris[c(rep(TRUE, 4), rep(FALSE, 146))])
    expect_equal(Iris[1:4], Iris[c(rep(TRUE, 4), rep(FALSE, 146)), ])
})

## no isomap specific tests, because forward method is not really
## exact.


test_that("check vs vegan isomap", {
  if (!requireNamespace(c("vegan", dimRed:::getMethodDependencies("Isomap")), quietly = TRUE)) {
    skip("Not all required packages for isomap tests installed")
  }

  eps <- 1e-8
  a <- loadDataSet("3D S Curve", n = 200)

  vegiso <- vegan::isomap(dist(getData(a)), k = 8, ndim = 2)
  vegy <- vegan::scores(vegiso)

  drdiso <- embed(a, "Isomap", knn = 8, ndim = 2)
  drdy <- drdiso@data@data

  ## Randomly fails:
  ## expect_equal(drdy, vegy, ignore_attr = TRUE)

  err1 <- max(abs(drdy - vegy))

  drdy[, 2] <- -drdy[, 2]
  err2 <- max(abs(drdy - vegy))

  drdy[, 1] <- -drdy[, 1]
  err3 <- max(abs(drdy - vegy))

  drdy[, 2] <- -drdy[, 2]
  err4 <- max(abs(drdy - vegy))

  err <- min(err1, err2, err3, err4)

  expect_true(err < eps, info = paste0("err = ", err,
                                       ", eps = ", eps,
                                       ", expected err < eps"))

})


test_that("check other.data", {
  if (!requireNamespace(dimRed:::getMethodDependencies("Isomap"), quietly = TRUE)) {
    skip("Not all required packages for isomap tests installed")
  }

  a <- loadDataSet("3D S Curve", n = 200)
  drdiso <- embed(a, "Isomap", knn = 8, ndim = 2, get_geod = TRUE)
  expect_true(inherits(getOtherData(drdiso)$geod, "dist"))
})

#' Isomap embedding
#'
#' An S4 Class implementing the Isomap Algorithm
#'
#' The Isomap algorithm approximates a manifold using geodesic
#' distances on a k nearest neighbor graph. Then classical scaling is
#' performed on the resulting distance matrix.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' Isomap can take the following parameters:
#' \describe{
#'   \item{knn}{The number of nearest neighbors in the graph. Defaults to 50.}
#'   \item{ndim}{The number of embedding dimensions, defaults to 2.}
#'   \item{get_geod}{Should the geodesic distance matrix be kept,
#'                   if \code{TRUE}, access it as \code{getOtherData(x)$geod}}
#' }
#'
#' @section Implementation:
#'
#' The dimRed package uses its own implementation of Isomap which also
#' comes with an out of sample extension (known as landmark
#' Isomap). The default Isomap algorithm scales computationally not
#' very well, the implementation here uses \code{\link[RANN]{nn2}} for
#' a faster search of the nearest neighbors.  If data are too large it
#' may be useful to fit a subsample of the data and use the
#' out-of-sample extension for the other points.
#'
#' @references
#' Tenenbaum, J.B., Silva, V. de, Langford, J.C., 2000. A Global Geometric
#' Framework for Nonlinear Dimensionality Reduction. Science 290, 2319-2323.
#' https://doi.org/10.1126/science.290.5500.2319
#'
#' @examples
#' if(requireNamespace(c("RSpectra", "igraph", "RANN"), quietly = TRUE)) {
#'
#' dat <- loadDataSet("3D S Curve", n = 500)
#' emb <- embed(dat, "Isomap", knn = 10)
#' plot(emb)
#'
#' ## or simpler, use embed():
#' samp <- sample(nrow(dat), size = 200)
#' emb2 <- embed(dat[samp], "Isomap", .mute = NULL, knn = 10)
#' emb3 <- predict(emb2, dat[-samp])
#'
#' plot(emb2, type = "2vars")
#' plot(emb3, type = "2vars")
#'
#' }
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export Isomap
#' @exportClass Isomap
Isomap <- setClass(
    "Isomap",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(knn = 50,
                       ndim = 2,
                       get_geod = FALSE),
        fun = function (data, pars,
                        keep.org.data = TRUE) {
        chckpkg("RSpectra")
        chckpkg("igraph")
        chckpkg("RANN")
        message(Sys.time(), ": Isomap START")
        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        indata <- data@data

        if (is.null(pars$eps))      pars$eps <- 0
        if (is.null(pars$get_geod)) pars$get_geod <- FALSE

        ## geodesic distances
        message(Sys.time(), ": constructing knn graph")
        knng <- makeKNNgraph(x = indata, k = pars$knn, eps = pars$eps)
        message(Sys.time(), ": calculating geodesic distances")
        geodist <- igraph::distances(knng, algorithm = "dijkstra")

        message(Sys.time(), ": Classical Scaling")
        ## TODO: add regularization
        k <- geodist ^ 2
        k <- .Call(stats:::C_DoubleCentre, k)
        k <- - k / 2
        ## TODO: explicit symmetrizing
        ## TODO: return eigenvectors?
        e <- RSpectra::eigs_sym(k, pars$ndim, which = "LA",
                                opts = list(retvec = TRUE))
        e_values <- e$values
        e_vectors <- e$vectors
        neig <- sum(e_values > 0)
        if (neig < pars$ndim) {
          warning("Isomap: eigenvalues < 0, returning less dimensions!")
          e_values <- e_values[seq_len(neig)]
          e_vectors <- e_vectors[, seq_len(neig), drop = FALSE]
        }

        e_vectors <- e_vectors * rep(sqrt(e_values), each = nrow(e_vectors))

        colnames(e_vectors) <- paste("iso", seq_len(neig))

        appl <- function (x) {
            message(Sys.time(), ": L-Isomap embed START")
            appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
            indata    <- if (inherits(x, "dimRedData")) x@data else x

            if (ncol(indata) != ncol(data@data))
                stop("x must have the same number of dimensions as the original data")

            nindata <- nrow(indata)
            norg <- nrow(orgdata)

            message(Sys.time(), ": constructing knn graph")
            lknng <- makeKNNgraph(rbind(indata, orgdata),
                                  k = pars$knn, eps = pars$eps)
            message(Sys.time(), ": calculating geodesic distances")
            lgeodist <- igraph::distances(lknng,
                                          seq_len(nindata),
                                          nindata + seq_len(norg))

            message(Sys.time(), ": embedding")
            dammu <- sweep(lgeodist ^ 2, 2, colMeans(geodist ^ 2), "-")
            Lsharp <- sweep(e_vectors, 2, e_values, "/")
            out <- -0.5 * (dammu %*% Lsharp)

            message(Sys.time(), ": DONE")
            return(new("dimRedData", data = out, meta = appl.meta))
        }

        return(new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = e_vectors,
                               meta = meta),
            org.data     = orgdata,
            has.org.data = keep.org.data,
            apply        = appl,
            has.apply    = TRUE,
            method       = "Isomap",
            pars         = pars,
            other.data   = if (pars$get_geod) list(geod = as.dist(geodist))
                           else               list()
        ))


        },
        requires = c("RSpectra", "igraph", "RANN")
    )
)


## input data(matrix or data frame) return knn graph implements
## "smart" choices on RANN::nn2 parameters we ignore radius search
## TODO: find out a good limit to switch from kd to bd trees COMMENT:
## bd trees are buggy, they dont work if there are duplicated data
## points and checking would neutralize the performance gain, so bd
## trees are not really usable.

makeKNNgraph <- function (x, k, eps = 0, diag = FALSE){
    ## requireNamespace("RANN")
    ## requireNamespace("igraph")

    ## consts
    INF_VAL <- 1.340781e+15
    NA_IDX  <- 0
    BDKD_LIM <- 1000000                 #todo: figure out a good value here

    ## select parameters
    M <- nrow(x)
    treetype <- "kd"                # if (M < BDKD_LIM) "kd" else "bd"
                                    # see:
                                    # https://github.com/jefferis/RANN/issues/19
    searchtype <- if (eps == 0) "standard" else "priority"

    ## RANN::nn2 returns the points in data with respect to query
    ## e.g. the rows in the output are the points in query and the
    ## columns the points in data.
    nn2res <- RANN::nn2(data = x, query = x, k = k + 1, treetype = treetype,
                        searchtype = searchtype, eps = eps)

    ## create graph: the first ny nodes will be y, the last nx nodes
    ## will be x, if x != y
    ## it is not really pretty to create a
    ## directed graph first and then make it undirected.
    g <- igraph::make_empty_graph(M, directed = TRUE)
    g[from = if (diag) rep(seq_len(M), times = k + 1)
             else      rep(seq_len(M), times = k),
      to   = if (diag) as.vector(nn2res$nn.idx)
             else      as.vector(nn2res$nn.idx[, -1]),
      attr = "weight"] <-
        if (diag)  as.vector(nn2res$nn.dists)
        else as.vector(nn2res$nn.dists[, -1])

    return(igraph::as.undirected(g, mode = "collapse", edge.attr.comb = "first"))
}

## the original isomap method I'll keep it here for completeness:
## isomap <- new("dimRedMethod",
##               stdpars = list(knn  = 50,
##                              d    = dist,
##                              ndim = 2)
##               fun = function (data, pars,
##                               keep.org.data = TRUE) {
##     chckpkg("vegan")

##     meta <- data@meta
##     orgdata <- if (keep.org.data) data@data else NULL
##     indata <- data@data

##     outdata <- vegan::isomap(pars$d(indata),
##                              ndim = pars$ndim,
##                              k = pars$knn)$points

##     colnames(outdata) <- paste0("Iso", 1:ncol(outdata))

##     return(new(
##         "dimRedResult",
##         data         = new("dimRedData",
##                            data = outdata,
##                            meta = meta),
##         org.data     = orgdata,
##         has.org.data = keep.org.data,
##         method       = "isomap",
##         pars         = pars
##     ))
## })

## #' Soft Ordinal Embedding
## #'
## #' Instance of \code{\link{dimRedMethod}} for Soft Ordinal Embedding.
## #'
## #' For details see \code{\link[loe]{SOE}}.
## #'
## #'
## #' @examples
## #' dat <- loadDataSet("3D S Curve", n = 50)
## #' soe <- SOE()
## #' emb <- soe@fun(dat, soe@stdpars)
## #'
## #'
## #' plot(emb@data@data)
## #'
## #'
## #' @include dimRedResult-class.R
## #' @include dimRedMethod-class.R
## #' @export
## SOE <- setClass(
##     "SOE",
##     contains = "dimRedMethod",
##     prototype = list(
##         stdpars = list(d = stats::dist, knn = 50, ndim = 2),
##         fun = function (data,
##                         pars,
##                         keep.org.data = TRUE) {
##         chckpkg("loe")

##         meta <- data@meta
##         orgdata <- if (keep.org.data) data@data else NULL
##         indata <- data@data

##         outdata <- loe::SOE(loe::get.order(as.matrix(pars$d(indata))),
##                             N = nrow(indata), p = pars$ndim)$X

##         colnames(outdata) <- paste0("SOE", 1:ncol(outdata))

##         return(new(
##             "dimRedResult",
##             data         = new("dimRedData",
##                                data = outdata,
##                                meta = meta),
##             org.data     = orgdata,
##             has.org.data = keep.org.data,
##             method       = "soe",
##             pars         = pars
##         ))
##     })
## )

#' @title The dimRed package
#'
#' @description This package simplifies dimensionality reduction in R by
#'   providing a framework of S4 classes and methods. dimRed collects
#'   dimensionality reduction methods that are implemented in R and implements
#'   others. It gives them a common interface and provides plotting
#'   functions for visualization and functions for quality assessment.
#'
#' Funding provided by the Department for Biogeochemical Integration,
#' Empirical Inference of the Earth System Group, at the Max Plack
#' Institute for Biogeochemistry, Jena.
#'
#' @references
#'
#' Lee, J.A., Renard, E., Bernard, G., Dupont, P., Verleysen, M.,
#' 2013. Type 1 and 2 mixtures of Kullback-Leibler divergences as cost
#' functions in dimensionality reduction based on similarity
#' preservation. Neurocomputing. 112,
#' 92-107. doi:10.1016/j.neucom.2012.12.036
#'
#' Lee, J.A., Lee, J.A., Verleysen, M., 2008. Rank-based quality
#' assessment of nonlinear dimensionality reduction. Proceedings of
#' ESANN 2008 49-54.
#'
#' Chen, L., Buja, A., 2006. Local Multidimensional Scaling for
#' Nonlinear Dimension Reduction, Graph Layout and Proximity Analysis.
#'
#'
#' @import methods
#' @importFrom magrittr %>%
#'
"_PACKAGE"

#' getRotationMatrix
#'
#' Extract the rotation matrix from \code{\link{dimRedResult}} objects derived from PCA and FastICA
#'
#' The data has to be pre-processed the same way as the method does, e.g.
#' centering and/or scaling.
#'
#' @param x of type \code{\link{dimRedResult}}
#' @return a matrix
#'
#' @examples
#' dat <- loadDataSet("Iris")
#'
#' pca <- embed(dat, "PCA")
#' rot_pca <- getRotationMatrix(pca)
#' scale(getData(dat), TRUE, FALSE) %*% rot_pca - getData(getDimRedData(pca))
#'
#'
#' if(requireNamespace("fastICA", quietly = TRUE)) {
#'   ica <- embed(dat, "FastICA")
#'   rot_ica <- getRotationMatrix(ica)
#'   scale(getData(dat), TRUE, FALSE) %*% rot_ica - getData(getDimRedData(ica))
#' }
#'
#'
#' @family convenience functions
#' @export
getRotationMatrix <- function(x) {
  if(!inherits(x, "dimRedResult")) stop("x must be of type 'dimRedResult'")
  if(x@method == "PCA")     return(environment(x@apply)$rot)
  if(x@method == "PCA_L1")  return(environment(x@apply)$rot)
  if(x@method == "FastICA") return(environment(x@apply)$res$K %*% environment(x@apply)$res$W)
  stop(paste("Not implemented for", x@method))
}

## this function produces segfaults and is super slow

## #' Local Ordinal Embedding
## #'
## #' Instance of \code{\link{dimRedMethod}} for Local Ordinal Embedding.
## #'
## #' For details see \code{\link[loe]{LOE}}
## #'
## #' @examples
## #' # for whatever reason the loe package has problems if I run this
## #' # with R CMD check, running it in the REPL works just fine
## #' if(requireNamespace("loe")) {
## #' dat <- loadDataSet("Iris")[sample(20)]
## #' loe <- LOE()
## #' emb <- loe@fun(dat, loe@stdpars)
## #'
## #'
## #' plot(emb@data@data)
## #' }
## #' @include dimRedResult-class.R
## #' @include dimRedMethod-class.R
## #' @export
## LOE <- setClass(
##     "LOE",
##     contains = "dimRedMethod",
##     prototype = list(
##         stdpars = list(d = stats::dist, knn = 50, ndim = 2),
##         fun = function (data, pars,
##                         keep.org.data = TRUE) {
##         chckpkg("loe")

##         meta <- data@meta
##         orgdata <- if (keep.org.data) data@data else NULL
##         indata <- data@data

##         data.adj <- loe:::make.kNNG(as.matrix(pars$d(indata)), k = pars$knn)
##         outdata <- loe::LOE(data.adj, p = pars$ndim, method = "MM")$X

##         colnames(outdata) <- paste0("LOE", 1:ncol(outdata))

##         return(new(
##             "dimRedResult",
##             data         = new("dimRedData",
##                                data = outdata,
##                                meta = meta),
##             org.data     = orgdata,
##             has.org.data = keep.org.data,
##             method       = "loe",
##             pars         = pars
##         ))
##     })
## )

#' Non-Negative Matrix Factorization
#'
#' S4 Class implementing NNMF.
#'
#' NNMF is a method for decomposing a matrix into a smaller
#'  dimension such that the constraint that the data (and the
#'  projection) are not negative is taken into account.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' The method can take the following parameters:
#' \describe{
#'   \item{ndim}{The number of output dimensions.}
#'   \item{method}{character, which algorithm should be used. See
#'    \code{\link[NMF]{nmf}} for possible values. Defaults to
#'    "brunet"}
#'   \item{nrun}{integer, the number of times the computations are
#'    conducted. See \code{\link[NMF]{nmf}}}
#'   \item{seed}{integer, a value to control the random numbers used.}
#'   \item{options}{named list, other options to pass to  \code{\link[NMF]{nmf}}}
#' }
#'
#' @section Implementation:
#'
#' Wraps around \code{\link[NMF]{nmf}}. Note that the estimation uses random
#'  numbers. To create reproducible results, set the random number seed in the
#'  function call. Also, in many cases, the computations will be conducted
#'  in parallel using multiple cores. To disable this, use the option
#'  \code{.pbackend = NULL}.
#'
#' @references
#'
#' Lee, D.D., Seung, H.S., 1999. Learning the parts of objects by non-negative
#' matrix factorization. Nature 401, 788-791. https://doi.org/10.1038/44565
#'
#' @examples
#' if(requireNamespace(c("NNMF", "MASS"), quietly = TRUE)) {
#'
#' set.seed(4646)
#' dat <- loadDataSet("Iris")
#' emb <- embed(dat, "NNMF")
#'
#' plot(emb)
#'
#' # project new values:
#' nn_proj <- predict(emb, dat[1:7])
#' plot(nn_proj)
#'
#' }
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export NNMF
#' @exportClass NNMF
NNMF <- setClass(
  "NNMF",
  contains = "dimRedMethod",
  prototype = list(
    stdpars = list(ndim = 2L,
                   method = "brunet",
                   nrun = 1,
                   seed = sample.int(10^5, 1),
                   options = list()),
    fun = function (data, pars, keep.org.data = TRUE) {
      chckpkg("NMF")
      chckpkg("MASS")
      ## TODO: remove this, depends on https://github.com/renozao/NMF/issues/114
      ## require("NMF")

      meta <- data@meta
      orgdata <- if (keep.org.data) data@data else NULL
      data <- data@data
      if (!is.matrix(data))
        data <- as.matrix(data)
      # NMF expects variables in rows and samples in columns
      data <- t(data)
      if (pars$ndim > nrow(data))
        stop("`ndim` should be less than the number of columns.",
             call. = FALSE)
      if (length(pars$method) != 1)
        stop("only supply one `method`", call. = FALSE)

      args <- list(x = quote(data), rank = pars$ndim, method = pars$method,
                   nrun = pars$nrun, seed = pars$seed)

      if (length(pars$options) > 0)
        args <- c(args, pars$options)

      nmf_result <- do.call(NMF::nmf, args)

      # this should work but doesn't
      # call <- c(list(quote(NMF::nmf)), args)

      w <- NMF::basis(nmf_result)
      h <- t(NMF::coef(nmf_result))

      colnames(w) <- paste0("NNMF", 1:ncol(w))

      other.data <- list(w = w)
      colnames(h) <- paste0("NNMF", 1:ncol(h))

      # evaluate results here for functions

      appl <- function (x) {
        appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
        dat <- if (inherits(x, "dimRedData")) x@data else x

        if (!is.matrix(dat))
          dat <- as.matrix(dat)

        if (ncol(dat) != nrow(w))
          stop("x must have the same number of columns ",
               "as the original data (", nrow(w), ")",
               call. = FALSE)

        res <- dat %*% t(MASS::ginv(w))

        colnames(res) <- paste0("NNMF", 1:ncol(res))

        scores <- new("dimRedData", data = res, meta = appl.meta)
        return(scores)
      }

      inv <- function (x) {
        appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
        proj <- if (inherits(x, "dimRedData")) x@data else x

        if (ncol(proj) > ncol(w))
          stop("x must have less or equal number of dimensions ",
               "as the original data")

        res <- tcrossprod(proj, w)
        colnames(res) <- colnames(data)

        res <- new("dimRedData", data = res, meta = appl.meta)
        return(res)
      }

      ## inv <- function(x) {
      ##   appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
      ##   proj <- if (inherits(x, "dimRedData")) x@data else x
      ##   if (ncol(proj) > ncol(data))
      ##     stop("x must have less or equal number of dimensions ",
      ##          "as the original data")

      ##   reproj <- proj %*% other.data$H
      ##   reproj <- new("dimRedData", data = reproj, meta = appl.meta)

      ##   return(reproj)
      ## }

      res <- new(
        "dimRedResult",
        data         = new("dimRedData",
                           data = h,
                           meta = meta),
        org.data     = orgdata,
        apply        = appl,
        inverse      = inv,
        has.org.data = keep.org.data,
        has.apply    = TRUE,
        has.inverse  = TRUE,
        method       = "NNMF",
        pars         = pars,
        other.data   = other.data
      )

      return(res)
    },
    requires = c("NMF", "MASS"))
)

## rotate X in such a way that the values of Y have maximum squared
## correlation with the dimensions specified in axes. We optimize
## axes[1] first, then axes[2] without axes[1], ...

## we maximize the squared correlations of the original variables
## with the axis of the embeding and the final result is the sum_{axes} sum(squared(correlation(variables, axis)))

setGeneric(
    "maximize_correlation",
    function(object, ...) standardGeneric("maximize_correlation"),
    valueClass = "dimRedResult"
)

#' Maximize Correlation with the Axes
#'
#' Rotates the data in such a way that the correlation with the first
#' \code{naxes} axes is maximized.
#'
#' Methods that do not use eigenvector decomposition, like t-SNE often
#' do not align the data with axes according to the correlation of
#' variables with the data. \code{maximize_correlation} uses the
#' \code{\link[optimx]{optimx}} package to rotate the data in such a
#' way that the original variables have maximum correlation with the
#' embedding axes.
#'
#' @param object A dimRedResult object
#' @param naxes the number of axes to optimize for.
#' @param cor_method which correlation method to use
#'
#' @aliases maximize_correlation
#' @export
setMethod(
    "maximize_correlation",
    "dimRedResult",
    function(object, naxes = ncol(object@data@data), cor_method = "pearson"){
        ## if (missing(naxes))      naxes      <- ncol(object@data@data)
        ## if (missing(cor_method)) cor_method <- "pearson"

        if (!object@has.org.data) stop("object requires original data")
        if (length(naxes) != 1 || naxes < 1 || naxes > ncol(object@data@data))
            stop("naxes must specify the numbers of axes to optimize for, ",
                 "i.e. a single integer between 1 and ncol(object@data@data)")
        ## try to partially match cor_method:
        cor_method <-
            cor_method[pmatch(cor_method, c("pearson", "kendall", "spearman"))]
        if (is.na(cor_method))
            stop("cor_method must match one of ",
                 "'pearson', 'kendall', or 'spearman', ",
                 "at least partially.")

        mcres <- .maximize_correlation(object@data@data,
                                       object@org.data,
                                       1:naxes,
                                       cor_method)

        res <- object
        res@data@data <- mcres$rotated
        return(res)
    }
)

.maximize_correlation <- function(X, Y,
                                  axes = 1:ncol(X),
                                  cor_method = "pearson"){

  if (nrow(X) != nrow(Y))
    stop("'X' and 'Y' must have the same number of rows")
  if (max(axes) > ncol(X)){
    axes <- axes[ axes <= ncol(X) ]
    warning("'max(axes)' must be <= 'ncol(X)', removing some axes")
  }

  chckpkg("optimx")

  xndim <- ncol(X)
  without_axes <- integer(0)
  res <- list()

  for (axis in axes){
    without_axes <- c(without_axes, axis)

    nplanes <- xndim - length(without_axes)
    planes <- matrix(NA, 2, nplanes)
    planes[1, ] <- axis
    planes[2, ] <- (1:xndim)[-without_axes]
    if (ncol(planes) == 0)
      break

    o <- optimx::optimx(
      par = rep(0, nplanes),
      fn = obj,
      method = "L-BFGS-B",
      lower = 0,
      upper = 2 * pi,
      X = as.matrix(X),
      Y = as.matrix(Y),
      axis = axis,
      without_axes = without_axes,
      cor_method = cor_method
    )
    ## The result looks like this:
    ##          p1      value fevals gevals niter convcode  kkt1 kkt2 xtimes
    ## L-BFGS-B  0 -0.1613494      1      1    NA        0 FALSE   NA  0.016

    if (o$convcode > 0) stop("rotation did not converge.")

    res_idx <- length(res) + 1
    res[[res_idx]] <- list()
    res[[res_idx]]$axis <- axis
    res[[res_idx]]$without_axes <- without_axes
    res[[res_idx]]$angs <- unname( unlist(o[1, 1:nplanes]) )
    res[[res_idx]]$planes <- planes
    res[[res_idx]]$X <- rotate(res[[res_idx]]$angs, planes, X)
    res[[res_idx]]$cor <- -o$value
  }

  ## calculate the correlation for axes
  nres <- length(res)
  if (nres > 0) {
    ## the result is the sum of the mean squared correlations of the
    ## original variables with the axes. "res[[i]]$cor" contains the
    ## mean squared correlation of the variables with axis "i"
    res$result <- 0
    for (i in 1:nres)
      res$result <- res$result + res[[i]]$cor ^ 2
    ## res$result <- res$result / length(res)

    ## rotate the input to maximize correlations
    res$rotated <- X
    for (i in 1:nres)
      res$rotated <- rotate(res[[i]]$angs, res[[i]]$planes, res$rotated)
  } else {
    ## if we only had one dimension, simply return the means squared
    ## correlation and don't rotate
    res$result <- sum(correlate(X, Y, cor_method) ^ 2)
    res$rotated <- X
  }

  res
}




#### helper functions for rotation

## we create a number or rotation matrices around the 2d planes
## spanned by the orthonormal matrices, multiply them for a general
## rotation which is then applied to the data X
rotate <- function (angs, planes, X) {
  ndim <- ncol(X)
  nplanes <- ncol(planes)
  if (length(angs) != nplanes)
    stop("length(angs) not equal to chose(ndim, 2)")

  ## loop over the planes to construct general rotation matrix
  rotmat <- diag(ndim)
  for (p in 1:nplanes) {
    ## 2d rotation
    ## possible optimization: create large rotation matrix
    ## directly and insert values linearly without a for loop
    rotmat2d <- matrix(
      c(cos(angs[p]), -sin(angs[p]),
        sin(angs[p]),  cos(angs[p])),
      2, 2, byrow = TRUE
    )
    p_rotmat <- diag(ndim)
    for (i in 1:2)
      for (j in 1:2)
        p_rotmat[ planes[i, p], planes[j, p] ] <- rotmat2d[i, j]
    rotmat <- rotmat %*% p_rotmat
  }

  t(rotmat %*% t(X))
}

get_planes <- function(ndims, axis, without_axes){
  nplanes <- ndims - length(without_axes)
  planes <- matrix(NA, 2, nplanes)
  planes[1, ] <- axis
  planes[2, ] <- (1:ndims)[c(-axis, -without_axes)]
  planes
}


obj <- function(alpha, X, Y, axis, without_axes, cor_method = "pearson"){
  ## correlation with first axis
  xndim <- ncol(X)

  planes <- get_planes(xndim, axis, without_axes)

  X2 <- rotate(alpha, planes, X)


  ## cor(x, y) returns a matrix with the correlations between the
  ## columns of x = X2 (rows) and the columns of y = Y (columns) we
  ## want the mean of squared correlations of all variables original
  ## variables with the first axis, i.e. we require the relevant
  ## (axis) column of the resulting matrix.

  ## Possible optimization: use only the relevant column of Y

  -mean(correlate(
    X2, Y,
    #use = "pairwise.complete.obs",
    method = cor_method
  )[axis, ] ^ 2)
}

correlate <- function (x, y, method, ...) {
  if (method != "kendall"){
      return(stats::cor(x, y, method = method, ...))
  } else {
      chckpkg("pcaPP")
      ## make the cor.fk method behave like cor for matrices:
      if (is.matrix(x) && is.matrix(y)) {
          res <- matrix(
              NA, nrow = ncol(x), ncol = ncol(y),
              dimnames = list(colnames(x), colnames(y))
          )
          for (i in 1:ncol(x)) {
              for (j in 1:ncol(y)){
                  res[i, j] <- pcaPP::cor.fk(x[, i], y[, j])
              }
          }
          return(res)
      } else if (is.null(dim(x)) && is.null(dim(y))){
          return(pcaPP::cor.fk(x, y))
      } else {
          stop("something is wrong with the input of 'correlate()'")
      }
  }
}

#' Class "dimRedMethod"
#'
#' A virtual class "dimRedMethod" to serve as a template to implement
#' methods for dimensionality reduction.
#'
#' Implementations of dimensionality reductions should inherit from
#' this class.
#'
#' The \code{fun} slot should be a function that takes three arguments
#' \describe{
#'   \item{data}{An object of class \code{\link{dimRedData}}.}
#'   \item{pars}{A list with the standard parameters.}
#'   \item{keep.org.data}{Logical. If the original data should be kept in the output.}
#' }
#' and returns an object of class \code{\link{dimRedResult}}.
#'
#' The \code{stdpars} slot should take a list that contains standard
#' parameters for the implemented methods.
#'
#' This way the method can be called by \code{embed(data, "method-name",
#' ...)}, where \code{...} can be used to to change single parameters.
#'
#'
#' @slot fun A function that does the embedding.
#' @slot stdpars A list with the default parameters for the \code{fun} slot.
#' @slot requires A vector with all packages R packages that need to be
#'   installed to run the method. In some occasions a method may work without
#'   one of the packages. Does not include Python dependencies such as
#'   Tensorflow. Used to auto skip tests
#'
#' @family dimensionality reduction methods
#' @export
setClass("dimRedMethod",
         contains  = "VIRTUAL",
         slots     = c(fun     = "function",
                       stdpars = "list",
                       requires = "character"))


#' dimRedMethodList
#'
#' Get the names of all methods for dimensionality reduction.
#'
#' Returns the name of all classes that inherit from
#' \code{\link{dimRedMethod-class}} to use with \code{\link{embed}}.
#' @param filter filter methods by methods that have their dependencies installed
#' @return a character vector with the names of classes that inherit
#'     from \code{dimRedMethod}.
#'
#' @examples
#' dimRedMethodList()
#'
#' @family dimensionality reduction methods
#' @export
dimRedMethodList <- function (filter = FALSE) {
   all_methods <- names(completeClassDefinition("dimRedMethod", doExtends = FALSE)@subclasses)
   if(!filter)
     return(all_methods)
   all_deps <- lapply(all_methods, getMethodDependencies)
   is_possible <- sapply(all_deps, function(x) {
     if(length(x) > 0)
       requireNamespace(x, quietly = TRUE)
     else
       TRUE
   })
   all_methods[is_possible]
}


# to put standard values for omitted arguments

setGeneric("matchPars", function(object, pars) standardGeneric("matchPars"),
           valueClass = c("list"))


setMethod("matchPars",
          signature(object = "dimRedMethod",
                    pars   = "list"),
          definition = function(object, pars) {
    nsp <- names(object@stdpars)
    ncp <- names(pars)
    nap <- union(nsp, ncp)

    res <- list()

    ## exists can deal with elements being NULL
    ## to assign list@el <- NULL do:
    ## list["el"] <- list(NULL)
    for (np in nap) {
        miss.std <- !exists(np, where = object@stdpars)
        miss.par <- !exists(np, where = pars)
        if (miss.std) {
            warning("Parameter matching: ", np,
                    " is not a standard parameter, ignoring.")
        } else if (miss.par) {
            res[np] <- object@stdpars[np]
        } else {
            res[np] <- pars[np]
        }
    }

    ## if the method does not accept parameters we have to return
    ## null, so in embed there is no args$par created. and passed by
    ## do.call in the embed() function.  if (length(res) != 0)
    ## return(res) else return(NULL)

    ## first try without the above, all methods should have a pars
    ## argument.
    return(res)
})

getMethodDependencies <- function(method) {
  getMethodObject(method)@requires
}

method_can_run <- function(method) {
  all(getMethodDependencies(method) %in% row.names(installed.packages()))
}

test_that("HLLE", {
  if(requireNamespace(dimRed:::getMethodDependencies("HLLE"), quietly = TRUE))
    expect_error(embed(iris[1:4], "HLLE", ndim = 1, .mute = c("message", "output")),
                 "ndim must be 2 or larger.")
})

test_that("pars matching", {
    for (m in dimRedMethodList()) {
        mo <- getMethodObject(m)
        expect(
            all.equal(
                mo@stdpars,
                matchPars(mo, list())
            ),
            paste("par matching for", m, "failed")
        )
    }

    expect_warning(
      embed(iris[1:4], "PCA", asdf = 1234),
      "Parameter matching: asdf is not a standard parameter, ignoring."
    )
})

#' Dimensionality Reduction via Regression
#'
#' An S4 Class implementing Dimensionality Reduction via Regression (DRR).
#'
#' DRR is a non-linear extension of PCA that uses Kernel Ridge regression.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' DRR can take the following parameters:
#' \describe{
#'   \item{ndim}{The number of dimensions}
#'   \item{lambda}{The regularization parameter for the ridge
#'   regression.}
#'   \item{kernel}{The kernel to use for KRR, defaults to
#'   \code{"rbfdot"}.}
#'   \item{kernel.pars}{A list with kernel parameters, elements depend
#'   on the kernel used, \code{"rbfdot"} uses \code{"sigma"}.}
#'   \item{pca}{logical, should an initial pca step be performed,
#'   defaults to \code{TRUE}.}
#'   \item{pca.center}{logical, should the data be centered before the
#'   pca step. Defaults to \code{TRUE}.}
#'   \item{pca.scale}{logical, should the data be scaled before the
#'   pca ste. Defaults to \code{FALSE}.}
#'   \item{fastcv}{logical, should \code{\link[CVST]{fastCV}} from the
#'   CVST package be used instead of normal cross-validation.}
#'   \item{fastcv.test}{If \code{fastcv = TRUE}, separate test data set for fastcv.}
#'   \item{cv.folds}{if \code{fastcv = FALSE}, specifies the number of
#'   folds for crossvalidation.}
#'   \item{fastkrr.nblocks}{integer, higher values sacrifice numerical
#'   accuracy for speed and less memory, see below for details.}
#'   \item{verbose}{logical, should the cross-validation results be
#'   printed out.}
#' }
#'
#' @section Implementation:
#' Wraps around \code{\link[DRR]{drr}}, see there for details. DRR is
#' a non-linear extension of principal components analysis using Kernel
#' Ridge Regression (KRR, details see \code{\link[CVST]{constructKRRLearner}}
#' and \code{\link[DRR]{constructFastKRRLearner}}). Non-linear
#' regression is used to explain more variance than PCA. DRR provides
#' an out-of-sample extension and a backward projection.
#'
#' The most expensive computations are matrix inversions therefore the
#' implementation profits a lot from a multithreaded BLAS library.
#' The best parameters for each KRR are determined by cross-validaton
#' over all parameter combinations of \code{lambda} and
#' \code{kernel.pars}, using less parameter values will speed up
#' computation time. Calculation of KRR can be accelerated by
#' increasing \code{fastkrr.nblocks}, it should be smaller than
#' n^{1/3} up to sacrificing some accuracy, for details see
#' \code{\link[DRR]{constructFastKRRLearner}}. Another way to speed up
#' is to use \code{pars$fastcv = TRUE} which might provide a more
#' efficient way to search the parameter space but may also miss the
#' global maximum, I have not ran tests on the accuracy of this method.
#'
#'
#'
#' @references
#' Laparra, V., Malo, J., Camps-Valls, G.,
#'     2015. Dimensionality Reduction via Regression in Hyperspectral
#'     Imagery. IEEE Journal of Selected Topics in Signal Processing
#'     9, 1026-1036. doi:10.1109/JSTSP.2015.2417833
#'
#' @examples
#' \dontrun{
#' if(requireNamespace(c("kernlab", "DRR"), quietly = TRUE)) {
#'
#' dat <- loadDataSet("variable Noise Helix", n = 200)[sample(200)]
#'
#' emb <- embed(dat, "DRR", ndim = 3)
#'
#' plot(dat, type = "3vars")
#' plot(emb, type = "3vars")
#'
#' # We even have function to reconstruct, also working for only the first few dimensions
#' rec <- inverse(emb, getData(getDimRedData(emb))[, 1, drop = FALSE])
#' plot(rec, type = "3vars")
#' }
#'
#' }
#'
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @import DRR
#' @family dimensionality reduction methods
#' @export DRR
#' @exportClass DRR
DRR <- setClass(
    "DRR",
    contains  = "dimRedMethod",
    prototype = list(
        stdpars = list(ndim            = 2,
                       lambda          = c(0, 10 ^ (-3:2)),
                       kernel          = "rbfdot",
                       kernel.pars     = list(sigma = 10 ^ (-3:4)),
                       pca             = TRUE,
                       pca.center      = TRUE,
                       pca.scale       = FALSE,
                       fastcv          = FALSE,
                       cv.folds        = 5,
                       fastcv.test     = NULL,
                       fastkrr.nblocks = 4,
                       verbose         = TRUE),
        fun = function (data, pars,
                        keep.org.data = TRUE) {
        chckpkg("DRR")
        chckpkg("kernlab")


        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        indata <- data@data

        res <- do.call(DRR::drr, c(list(X = indata), pars))

        outdata <- res$fitted.data
        colnames(outdata) <- paste0("DRR", 1:ncol(outdata))

        appl <- function(x){
            appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
            proj <- if (inherits(x, "dimRedData")) x@data else x

            if (ncol(proj) != ncol(data@data))
                stop("x must have the same number of dimensions ",
                     "as the original data")

            appl.out <- new("dimRedData",
                            data = res$apply(proj),
                            meta = appl.meta)
            dimnames(appl.out@data) <- list(
                rownames(x), paste0("DRR", seq_len(ncol(appl.out@data)))
            )
            return(appl.out)
        }

        inv <- function(x) {
            appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
            proj <- if (inherits(x, "dimRedData")) x@data else x

            if (ncol(proj) > ncol(data@data))
                stop("x must have less or equal number of dimensions ",
                     "as the original data")

            inv.out <- new("dimRedData",
                           data = res$inverse(proj),
                           meta = appl.meta)
            dimnames(inv.out@data) <- list(rownames(proj), colnames(data@data))
            return(inv.out)
        }


        return(
            new("dimRedResult",
                data = new("dimRedData",
                           data = outdata,
                           meta = meta),
                org.data = orgdata,
                apply = appl,
                inverse = inv,
                has.org.data = keep.org.data,
                has.apply = TRUE,
                has.inverse = TRUE,
                method = "drr",
                pars = pars
                )
        )
        },
      requires = c("DRR", "kernlab"))
)

#' Metric Dimensional Scaling
#'
#' An S4 Class implementing classical scaling (MDS).
#'
#' MDS tries to maintain distances in high- and low-dimensional space,
#' it has the advantage over PCA that arbitrary distance functions can
#' be used, but it is computationally more demanding.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' MDS can take the following parameters:
#' \describe{
#'   \item{ndim}{The number of dimensions.}
#'   \item{d}{The function to calculate the distance matrix from the input coordinates, defaults to euclidean distances.}
#' }
#'
#' @section Implementation:
#'
#' Wraps around \code{\link[stats]{cmdscale}}. The implementation also
#' provides an out-of-sample extension which is not completely
#' optimized yet.
#'
#' @references
#'
#' Torgerson, W.S., 1952. Multidimensional scaling: I. Theory and method.
#' Psychometrika 17, 401-419. https://doi.org/10.1007/BF02288916
#'
#' @examples
#' \dontrun{
#' dat <- loadDataSet("3D S Curve")
#' emb <- embed(dat, "MDS")
#' plot(emb, type = "2vars")
#'
#' # a "manual" kPCA:
#' emb2 <- embed(dat, "MDS", d = function(x) exp(stats::dist(x)))
#' plot(emb2, type = "2vars")
#'
#' # a "manual", more customizable, and slower Isomap:
#' emb3 <- embed(dat, "MDS", d = function(x) vegan::isomapdist(vegan::vegdist(x, "manhattan"), k = 20))
#' plot(emb3)
#'
#' }
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export MDS
#' @exportClass MDS
MDS <- setClass(
    "MDS",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(d = stats::dist, ndim = 2),
        fun = function (data, pars,
                        keep.org.data = TRUE) {
        ##
        meta <- data@meta
        orgdata <- if (keep.org.data) data@data else NULL
        indata <- data@data

        ## there are only efficient implementations for euclidean
        ## distances: extra efficient implementation for euclidean
        ## distances are possible, D is quared several times, it would be
        ## much faster to compute the squared distance right away.
        has.apply <- identical(all.equal(pars$d, dist), TRUE) # == TRUE
                                        # necessary,
                                        # because
                                        # all.equal
                                        # returns
                                        # TRUE or an
                                        # error
                                        # string!!!!

        D <- as.matrix(pars$d(indata))
        if (has.apply) mD2 <- mean(D ^ 2)

        ## cmdscale square the matrix internally
        res <- stats::cmdscale(D, k = pars$ndim)
        outdata <- res

        D <- NULL
        ## Untested: remove that from environment before creating
        ## appl function, else it will stay in its environment
        ## forever

        appl <- if (!has.apply) function(x) NA else function(x) {
            appl.meta <- if (inherits(x, "dimRedData")) x@meta else data.frame()
            proj <- if (inherits(x, "dimRedData")) x@data else x

            ## double center new data with respect to old: TODO: optimize
            ## this method, according to the de Silva, Tenenbaum(2004)
            ## paper. Need an efficient method to calculate the distance
            ## matrices between different point sets and arbitrary
            ## distances.
            Kab <- as.matrix(pars$d(proj) ^ 2)
            Exa <- colMeans(pdist2(indata, proj))
            Kab <- sweep(Kab, 1, Exa)       #, "-")
            Kab <- sweep(Kab, 2, Exa)       #, "-")
            Kab <- -0.5 * (Kab + mD2)

            ## Eigenvalue decomposition
            tmp  <- eigen(Kab, symmetric = TRUE)
            ev   <- tmp$values[seq_len(pars$ndim)]
            evec <- tmp$vectors[, seq_len(pars$ndim), drop = FALSE]

            k1 <- sum(ev > 0)
            if (k1 < pars$ndim) {
                warning(gettextf("only %d of the first %d eigenvalues are > 0",
                                 k1, k), domain = NA)
                evec <- evec[, ev > 0, drop = FALSE]
                ev <- ev[ev > 0]
            }
            points <- evec * rep(sqrt(ev), each = nrow(proj))
            dimnames(points) <- list(NULL, paste0("MDS", seq_len(ncol(points))))

            new("dimRedData", data = points, meta = appl.meta)
        }



        colnames(outdata) <- paste0("MDS", seq_len(ncol(outdata)))

        return(new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = outdata,
                               meta = meta),
            org.data     = orgdata,
            apply        = appl,
            has.org.data = keep.org.data,
            has.apply    = has.apply,
            method       = "mds",
            pars         = pars
        ))
    })
)

#' Hessian Locally Linear Embedding
#'
#' An S4 Class implementing Hessian Locally Linear Embedding (HLLE)
#'
#' HLLE uses local hessians to approximate the curvines and is an
#' extension to non-convex subsets in lowdimensional space.
#'
#' @template dimRedMethodSlots
#'
#' @template dimRedMethodGeneralUsage
#'
#' @section Parameters:
#' HLLE can take the following parameters:
#' \describe{
#'   \item{knn}{neighborhood size}
#'   \item{ndim}{number of output dimensions}
#' }
#'
#' @section Implementation:
#' Own implementation, sticks to the algorithm in Donoho and Grimes
#' (2003). Makes use of sparsity to speed up final embedding.
#'
#' @references
#' Donoho, D.L., Grimes, C., 2003. Hessian eigenmaps: Locally linear
#' embedding techniques for high-dimensional data. PNAS 100,
#' 5591-5596. doi:10.1073/pnas.1031596100
#'
#' @examples
#' if(requireNamespace(c("RSpectra", "Matrix", "RANN"), quietly = TRUE)) {
#'
#' dat <- loadDataSet("3D S Curve", n = 300)
#' emb <- embed(dat, "HLLE", knn = 15)
#' plot(emb, type = "2vars")
#'
#' }
#'
#' @include dimRedResult-class.R
#' @include dimRedMethod-class.R
#' @family dimensionality reduction methods
#' @export HLLE
#' @exportClass HLLE
HLLE <- setClass(
    "HLLE",
    contains = "dimRedMethod",
    prototype = list(
        stdpars = list(knn = 50, ndim = 2),
        fun = function(data, pars,
                       keep.org.data = TRUE) {
        chckpkg("RSpectra")
        chckpkg("Matrix")
        chckpkg("RANN")

        if (pars$ndim < 2) stop("ndim must be 2 or larger.")

        if (is.null(pars$knn))   pars$knn  <- 50
        if (is.null(pars$ndim))  pars$ndim <- 2

        indata <- data@data
        n <- nrow(indata)
        hs <- pars$ndim * (pars$ndim + 1) / 2
        W <- Matrix::sparseMatrix(i = numeric(0),
                                  j = numeric(0),
                                  x = numeric(0),
                                  dims = c(n, hs * n))
        ii <- jj <- ww <- list()
        ## Identify neighbors:
        message(Sys.time(), ": Finding nearest neighbors", sep = "")
        nnidx <- RANN::nn2(data = indata, query = indata, k = pars$knn + 1,
                           treetype = "kd", "standard", eps = 0)$nn.idx#[, -1]
        message(Sys.time(), ": Calculating Hessian", sep = "")
        for (i in seq_len(n)) {
            cat(i, "/", n, "\r", sep = "")
            ## get neighborhood
            Nui <- indata[nnidx[i, ], , drop = FALSE]

            ## Form tangent coordinates:
            Nui <- sweep(Nui, 2, colMeans(Nui), "-")
            tc <- svd(Nui, nu = pars$ndim, nv = 0)$u

            ## Develop Hessian Estimator
            Xi <- cbind(
                1, tc, tc ^ 2,
                apply(combn(seq_len(pars$ndim), 2), 2,
                      function(x) tc[, x[1]] * tc[, x[2]])
            )
            tHi <- qr.Q(qr(Xi))[, -(1:(pars$ndim + 1)),
                                drop = FALSE]

            ## Add quadratic form to hessian
            ii[[i]] <- rep(nnidx[i, ], hs)
            jj[[i]] <- rep((i - 1) * hs + (1:hs), each = ncol(nnidx))
            ww[[i]] <- as.vector(tHi)
        }
        H <- as(Matrix::tcrossprod(Matrix::spMatrix(
            i = unlist(ii, FALSE, FALSE),
            j = unlist(jj, FALSE, FALSE),
            x = unlist(ww, FALSE, FALSE),
            nrow = n, ncol = n * hs)
        ), "dgCMatrix")

        ## Find null space:
        message(Sys.time(), ": Embedding", sep = "")
        ## eigs and eigs_sym converges much more reliably and faster
        ## with sigma = -eps than with which = "L*"
        outdata <- RSpectra::eigs_sym(H, k = pars$ndim + 1, sigma = -1e-5)

        message(paste(c("Eigenvalues:", format(outdata$values)),
                      collapse = " "))
        outdata <- outdata$vectors[, order(outdata$values)[-1], drop = FALSE]

        colnames(outdata) <- paste0("HLLE", seq_len(ncol(outdata)))

        message(Sys.time(), ": DONE", sep = "")
        return(new(
            "dimRedResult",
            data         = new("dimRedData",
                               data = outdata,
                               meta = data@meta),
            org.data     = if (keep.org.data) data@data else NULL,
            has.org.data = keep.org.data,
            method       = "HLLE",
            pars         = pars
        ))
        },
        requires = c("RSpectra", "Matrix", "RANN"))
)

#' Plotting of dimRed* objects
#'
#' Plots a object of class dimRedResult and dimRedData. For the
#' documentation of the plotting function in base see here:
#' \code{\link{plot.default}}.
#'
#' Plotting functions for the classes usind in \code{dimRed}. they are
#' intended to give a quick overview over the results, so they are
#' somewhat inflexible, e.g. it is hard to modify color scales or
#' plotting parameters.
#'
#' If you require more control over plotting, it is better to convert
#' the object to a \code{data.frame} first and use the standard
#' functions for plotting.
#'
#' @param x dimRedResult/dimRedData class, e.g. output of
#'     embedded/loadDataSet
#' @param y Ignored
#' @param type plot type, one of \code{c("pairs", "parpl", "2vars",
#'     "3vars", "3varsrgl")}
#' @param col the columns of the meta slot to use for coloring, can be
#'     referenced as the column names or number of x@data
#' @param vars the axes of the embedding to use for plotting
#' @param ... handed over to the underlying plotting function.
#'
#' @examples
#' scurve = loadDataSet("3D S Curve")
#' if(requireNamespace("graphics", quietly = TRUE))
#'   plot(scurve, type = "pairs", main = "pairs plot of S curve")
#' if(requireNamespace("MASS", quietly = TRUE))
#'   plot(scurve, type = "parpl")
#' if(requireNamespace("graphics", quietly = TRUE))
#'   plot(scurve, type = "2vars", vars = c("y", "z"))
#' if(requireNamespace("scatterplot3d", quietly = TRUE))
#'   plot(scurve, type = "3vars")
#' if(requireNamespace("rgl", quietly = TRUE))
#'   plot(scurve, type = "3varsrgl")
#'
#' @include mixColorSpaces.R
#' @include dimRedData-class.R
#' @importFrom graphics plot
#'
#' @aliases plot.dimRed
#' @export
setGeneric(
    "plot", function(x, y, ...) standardGeneric("plot"),
    useAsDefault = graphics::plot
)

#' @describeIn plot Ploting of dimRedData objects
#' @aliases plot.dimRedData
#' @export
setMethod(
    f = "plot",
    signature = c("dimRedData"),
    definition = function(x, type = "pairs",
                          vars = seq_len(ncol(x@data)),
                          col = seq_len(min(3, ncol(x@meta))), ...) {
        cols <- colorize(x@meta[, col, drop = FALSE])
        switch(
            type,
            "pairs"    = {
                chckpkg("graphics")
                graphics::pairs(x@data[, vars],      col = cols,   ... )
            },
            "parpl"    = {
                chckpkg("MASS")
                MASS::parcoord(x@data[, vars],      col = cols,   ... )
            },
            "2vars"    = {
                chckpkg("graphics")
                graphics::plot(x@data[, vars[1:2]], col = cols,   ... )
            },
            "3vars"    = {
                chckpkg("scatterplot3d")
                scatterplot3d::scatterplot3d(x@data[, vars[1:3]],
                                             color = cols,
                                             ...)
            },
            "3varsrgl" = {
                chckpkg("rgl")
                rgl::plot3d(x@data[, vars[1:3]], col = cols,   ... )
            },
            stop("wrong argument to plot.dimRedData")
        )
    }
)


#' @describeIn plot Ploting of dimRedResult objects.
#' @aliases plot.dimRedResult
#' @export
setMethod(
    f = "plot",
    signature = c("dimRedResult"),
    definition = function (x, type = "pairs",
                           vars = seq_len(ncol(x@data@data)),
                           col = seq_len(min(3, ncol(x@data@meta))), ...) {
        plot(x = x@data, type = type, vars = vars, col = col, ...)
    }
)

#' plot_R_NX
#'
#' Plot the R_NX curve for different embeddings. Takes a list of
#' \code{\link{dimRedResult}} objects as input.
#' Also the Area under the curve values are computed for a weighted K
#' (see \link{AUC_lnK_R_NX} for details) and appear in the legend.
#'
#' @param x a list of \code{\link{dimRedResult}} objects. The names of the list
#'   will appear in the legend with the AUC_lnK value.
#' @param ndim the number of dimensions, if \code{NA} the original number of
#'   embedding dimensions is used, can be a vector giving the embedding
#'   dimensionality for each single list element of \code{x}.
#' @param weight the weight function used for K when calculating the AUC, one of
#'   \code{c("inv", "log", "log10")}
#' @family Quality scores for dimensionality reduction
#' @return A ggplot object, the design can be changed by appending
#'   \code{theme(...)}
#'
#' @examples
#' if(requireNamespace(c("RSpectra", "igraph", "RANN", "ggplot", "tidyr", "scales"), quietly = TRUE)) {
#' ## define which methods to apply
#' embed_methods <- c("Isomap", "PCA")
#' ## load test data set
#' data_set <- loadDataSet("3D S Curve", n = 200)
#' ## apply dimensionality reduction
#' data_emb <- lapply(embed_methods, function(x) embed(data_set, x))
#' names(data_emb) <- embed_methods
#' ## plot the R_NX curves:
#' plot_R_NX(data_emb) +
#'     ggplot2::theme(legend.title = ggplot2::element_blank(),
#'                    legend.position = c(0.5, 0.1),
#'                    legend.justification = c(0.5, 0.1))
#' }
#' @export
plot_R_NX <- function(x, ndim = NA, weight = "inv") {
    chckpkg("ggplot2")
    chckpkg("tidyr")
    chckpkg("scales")
    lapply(
        x,
        function(x)
        if (!inherits(x, "dimRedResult"))
            stop("x must be a list and ",
                 "all items must inherit from 'dimRedResult'")
    )

    rnx <- mapply(function(x, ndim) if(is.na(ndim)) R_NX(x) else R_NX(x, ndim),
                  x = x, ndim = ndim)

    weight <- match.arg(weight, c("inv", "ln", "log", "log10"))
    w_fun <- switch(
      weight,
      inv   = auc_ln_k_inv,
      log   = auc_log_k,
      ln    = auc_log_k,
      log10 = auc_log10_k,
      stop("wrong parameter for weight")
    )
    auc <- apply(rnx, 2, w_fun)

    df <- as.data.frame(rnx)
    df$K <- seq_len(nrow(df))

    qnxgrid <- expand.grid(K = df$K,
                           rnx = seq(0.1, 0.9, by = 0.1))
    ## TODO: FIND OUT WHY THIS AS IN THE PUBLICATION BUT IS WRONG!
    qnxgrid$qnx <- rnx2qnx(qnxgrid$rnx, K = qnxgrid$K, N = nrow(df)) #
    qnxgrid$rnx_group <- factor(qnxgrid$rnx)

    df <- tidyr::gather_(df,
                         key_col = "embedding",
                         value_col = "R_NX",
                         names(x))

    ggplot2::ggplot(df) +
        ggplot2::geom_line(ggplot2::aes_string(y = "R_NX", x = "K",
                                               color = "embedding")) +
        ## TODO: find out if this is wrong:
        ## ggplot2::geom_line(data = qnxgrid,
        ##                    mapping = ggplot2::aes_string(x = "K", y = "qnx",
        ##                                                  group = "rnx_group"),
        ##                    linetype = 2,
        ##                    size = 0.1) +
        ggplot2::geom_line(data = qnxgrid,
                           mapping = ggplot2::aes_string(x = "K", y = "rnx",
                                                         group = "rnx_group"),
                           linetype = 3,
                           size = 0.1) +
        ggplot2::scale_x_log10(
            labels = scales::trans_format("log10",
                                          scales::math_format()),
            expand = c(0, 0)
        ) +
        ggplot2::scale_y_continuous(expression(R[NX]),
                                    limits = c(0, 1),
                                    expand = c(0, 0)) +
        ggplot2::annotation_logticks(sides = "b") +
        ggplot2::scale_color_discrete(
                     breaks = names(x),
                     labels = paste(format(auc, digits = 3),
                                    names(x))) +
        ggplot2::labs(title = paste0(
                        "R_NX vs. K",
                        if (length(ndim) == 1 && !is.na(ndim))
                          paste0(", d = ", ndim)
                        else
                          ""
                      )) +
        ggplot2::theme_classic()
}

#' dispatches the different methods for dimensionality reduction
#'
#' wraps around all dimensionality reduction functions.
#'
#' Method must be one of \code{\link{dimRedMethodList}()}, partial matching
#' is performed. All parameters start with a dot, to avoid clashes
#' with partial argument matching (see the R manual section 4.3.2), if
#' there should ever occur any clashes in the arguments, call the
#' function with all arguments named, e.g. \code{embed(.data = dat,
#' .method = "mymethod", .d = "some parameter")}.
#'
#' @param .data object of class \code{\link{dimRedData}}, will be converted to
#'   be of class \code{\link{dimRedData}} if necessary; see examples for
#'   details.
#' @param .method character vector naming one of the dimensionality reduction
#'   techniques.
#' @param .mute a character vector containing the elements you want to mute
#'   (\code{c("message", "output")}), defaults to \code{character(0)}.
#' @param .keep.org.data \code{TRUE}/\code{FALSE} keep the original data.
#' @param ... the parameters, internally passed as a list to the dimensionality
#'   reduction method as \code{pars = list(...)}
#' @return an object of class \code{\link{dimRedResult}}
#'
#' @examples
#' ## embed a data.frame using a formula:
#' as.data.frame(
#'   embed(Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,
#'         iris, "PCA")
#' )
#'
#' ## embed a data.frame and return a data.frame
#' as.data.frame(embed(iris[, 1:4], "PCA"))
#'
#' ## embed a matrix and return a data.frame
#' as.data.frame(embed(as.matrix(iris[, 1:4]), "PCA"))
#'
#' \dontrun{
#' ## embed dimRedData objects
#' embed_methods <- dimRedMethodList()
#' quality_methods <- dimRedQualityList()
#' dataset <- loadDataSet("Iris")
#'
#' quality_results <- matrix(NA, length(embed_methods), length(quality_methods),
#'                               dimnames = list(embed_methods, quality_methods))
#' embedded_data <- list()
#'
#' for (e in embed_methods) {
#'   message("embedding: ", e)
#'   embedded_data[[e]] <- embed(dataset, e, .mute = c("message", "output"))
#'   for (q in quality_methods) {
#'     message("  quality: ", q)
#'     quality_results[e, q] <- tryCatch(
#'       quality(embedded_data[[e]], q),
#'       error = function(e) NA
#'     )
#'   }
#' }
#'
#' print(quality_results)
#' }
#' @export
setGeneric("embed", function(.data, ...) standardGeneric("embed"),
           valueClass = "dimRedResult")

#' @describeIn embed embed a data.frame using a formula.
#' @param .formula a formula, see \code{\link{as.dimRedData}}.
#' @export
setMethod(
    "embed",
    "formula",
    function(.formula, .data, .method = dimRedMethodList(),
             .mute = character(0), .keep.org.data = TRUE,
             ...) {
        if (!is.data.frame(.data)) stop(".data must be a data.frame")

        .data <- as.dimRedData(.formula, .data)
        embed(.data, .method, .mute, .keep.org.data, ...)
    }
)

#' @describeIn embed Embed anything as long as it can be coerced to
#'     \code{\link{dimRedData}}.
#' @export
setMethod(
    "embed",
    "ANY",
    function(.data, .method = dimRedMethodList(),
             .mute = character(0), .keep.org.data = TRUE,
             ...) {
        embed(as(.data, "dimRedData"), .method, .mute, .keep.org.data, ...)
    }
)

#' @describeIn embed Embed a dimRedData object
#' @export
setMethod(
    "embed",
    "dimRedData",
    function(.data, .method = dimRedMethodList(),
             .mute = character(0), #c("message", "output"),
             .keep.org.data = TRUE,
             ...) {
      .method <- if (all(.method == dimRedMethodList())) "PCA"
                 else match.arg(.method)

        methodObject <- getMethodObject(.method)

        args <- list(
            data          = as(.data, "dimRedData"),
            keep.org.data = .keep.org.data
        )
        args$pars <- matchPars(methodObject, list(...))

        devnull <- if (Sys.info()["sysname"] != "Windows")
                       "/dev/null"
                   else
                       "NUL"
        if ("message" %in% .mute){
            devnull1 <- file(devnull,  "wt")
            sink(devnull1, type = "message")
            on.exit({
                sink(file = NULL, type = "message")
                close(devnull1)
            }, add = TRUE)
        }
        if ("output" %in% .mute) {
            devnull2 <- file(devnull,  "wt")
            sink(devnull2, type = "output")
            on.exit({
                sink()
                close(devnull2)
            }, add = TRUE)
        }

        do.call(methodObject@fun, args)
    }
)

getMethodObject <- function (method) {
    ## switch(
    ##     method,
    ##     graph_kk  = kamada_kawai,
    ##     graph_drl = drl,
    ##     graph_fr  = fruchterman_reingold,
    ##     drr       = drr,
    ##     isomap    = isomap,
    ##     diffmap   = diffmap,
    ##     tsne      = tsne,
    ##     nmds      = nmds,
    ##     mds       = mds,
    ##     ica       = fastica,
    ##     pca       = pca,
    ##     lle       = lle,
    ##     loe       = loe,
    ##     soe       = soe,
    ##     leim      = leim,
    ##     kpca      = kpca
    ## )
    method <- match.arg(method, dimRedMethodList())
    do.call(method, list())
}

#' @include misc.R
NULL


#' Class "dimRedData"
#'
#' A class to hold data for dimensionality reduction and methods.
#'
#' The class hast two slots, \code{data} and \code{meta}. The
#' \code{data} slot contains a \code{numeric matrix} with variables in
#' columns and observations in rows. The \code{meta} slot may contain
#' a \code{data.frame} with additional information. Both slots need to
#' have the same number of rows or the \code{meta} slot needs to
#' contain an empty \code{data.frame}.
#'
#' See examples for easy conversion from and to \code{data.frame}.
#'
#' For plotting functions see \code{\link{plot.dimRedData}}.
#'
#' @slot data of class \code{matrix}, holds the data, observations in
#'     rows, variables in columns
#' @slot meta of class \code{data.frame}, holds meta data such as
#'     classes, internal manifold coordinates, or simply additional
#'     data of the data set. Must have the same number of rows as the
#'     \code{data} slot or be an empty data frame.
#'
#'
#' @examples
#' ## Load an example data set:
#' s3d <- loadDataSet("3D S Curve")
#'
#' ## Create using a constructor:
#'
#' ### without meta information:
#' dimRedData(iris[, 1:4])
#'
#' ### with meta information:
#' dimRedData(iris[, 1:4], iris[, 5])
#'
#' ### using slot names:
#' dimRedData(data = iris[, 1:4], meta = iris[, 5])
#'
#' ## Convert to a dimRedData objects:
#' Iris <- as(iris[, 1:4], "dimRedData")
#'
#' ## Convert to data.frame:
#' head(as(s3d, "data.frame"))
#' head(as.data.frame(s3d))
#' head(as.data.frame(as(iris[, 1:4], "dimRedData")))
#'
#' ## Extract slots:
#' head(getData(s3d))
#' head(getMeta(s3d))
#'
#' ## Get the number of observations:
#' nrow(s3d)
#'
#' ## Subset:
#' s3d[1:5, ]
#'
#' @family dimRedData
#' @import methods
#' @export dimRedData
#' @exportClass dimRedData
dimRedData <- setClass(
    "dimRedData",
    slots     = c(data = "matrix", meta = "data.frame"),
    prototype = prototype(data = matrix(numeric(0), 0, 0), meta = data.frame()),
    validity  = function (object) {
        retval <- NULL
        if (!is.matrix(object@data)) {
            retval <- c(
                retval,
                c("data must be a matrix with ",
                  "observations in rows and dimensions in columns")
            )
        }
        if (!is.numeric(object@data)) {
            retval <- c(
                retval,
                c("data must be numeric")
            )
        }
        if ((nrow(object@meta) != 0) &&
            (nrow(object@meta) != nrow(object@data))){
            retval <- c(
                retval,
                c("data and meta must have the same numbers of rows")
            )
        }

        return(if (is.null(retval)) TRUE else retval)
    }
)

setMethod("initialize",
          signature = c("dimRedData"),
          function (.Object,
                    data = matrix(numeric(0), 0, 0),
                    meta = data.frame()) {
    data <- as.matrix(data)
    meta <- as.data.frame(meta)
    .Object <- callNextMethod()
    return(.Object)
})


setAs(from = "ANY", to = "dimRedData",
      def = function(from) new("dimRedData", data = as.matrix(from)))

setAs(from = "dimRedData", to = "data.frame",
      def = function(from) as.data.frame(from))

#' @param meta.prefix Prefix for the columns of the meta data names.
#' @param data.prefix Prefix for the columns of the variable names.
#'
#' @family dimRedData
#' @describeIn dimRedData convert to data.frame
#' @export
setMethod(f = "as.data.frame",
          signature = c("dimRedData"),
          definition = function(x, meta.prefix = "meta.",
                                data.prefix = "") {
    tmp <- list()

    if (nrow(x@meta) > 0){
        tmp$meta <- as.data.frame(x@meta, stringsAsFactors = FALSE)
        names(tmp$meta) <- paste0(meta.prefix, colnames(x@meta))
    }
    tmp$data <- as.data.frame(x@data, stringsAsFactors = FALSE)
    names(tmp$data) <- paste0(data.prefix, colnames(x@data))
    names(tmp) <- NULL
    data.frame(tmp, stringsAsFactors = FALSE)
})


#' @param data Will be coerced into a \code{\link{data.frame}} with
#'   \code{\link{as.data.frame}}
#'
#' @examples
#' ## create a dimRedData object using a formula
#' as.dimRedData(Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,
#'               iris)[1:5]
#'
#' @include misc.R
#' @family dimRedData
#' @describeIn as.dimRedData Convert a \code{data.frame} to a dimRedData
#'     object using a formula
#' @export
setMethod(f = "as.dimRedData",
          signature = c("formula"),
          definition = function(formula, data) {
    data <- as.data.frame(data)
    meta <- stats::model.frame(lhs(formula), data)
    data <- stats::model.matrix(rhs(formula), data)
    return(new("dimRedData", data = data, meta = meta))
})



#' @param object Of class dimRedData.
#' @describeIn dimRedData Get the data slot.
#' @export
setMethod("getData", "dimRedData", function(object) object@data)


#' @describeIn dimRedData Get the meta slot.
#' @export
setMethod("getMeta", "dimRedData", function(object) object@meta)

#' @param x Of class dimRedData
#' @describeIn dimRedData Get the number of observations.
#' @export
setMethod("nrow", "dimRedData", function(x) nrow(x@data))

#' @param i a valid index for subsetting rows.
#' @examples
#' ## Shuffle data:
#' s3 <- s3d[nrow(s3d)]
#'
#' @describeIn dimRedData Subset rows.
#' @export
setMethod("[", signature(x = "dimRedData",
                         i = "ANY"),
          function(x, i) {
    x@data <- x@data[i, , drop = FALSE]
    if (nrow(x@meta) != 0)
        x@meta <- x@meta[i, , drop = FALSE]
    # validObject returns a string with the description of what is wrong or
    # TRUE, so the following lines have to be as they are!
    vv <- validObject(x)
    if (vv == TRUE) return(x)
    else stop("cannot subset dimRedData object: \n",
              paste(vv, collapse = "\n"))
})


#' @describeIn dimRedData Extract the number of Variables from the data.
#'
#' @examples
#' ## Get the number of variables:
#' ndims(s3d)
#'
#' @export
setMethod("ndims", "dimRedData", function(object) ncol(object@data))

#' Mixing color ramps
#'
#' mix different color ramps
#'
#' automatically create colors to represent a varying number of
#' dimensions.
#'
#' @param vars a list of variables
#' @param ramps a list of color ramps, one for each variable.
#'
#' @examples
#' cols <- expand.grid(x = seq(0, 1, length.out = 10),
#'                     y = seq(0, 1, length.out = 10),
#'                     z = seq(0, 1, length.out = 10))
#' mixed <- mixColor3Ramps(cols)
#'
#' \dontrun{
#' if(requireNamespace("rgl", quietly = TRUE)) {
#' rgl::plot3d(cols$x, cols$y, cols$z, col = mixed, pch = 15)
#' }
#'
#' cols <- expand.grid(x = seq(0, 1, length.out = 10),
#'                     y = seq(0, 1, length.out = 10))
#' mixed <- mixColor2Ramps(cols)
#'
#' if(requireNamespace("graphics", quietly = TRUE)) {
#' plot(cols$x, cols$y, col = mixed, pch = 15)
#' }
#' }
#' @importFrom grDevices colorRamp
#' @importFrom grDevices rgb
#' @export
mixColorRamps <- function (vars, ramps) {
    if (length(vars) > length(ramps)) stop("need more or equal ramps than vars")

    nvars <- length(vars)

    rgbs <- list()
    for (i in 1:nvars){
        rgbs[[i]] <- ramps[[i]](scale01(as.numeric(vars[[i]])))
    }

    retrgb <- Reduce(`+`, rgbs)

    res <- apply(retrgb, 2,  function(x) (x - min(x)) / (max(x) - min(x)))
    res[is.nan(res)] <- 0

    return(rgb(res))
}

#' @rdname mixColorRamps
#' @export
mixColor1Ramps <- function (vars,
                            ramps = colorRamp(c("blue", "black", "red"))) {
    mixColorRamps(vars, list(ramps))
}

#' @rdname mixColorRamps
#' @export
mixColor2Ramps <- function (vars,
                            ramps = list(colorRamp(c("blue", "green")),
                                         colorRamp(c("blue", "red")))) {
    mixColorRamps(vars, ramps)
}

#' @rdname mixColorRamps
#' @export
mixColor3Ramps <- function (vars,
                           ramps = list(colorRamp(c("#001A00", "#00E600")),
                                        colorRamp(c("#00001A", "#0000E6")),
                                        colorRamp(c("#1A0000", "#E60000")))) {
    mixColorRamps(vars, ramps)
}


colorize <- function (vars) {
    l <- length(vars)
    if (l == 1) return(mixColor1Ramps(vars))
    if (l == 2) return(mixColor2Ramps(vars))
    if (l == 3) return(mixColor3Ramps(vars))
    return("#000000")
}

scale01 <- function(x,
                    low = min(x, na.rm = TRUE),
                    high = max(x, na.rm = FALSE)) {
    x <- (x - low) / (high - low)
    x
}

test_that("DiffusionMaps", {
  if(!requireNamespace("diffusionMap", quietly = TRUE))
    skip("diffusionMap not available")
  expect_s4_class(embed(iris[1:4], "DiffusionMaps", ndim = 1,
                        .mute = c("message", "output")),
                  "dimRedResult")
  x <- embed(iris[1:4], "DiffusionMaps", ndim = 1,
             .mute = c("message", "output"))
  expect_equal(dim(x@data@data), c(150, 1))
})

test_that("standard method is PCA", {

  res <- embed(iris[1:4])
  expect_equal(res@method, "PCA")

})

test_that("correctly convert .keep.org.data argument", {
  res1 <- embed(Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,
                iris, "PCA", .keep.org.data = FALSE)


  res2 <- embed(Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,
                iris, "PCA", .keep.org.data = TRUE)

  attr(res2@org.data, "assign") <- NULL

  expect_equal(
    unname(as.matrix(iris[1:4])),
    unname(as.matrix(getOrgData(res2)@data))
  )

})

## skip_if_no_tensorflow <- function() {
##   if (!requireNamespace(c("tensorflow", "reticulate"), quietly = TRUE) ||
##         (!reticulate::py_module_available("tensorflow") &&
##          Sys.getenv("BNET_FORCE_AUTOENCODER_TESTS") != "1"))
##     skip("TensorFlow not available for testing")
## }
## skip_if_no_keras <- function() {
##   if (!requireNamespace(c("keras3", "reticulate"), quietly = TRUE) ||
##         (!keras::is_keras_available() &&
##            Sys.getenv("BNET_FORCE_AUTOENCODER_TESTS") != "1"))
##     skip("Keras not available for testing")
## }

## test_that("Check if tensorflow is installed correctly.", {
##   skip_if_no_tensorflow()
##   requireNamespace("tensorflow", quietly = TRUE)
##   tensorflow::tf$compat$v1$disable_v2_behavior()
##   # I have not found a way to suppress the warning tf gives on first use.
##   sess <- tensorflow::tf$compat$v1$Session()
##   hello <- "Hello, TensorFlow!"
##   tf_hello <- tensorflow::tf$compat$v1$constant(hello)
##   tf_hello_res <- sess$run(tf_hello)

##   # in python 3 this returns a `bytes` object $decode() transforms it into a
##   # sting, in python 2 this is a simple string
##   if(!is.character(tf_hello_res))
##     tf_hello_res <- tf_hello_res$decode()

##   ## print("tf_hello_res:")
##   ## print(str(tf_hello_res))
##   ## print(tf_hello_res)

##   expect(tf_hello_res == hello, paste("tensorflow does not work:\n",
##                                       "hello =", hello, "\n",
##                                       "sess$run(tf_hello) =", tf_hello_res))
## })

## test_that("Check errors when building autoencoder.", {
##   skip_if_no_tensorflow()
##   iris_data <- as(iris[, 1:4], "dimRedData")
##   expect_error(embed(iris_data, "AutoEncoder", activation = "sigmoid"),
##                "declare an activation function for each layer")
##   expect_error(embed(iris_data, "AutoEncoder", n_hidden = c(1, 2, 2, 1)),
##                "the number of layers must be impair")
##   expect_error(embed(iris_data, "AutoEncoder", weight_decay = -1),
##                "weight decay must be > 0")
##   expect_error(embed(iris_data, "AutoEncoder", learning_rate = -1),
##                "learning rate must be > 0")
##   expect_error(embed(iris_data, "AutoEncoder", n_steps = -1),
##                "n_steps must be > 0")
##   expect_error(embed(iris_data, "AutoEncoder", n_hidden = c(4, 2, 4), ndim = 3),
##                "the middle of n_hidden must be equal to ndim")
## })


## test_that("using autoencoder with parameters", {
##     skip_if_no_tensorflow()
##     iris_data <- as(iris[, 1:4], "dimRedData")
##     expect_equal(class(iris_data)[1], "dimRedData")

##     ae <- lapply(1:2, function(x) embed(iris_data, "AutoEncoder",
##                                         n_hidden = c(10, x, 10),
##                                         ndim = x,
##                                         n_steps = 100))
##     aq <- lapply(ae, function(x) quality(x, "reconstruction_rmse"))
##     lapply(ae, function(x) expect_s4_class(x, "dimRedResult"))

##     ## expect(aq[[1]] > aq[[2]], "the error should decrease with more dimensions")
##     ## expect(aq[[2]] > aq[[3]], "the error should decrease with more dimensions")
##     ## expect(aq[[3]] > aq[[4]], "the error should decrease with more dimensions")

##     lapply(1:length(ae), function(x) expect_equal(x, getNDim(ae[[x]])))


##     ae <- lapply(1:2, function(x) embed(iris_data,
##                                         "AutoEncoder",
##                                         n_hidden = c(10, x, 10),
##                                         ndim = x,
##                                         weight_decay = 0.1,
##                                         n_steps = 100))
##     aq <- lapply(ae, function(x) quality(x, "reconstruction_rmse"))
##     lapply(ae, function(x) expect_s4_class(x, "dimRedResult"))

##     ## expect(aq[[1]] > aq[[2]], "the error should decrease with more dimensions")
##     ## expect(aq[[2]] > aq[[3]], "the error should decrease with more dimensions")
##     ## expect(aq[[3]] > aq[[4]], "the error should decrease with more dimensions")

##     lapply(1:length(ae), function(x) expect_equal(x, getNDim(ae[[x]])))



##     ae <- lapply(1:2, function(x) embed(iris_data,
##                                         "AutoEncoder",
##                                         n_hidden = c(10, x, 10),
##                                         ndim = x,
##                                         learning_rate = 0.1,
##                                         weight_decay = 0.1,
##                                         n_steps = 100))
##     aq <- lapply(ae, function(x) quality(x, "reconstruction_rmse"))
##     lapply(ae, function(x) expect_s4_class(x, "dimRedResult"))

##     ## expect(aq[[1]] > aq[[2]], "the error should decrease with more dimensions")
##     ## expect(aq[[2]] > aq[[3]], "the error should decrease with more dimensions")
##     ## expect(aq[[3]] > aq[[4]], "the error should decrease with more dimensions")

##     lapply(1:length(ae), function(x) expect_equal(x, getNDim(ae[[x]])))



##     ae <- lapply(1:2, function(x) embed(iris_data,
##                                         "AutoEncoder",
##                                         n_hidden = c(10, x, 10),
##                                         activation = c("sigmoid", "sigmoid", "sigmoid"),
##                                         ndim = x,
##                                         learning_rate = 0.1,
##                                         weight_decay = 0.1,
##                                         n_steps = 100))
##     aq <- lapply(ae, function(x) quality(x, "reconstruction_rmse"))
##     lapply(ae, function(x) expect_s4_class(x, "dimRedResult"))

##     aa <- lapply(c("tanh", "sigmoid", "relu", "elu"),
##                  function(x) embed(iris_data,
##                                    "AutoEncoder",
##                                    n_hidden = c(10, 2, 10),
##                                    activation = c("sigmoid", "sigmoid", "sigmoid"),
##                                    ndim = 2,
##                                    learning_rate = 0.1,
##                                    weight_decay = 0.1,
##                                    n_steps = 100))
##     aaq <- lapply(aa, function(x) quality(x, "reconstruction_rmse"))
##     lapply(aa, function(x) expect_s4_class(x, "dimRedResult"))
##     ## expect(aq[[1]] > aq[[2]], "the error should decrease with more dimensions")
##     ## expect(aq[[2]] > aq[[3]], "the error should decrease with more dimensions")
##     ## expect(aq[[3]] > aq[[4]], "the error should decrease with more dimensions")

##     lapply(1:length(ae), function(x) expect_equal(x, getNDim(ae[[x]])))
## })

## test_that("using autoencoder with autoencoder results", {
##     skip_if_no_tensorflow()

##     tensorflow::tf$compat$v1$set_random_seed(2)
##     iris_data <- as(iris[, 1:4], "dimRedData")
##     expect_equal(class(iris_data)[1], "dimRedData")

##     ae1 <- lapply(1:2, function(x) embed(iris_data, "AutoEncoder",
##                                          n_hidden = c(10, x, 10),
##                                          ndim = x, n_steps = 1))
##     aq1 <- lapply(ae1, function(x) quality(x, "reconstruction_rmse"))

##     ae2 <- lapply(ae1, function(x) embed(iris_data, "AutoEncoder",
##                                          autoencoder = x, n_steps = 1000))
##     aq2 <- lapply(ae2, function(x) quality(x, "reconstruction_rmse"))

##     lapply(ae1, function(x) expect_s4_class(x, "dimRedResult"))
##     lapply(ae2, function(x) expect_s4_class(x, "dimRedResult"))

##     expect(aq1[[1]] > aq2[[1]], "the error should decrease with more steps")
##     expect(aq1[[2]] > aq2[[2]], "the error should decrease with more steps")
##     ## expect(aq1[[3]] > aq2[[3]], "the error should decrease with more steps")
##     ## expect(aq1[[4]] > aq2[[4]], "the error should decrease with more steps")

##     lapply(1:length(ae1), function(x) expect_equal(x, getNDim(ae1[[x]])))
##     lapply(1:length(ae2), function(x) expect_equal(x, getNDim(ae2[[x]])))
## })

## test_that("using autoencoder with keras", {
##   skip_if_no_tensorflow()
##   skip_if_no_keras()

##   encoder <- function(i) list(keras::layer_dense(units = 10,
##                                                  activation = "tanh"),
##                               keras::layer_dense(units = i))
##   decoder <- function() list(keras::layer_dense(units = 10,
##                                                 activation = "tanh"),
##                              keras::layer_dense(units = 4))

##   iris_data <- as(iris[, 1:4], "dimRedData")

##   ae1 <- lapply(1:2, function(x) embed(iris_data, "AutoEncoder",
##                                        keras_graph = list(encoder = encoder(x),
##                                                           decoder = decoder()),
##                                        n_steps = 2))
##   aq1 <- lapply(ae1, function(x) quality(x, "reconstruction_rmse"))

##   ae2 <- lapply(ae1, function(x) embed(iris_data, "AutoEncoder",
##                                        autoencoder = x))
##   aq2 <- lapply(ae2, function(x) quality(x, "reconstruction_rmse"))

##   lapply(ae1, function(x) expect_s4_class(x, "dimRedResult"))
##   lapply(ae2, function(x) expect_s4_class(x, "dimRedResult"))

##   ## expect(aq1[[1]] > aq2[[1]], "the error should decrease with more steps")
##   ## expect(aq1[[2]] > aq2[[2]], "the error should decrease with more steps")
##   ## expect(aq1[[3]] > aq2[[3]], "the error should decrease with more steps")
##   ## expect(aq1[[4]] > aq2[[4]], "the error should decrease with more steps")

##   lapply(1:length(ae1), function(x) expect_equal(x, getNDim(ae1[[x]])))
##   lapply(1:length(ae2), function(x) expect_equal(x, getNDim(ae2[[x]])))

## })



## ## test_that("garbage collection", {
## ##   skip_if_no_tensorflow()
## ##   tmp <- tf$get_session_handle(environment(ae[[1]]@apply)$dec)

## ##   tmp <- tf$get_default_session()

## ##   tmp$close
## ##   tmp
## ##   tf$get_session_handle()
## ##   tf$Session()
## ## })

context("UMAP")

skip_if_no_umap_learn <- function() {
  if (!reticulate::py_module_available("umap") &&
      Sys.getenv("BNET_FORCE_UMAP_TESTS") != 1)
    skip("umap-learn not available, install with `pip install umap-learn`")
}

test_that("UMAP python", {
  skip_if_no_umap_learn()
  res <- embed(iris[1:4], "UMAP", .mute = c("message", "output"))
  expect_s4_class(res, "dimRedResult")
  expect_equal(res@method, "UMAP")
  expect_equal(res@pars$d, "euclidean")
  expect_equal(res@pars$knn, 15)
  expect_equal(res@pars$method, "umap-learn")
  expect_equal(res@pars$ndim, 2)
})

test_that("UMAP R", {
  res <- embed(iris[1:4], "UMAP", method = "naive", .mute = c("message", "output"))
  expect_s4_class(res, "dimRedResult")
  expect_equal(res@method, "UMAP")
  expect_equal(res@pars$d, "euclidean")
  expect_equal(res@pars$knn, 15)
  expect_equal(res@pars$method, "naive")
  expect_equal(res@pars$ndim, 2)
})

skip_if_no_umap_learn <- function() {
  if (!reticulate::py_module_available("umap") &&
      Sys.getenv("BNET_FORCE_UMAP_TESTS") != 1)
    skip("umap-learn not available, install with `pip install umap-learn==0.4`")
}

test_that("UMAP python", {
  if (!requireNamespace("umap", quietly = TRUE)) {
    skip("umap not available")
  }

  skip_if_no_umap_learn()

  res1 <- embed(iris[1:4], "UMAP", .mute = c("message", "output"))
  res2 <- embed(iris[1:4], "UMAP", .mute = c("message", "output"), knn = 20)

  expect_s4_class(res1, "dimRedResult")
  expect_equal(res1@method, "UMAP")
  expect_equal(res1@pars$d, "euclidean")
  expect_equal(res1@pars$knn, 15)
  expect_equal(res1@pars$method, "umap-learn")
  expect_equal(res1@pars$ndim, 2)

  expect_s4_class(res2, "dimRedResult")
  expect_equal(res2@method, "UMAP")
  expect_equal(res2@pars$d, "euclidean")
  expect_equal(res2@pars$knn, 20)
  expect_equal(res2@pars$method, "umap-learn")
  expect_equal(res2@pars$ndim, 2)


  expect_true(any(res1@data@data != res2@data@data))

  pred1 <- predict(res1, iris[1:4])
  pred2 <- predict(res2, iris[1:4])

  expect_equal(dim(pred1@data), dim(res1@data@data))
  expect_equal(dim(pred2@data), dim(res2@data@data))
})

test_that("UMAP R", {
  if (!requireNamespace("umap", quietly = TRUE)) {
    skip("umap not available")
  }

  res1 <- embed(iris[1:4], "UMAP", method = "naive", .mute = c("message", "output"))
  res2 <- embed(iris[1:4], "UMAP", method = "naive", .mute = c("message", "output"), knn = 20)

  expect_s4_class(res1, "dimRedResult")
  expect_equal(res1@method, "UMAP")
  expect_equal(res1@pars$d, "euclidean")
  expect_equal(res1@pars$knn, 15)
  expect_equal(res1@pars$method, "naive")
  expect_equal(res1@pars$ndim, 2)

  expect_s4_class(res2, "dimRedResult")
  expect_equal(res2@method, "UMAP")
  expect_equal(res2@pars$d, "euclidean")
  expect_equal(res2@pars$knn, 20)
  expect_equal(res2@pars$method, "naive")
  expect_equal(res2@pars$ndim, 2)

  expect_true(any(res1@data@data != res2@data@data))

  pred1 <- predict(res1, iris[1:4])
  pred2 <- predict(res2, iris[1:4])

  expect_equal(dim(pred1@data), dim(res1@data@data))
  expect_equal(dim(pred2@data), dim(res2@data@data))
})

\documentclass{article}  

%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Dimensionality Reduction}
%\VignetteKeyword{Dimensionality Reduction}

\usepackage[utf8]{inputenc}
\usepackage[T1]{fontenc}

\usepackage{hyperref}
\usepackage{amsmath,amssymb}
\usepackage{booktabs}
\usepackage{tikz}
\usetikzlibrary{trees}
\usepackage[sectionbib,round]{natbib}

\title{\pkg{dimRed} and \pkg{coRanking}---Unifying Dimensionality Reduction in R}
\author{Guido Kraemer \and Markus Reichstein \and Miguel D.\ Mahecha}

% these are taken from RJournal.sty:
\makeatletter
\DeclareRobustCommand\code{\bgroup\@noligs\@codex}
\def\@codex#1{\texorpdfstring%
{{\normalfont\ttfamily\hyphenchar\font=-1 #1}}%
{#1}\egroup}
\newcommand{\kbd}[1]{{\normalfont\texttt{#1}}}
\newcommand{\key}[1]{{\normalfont\texttt{\uppercase{#1}}}}
\DeclareRobustCommand\samp{`\bgroup\@noligs\@sampx}
\def\@sampx#1{{\normalfont\texttt{#1}}\egroup'}
\newcommand{\var}[1]{{\normalfont\textsl{#1}}}
\let\env=\code
\newcommand{\file}[1]{{`\normalfont\textsf{#1}'}}
\let\command=\code
\let\option=\samp
\newcommand{\dfn}[1]{{\normalfont\textsl{#1}}}
% \acronym is effectively disabled since not used consistently
\newcommand{\acronym}[1]{#1}
\newcommand{\strong}[1]{\texorpdfstring%
{{\normalfont\fontseries{b}\selectfont #1}}%
{#1}}
\let\pkg=\strong
\newcommand{\CRANpkg}[1]{\href{https://CRAN.R-project.org/package=#1}{\pkg{#1}}}%
\let\cpkg=\CRANpkg
\newcommand{\ctv}[1]{\href{https://CRAN.R-project.org/view=#1}{\emph{#1}}}
\newcommand{\BIOpkg}[1]{\href{https://www.bioconductor.org/packages/release/bioc/html/#1.html}{\pkg{#1}}}
\makeatother

\begin{document}

\maketitle

\abstract{ %

  This document is based on the manuscript of \citet{kraemer_dimred_2018} which
  was published in the R-Journal and has been modified and extended to fit the
  format of a package vignette and to match the extended functionality of the
  \pkg{dimRed} package.

  ``Dimensionality reduction'' (DR) is a widely used approach to find low
  dimensional and interpretable representations of data that are natively
  embedded in high-dimensional spaces. %
  DR can be realized by a plethora of methods with different properties,
  objectives, and, hence, (dis)advantages. The resulting low-dimensional data
  embeddings are often difficult to compare with objective criteria. %
  Here, we introduce the \CRANpkg{dimRed} and \CRANpkg{coRanking} packages for
  the R language. %
  These open source software packages enable users to easily access multiple
  classical and advanced DR methods using a common interface. %
  The packages also provide quality indicators for the embeddings and easy
  visualization of high dimensional data. %
  The \pkg{coRanking} package provides the functionality for assessing DR methods in the
  co-ranking matrix framework. %
  In tandem, these packages allow for uncovering complex structures high
  dimensional data. %
  Currently 15 DR methods are available in the package, some of which were not
  previously available to R users. %
  Here, we outline the \pkg{dimRed} and \pkg{coRanking} packages and
  make the implemented methods understandable to the interested reader. %
}

\section{Introduction}
\label{sec:intro}

Dimensionality Reduction (DR) essentially aims to find low dimensional
representations of data while preserving their key properties. %
Many methods exist in literature, optimizing different criteria: %
maximizing the variance or the statistical independence of the projected data, %
minimizing the reconstruction error under different constraints, %
or optimizing for different error metrics, %
just to name a few. %
Choosing an inadequate method may imply that much of the underlying structure
remains undiscovered. %
Often the structures of interest in a data set can be well represented by fewer
dimensions than exist in the original data. %
Data compression of this kind has the additional benefit of making the encoded
information better conceivable to our brains  for further analysis tasks
like classification or regression problems. %

For example, the morphology of a plant's leaves, stems, and seeds reflect the
environmental conditions the species usually grow in (e.g.,\ plants with large
soft leaves will never grow in a desert but might have an advantage in a humid
and shadowy environment). %
Because the morphology of the entire plant depends on the environment, many
morphological combinations will never occur in nature and the morphological
space of all plant species is tightly constrained. %
\citet{diaz_global_2016} found that out of six observed morphological characteristics
only two embedding dimensions were enough to represent three quarters of the totally
observed variability. %

DR is a widely used approach for the detection of structure in multivariate
data, and has applications in a variety of fields. %
In climatology, DR is used to find the modes of some phenomenon, e.g.,\ the first
Empirical Orthogonal Function of monthly mean sea surface temperature of a given
region over the Pacific is often linked to the El Ni\~no Southern
Oscillation or
ENSO \citep[e.g.,\ ][]{hsieh_nonlinear_2004}. %
In ecology the comparison of sites with different species abundances is a
classical multivariate problem: each observed species adds an extra dimension,
and because species are often bound to certain habitats, there is a lot of
redundant information. Using DR is a popular technique to represent the sites in
few dimensions, e.g.,\ \citet{aart_distribution_1972} matches wolfspider
communities to habitat and \citet{morrall_soil_1974} match soil fungi data to
soil types. (In ecology the general name for DR is ordination or indirect
gradient analysis.) %
Today, hyperspectral satellite imagery collects so many bands that it is very
difficult to analyze and interpret the data directly. %
Resuming the data into a set of few, yet independent, components is one way to
reduce complexity \citep[e.g.,\ see][]{laparra_dimensionality_2015}. %
DR can also be used to visualize the interiors of deep neural networks
\citep[e.g.,\ see ][]{han_deep_2016}, where the high dimensionality comes from
the large number of weights used in a neural network and convergence can be
visualized by means of DR\@. %
We could find many more example applications here but this is not the main focus
of this publication. %

The difficulty in applying DR is that each DR method is designed to maintain
certain aspects of the original data and therefore may be appropriate for one
task and inappropriate for another. %
Most methods also have parameters to tune and follow different assumptions. The
quality of the outcome may strongly depend on their tuning, which adds
additional complexity. %
DR methods can be modeled after physical models with attracting and repelling
forces (Force Directed Methods), projections onto low dimensional planes (PCA,
ICA), divergence of statistical distributions (SNE family), or the reconstruction
of local spaces or points by their neighbors (LLE). %

As an example for how changing internal parameters of a method can have a great
impact, the breakthrough for Stochastic Neighborhood Embedding (SNE) methods
came when a Student's $t$-distribution was used instead of a normal distribution
to model probabilities in low dimensional space to avoid the ``crowding
problem'', that is,\ a sphere in high dimensional space has a much larger volume
than in low dimensional space and may contain too many points to be represented
accurately in few dimensions. %
The $t$-distribution, allows medium distances to be accurately represented in
few dimensions by larger distances due to its heavier tails. %
The result is called in $t$-SNE and is especially good at preserving local
structures in very few dimensions, this feature made $t$-SNE useful for a wide
array of data visualization tasks and the method became much more popular than
standard SNE (around six times more citations of
\citet{van_der_maaten_visualizing_2008} compared to
\citet{hinton_stochastic_2003} in Scopus \citep{noauthor_scopus_nodate}). %

There are a number of software packages for other languages providing collections of methods: In
Python there is scikit-learn \citep{scikit-learn}, which contains a module for
DR. In Julia we currently find ManifoldLearning.jl for nonlinear and
MultivariateStats.jl for linear DR methods. %
There are several toolboxes for DR implemented in Matlab
\citep{van_der_maaten_dimensionality_2009,
  arenas-garcia_kernel_2013}. The Shogun
toolbox \citep{soeren_sonnenburg_2017_1067840} implements a variety of methods for
dimensionality reduction in C++ and offers bindings for a many common high level
languages (including R, but the installation is anything but simple, as
there is no CRAN package). %
However, there is no comprehensive package for R and none of the former
mentioned software packages provides means to consistently compare the quality
of different methods for DR. %

For many applications it can be difficult to objectively find the right method
or parameterization for the DR task. %
This paper presents the \pkg{dimRed} and \pkg{coRanking} packages for
the popular programming language R. Together, they 
provide a standardized interface to various dimensionality reduction methods and quality
metrics for embeddings. They are implemented using the S4 class system
of R, making the packages
both easy to use and to extend.

The design goal for these packages is to enable researchers, who may not necessarily be experts in DR, to
apply the methods in their own work and to objectively identify the
most suitable
methods for their data. %
This paper provides an overview of the methods collected in the
packages and contains examples as to how
to use the packages. %

The notation in this paper will be as follows: $X = [x_i]_{1\leq i \leq n}^T \in
\mathbb{R}^{n\times p}$, and the observations $x_i \in \mathbb{R}^p$. %
These observations may be transformed prior to the dimensionality reduction
step (e.g.,\ centering and/or standardization) resulting in $X' = [x'_i]_{1\leq i
  \leq n}^T \in \mathbb{R}^{n\times p}$. %
A DR method then embeds each vector in $X'$ onto a vector in $Y = [y_i]_{1\leq i
  \leq n}^T \in \mathbb{R}^{n\times q}$ with $y_i \in \mathbb{R}^q$, ideally
with $q \ll p$. %
Some methods provide an explicit mapping $f(x'_i) = y_i$. Some even offer an
inverse mapping $f^{-1}(y_{i}) = \hat x'_{i}$, such that one can reconstruct a
(usually approximate) sample from the low-dimensional representation. %
For some methods, pairwise distances between points are needed, we set $d_{ij} =
d(x_{i}, x_{j})$ and $\hat{d}_{ij} = d(y_i, y_j)$, where $d$ is some appropriate
distance function.

When referring to \code{functions} in the \pkg{dimRed} package or base R simply the
function name is mentioned, functions from other packages are
referenced with their namespace, as with \code{package::function}.

\begin{figure}[htbp]
  \centering
  \input{classification_tree.tex}
  \caption{%
    Classification of dimensionality reduction methods. Methods
    in bold face are implemented in \pkg{dimRed}.
    Modified from \citet{van_der_maaten_dimensionality_2009}.
  }\label{fig:classification}
\end{figure}


\section{Dimensionality Reduction Methods}
\label{sec:dimredtec}

In the following section we do not aim for an exhaustive explanation to every
method in \pkg{dimRed} but rather to provide a general idea on how the
methods work. %
An overview and classification of the most commonly used DR methods can be found
in Figure~\ref{fig:classification}.

In all methods, parameters have to be optimized or decisions have to be made,
even if it is just about the preprocessing steps of data. %
The \pkg{dimRed} package tries to make the optimization process for parameters as easy as
possible, but, if possible, the parameter space should be narrowed down using
prior knowledge. %
Often decisions can be made based on theoretical knowledge. For example,\ sometimes an
analysis requires data to be kept in their original scales and sometimes this is
exactly what has to be avoided as when comparing different physical
units. %
Sometimes decisions based on the experience of others can be made, e.g.,\ the
Gaussian kernel is probably the most universal kernel and therefore should be
tested first if there is a choice. %

All methods presented here have the embedding dimensionality, $q$, as a
parameter (or \code{ndim} as a parameter for \code{embed}). %
For methods based on eigenvector decomposition, the result generally does not
depend on the number of dimensions, i.e.,\ the first dimension will be the same,
no matter if we decide to calculate only two dimensions or more. %
If more dimensions are added, more information is maintained, the first
dimension is the most important and higher dimensions are successively less
important. %
This means, that a method based on eigenvalue decomposition only has to be run
once if one wishes to compare the embedding in different dimensions. %
In optimization based methods this is generally not the case, the number of
dimensions has to be chosen a priori, an embedding of 2 and 3 dimensions may
vary significantly, and there is no ordered importance of dimensions. %
This means that comparing dimensions of optimization-based methods is
computationally much more expensive. %

We try to give the computational complexity of the methods. Because of the
actual implementation, computation times may differ largely. %
R is an interpreted language, so all parts of an algorithm that are implemented
in R often will tend to be slow compared to methods that call efficient
implementations in a compiled language. %
Methods where most of the computing time is spent for eigenvalue decomposition
do have very efficient implementations as R uses optimized linear algebra
libraries.  Although, eigenvalue decomposition itself does not scale very well in
naive implementations ($\mathcal{O}(n^3)$).

\subsection{PCA}
\label{sec:pca}

Principal Component Analysis (PCA) is the most basic technique for reducing
dimensions. It dates back to \citet{pearson_lines_1901}. PCA finds a linear
projection ($U$) of the high dimensional space into a low dimensional space $Y =
XU$, maintaining maximum variance of the data. It is based on solving the
following eigenvalue problem:

\begin{equation}
    (C_{XX}-\lambda_k I)u_k=0\label{eq:pca}
\end{equation}
where $C_{XX} = \frac 1 n X^TX$ is the covariance matrix, $\lambda_k$ and $u_k$
are the $k$-th eigenvalue and eigenvector, and $I$ is the identity matrix. %
The equation has several solutions for different values of $\lambda_k$ (leaving
aside the trivial solution $u_k = 0$). %
PCA can be efficiently applied to large data sets, because it computationally
scales as $\mathcal{O}(np^2 + p^3)$, that is, it scales linearly with the number of
samples and R uses specialized linear algebra libraries for such kind of
computations.

PCA is a rotation around the origin and there exist a forward and inverse
mapping. %
PCA may suffer from a scale problem, i.e.,\ when one variable dominates the
variance simply because it is in a higher scale, to remedy this, the data can be
scaled to zero mean and unit variance, depending on the use case, if this is
necessary or desired. %

Base R implements PCA in the functions \code{prcomp} and \code{princomp}; but
several other implementations exist i.e., \BIOpkg{pcaMethods} from Bioconductor
which implements versions of PCA that can deal with missing data. %
The \pkg{dimRed} package wraps \code{prcomp}.

\subsection{kPCA}
\label{sec:kpca}

Kernel Principal Component Analysis (kPCA) extends PCA to deal with nonlinear
dependencies among variables. %
The idea behind kPCA is to map the data into a high dimensional space using a
possibly non-linear function $\phi$ and then to perform a PCA in this high
dimensional space. %
Some mathematical tricks are used for efficient computation. %

If the columns of X are centered around $0$, then the principal components can
also be computed from the inner product matrix $K = X^TX$. %
Due to this way of calculating a PCA, we do not need to explicitly map all points
into the high dimensional space and do the calculations there, it is enough to
obtain the inner product matrix or kernel matrix $K \in \mathbb{R}^{n\times n}$
of the mapped points \citep{scholkopf_nonlinear_1998}. %

Here is an example calculating the kernel matrix using a Gaussian kernel:
\begin{equation}\label{eq:gauss}
  K = \phi(x_i)^T \phi(x_j) = \kappa(x_i, x_j) = \exp\left(
    -\frac{\| x_i- x_j\|^2}{2 \sigma^2}
  \right),
\end{equation}
where $\sigma$ is a length scale parameter accounting for the width of the
kernel. %
The other trick used is known as the ``representers theorem.'' The interested
reader is referred to \citet{scholkopf_generalized_2001}.

The kPCA method is very flexible and there exist many kernels for special
purposes. The most common kernel function is the Gaussian kernel
(Equation\ \ref{eq:gauss}). %
The flexibility comes at the price that the method has to be finely
tuned for the data set because some parameter combinations are simply
unsuitable for certain data. %
The method is not suitable for very large data sets, because memory
scales with $\mathcal{O}(n^2)$ and computation time with
$\mathcal{O}(n^3)$. %

Diffusion Maps, Isomap, Locally Linear Embedding, and some other techniques can
be seen as special cases of kPCA. In which case, an out-of-sample extension using the Nyström
formula can be applied \citep{bengio_learning_2004}. %
This can also yield applications for bigger data, where an embedding is trained
with a sub-sample of all data and then the data is embedded using the Nyström
formula.

Kernel PCA in R is implemented in the \CRANpkg{kernlab} package using the function
\code{kernlab::kpca}, and supports a number of kernels and
user defined functions. For details see the help page for \code{kernlab::kpca}.

The \pkg{dimRed} package wraps  \code{kernlab::kpca} but additionally
provides forward and inverse methods \citep{bakir_learning_2004} which can be
used to fit out-of-sample data or to visualize the transformation of the data
space. %

\subsection{Classical Scaling}
\label{sec:classscale}

What today is called Classical Scaling was first introduced by
\citet{torgerson_multidimensional_1952}. It uses an eigenvalue decomposition of
a transformed distance matrix to find an embedding that maintains the distances
of the distance matrix. %
The method works because of the same reason that kPCA works, i.e.,\ classical
scaling can be seen as a kPCA with kernel $x^Ty$. %
A matrix of Euclidean distances can be transformed into an inner product matrix
by some simple transformations and therefore yields the same result as a
PCA\@. %
Classical scaling is conceptually more general than PCA in that arbitrary
distance matrices can be used, i.e.,\ the method does not even need the original
coordinates, just a distance matrix $D$. %
Then it tries to find an embedding $Y$ so that $\hat d_{ij}$ is as similar to
$d_{ij}$ as possible.

The disadvantage is that it is computationally much more demanding, i.e.,\
an eigenvalue decomposition of an $n\times n$ matrix has to be computed.
This step requires $\mathcal{O}(n^2)$ memory and $\mathcal{O}(n^3)$
computation time, while PCA requires only the eigenvalue decomposition
of a $d\times d$ matrix and usually $n \gg d$. %
R implements classical scaling in the \code{cmdscale}
function. %

The \pkg{dimRed} package wraps \code{cmdscale} and allows the specification
of arbitrary distance functions for calculating the distance matrix. Additionally
a forward method is implemented.

\subsection{Isomap}
\label{sec:isomap}
As Classical Scaling can deal with arbitrarily defined distances,
\citet{tenenbaum_global_2000} suggested to approximate the
structure of the manifold by using geodesic distances. %
In practice, a graph is created by either keeping only
the connections between every point and its $k$ nearest neighbors to
produce a $k$-nearest neighbor graph ($k$-NNG), or simply by keeping all
distances smaller than a value $\varepsilon$ producing an
$\varepsilon$-neighborhood graph ($\varepsilon$-NNG). %
Geodesic distances are obtained by recording the distance on the
graph and classical scaling is used to find an embedding in fewer
dimensions. This leads to an ``unfolding'' of possibly convoluted
structures (see Figure~\ref{fig:knn}).

Isomap's computational cost is dominated by the eigenvalue decomposition and
therefore scales with $\mathcal{O}(n^3)$. %
Other related techniques can use more efficient algorithms
because the distance matrix becomes sparse due to a different preprocessing.

In R, Isomap is implemented in the \CRANpkg{vegan} package. The
\code{vegan::isomap} calculates an Isomap embedding and \code{vegan::isomapdist}
calculates a geodesic distance matrix. %
The \pkg{dimRed} package uses its own implementation. This implementation is
faster mainly due to using a KD-tree for the nearest neighbor search (from the
\CRANpkg{RANN} package) and to a faster implementation for the shortest path
search in the $k$-NNG (from the \CRANpkg{igraph} package). %
The implementation in \pkg{dimRed} also includes a forward method that can be
used to train the embedding on a subset of data points and then use these points
to approximate an embedding for the remaining points. This technique is
generally referred to as landmark Isomap \citep{de_silva_sparse_2004}. %

\subsection{Locally Linear Embedding}
\label{sec:lle}

Points that lie on a manifold in a high dimensional space can be
reconstructed through linear combinations of their neighborhoods if the
manifold is well sampled and the neighbohoods lie on a locally linear
patch. %
These reconstruction weights, $W$, are the same in the high dimensional
space as the internal coordinates of the manifold. %
Locally Linear Embedding \citep[LLE; ][]{roweis_nonlinear_2000} is a
technique that constructs a weight matrix
$W \in \mathbb{R}^{n\times n}$ with elements $w_{ij}$ so that
\begin{equation}
    \sum_{i=1}^n \bigg\| x_i-
        \sum_{j=1}^{n} w_{ij}x_j \bigg\|^2\label{eq:lle}
\end{equation}
is minimized under the constraint that $w_{ij} = 0 $ if $x_j$ does not belong
to the neighborhood and the constraint that $\sum_{j=1}^n w_{ij} = 1$. %
Finally the embedding is made in such a way that the following cost function is
minimized for $Y$,
\begin{equation}
    \sum_{i=1}^n\bigg\| y_i - \sum_{j=1}^n w_{ij}y_j
    \bigg\|^2.\label{eq:lle2}
\end{equation}
This can be solved using an eigenvalue decomposition.

Conceptually the method is similar to Isomap but it is
computationally much nicer because the weight matrix is
sparse and there exist efficient solvers. %
In R, LLE is implemented by the package \CRANpkg{lle}, the embedding can be
calculated with \code{lle::lle}.
Unfortunately the implementation does not make use of the sparsity of the weight matrix
$W$. %
The manifold must be well sampled and the neighborhood size must be
chosen appropriately for LLE to give good results. %

\subsection{Laplacian Eigenmaps}
\label{sec:laplaceigenmaps}

Laplacian Eigenmaps were originally developed under the name spectral clustering
to separate non-convex clusters. %
Later it was also used for graph embedding and DR
\citep{belkin_laplacian_2003}. %

A number of variants have been proposed. %
First, a graph is constructed, usually from a distance matrix, the graph can be
made sparse by keeping only the $k$ nearest neighbors, or by specifying an
$\varepsilon$ neighborhood. %
Then, a similarity matrix $W$ is calculated by using a Gaussian kernel (see Equation
\ref{eq:gauss}), if $c = 2 \sigma^2 = \infty$, then all distances are treated
equally, the smaller $c$ the more emphasis is given to differences in
distance. %
The degree of vertex $i$ is $d_i = \sum_{j=1}^n w_{ij}$ and the degree
matrix, $D$, is the diagonal matrix with entries $d_i$. %
Then we can form the graph Laplacian $L = D - W$ and, then, there are several ways how
to proceed, an overview can be found in \citet{luxburg_tutorial_2007}. %

The \pkg{dimRed} package implements the algorithm from
\citet{belkin_laplacian_2003}. Analogously to LLE, Laplacian eigenmaps
avoid computational complexity by creating a sparse matrix and not
having to estimate the distances between all pairs of points. %
Then the eigenvectors corresponding to the lowest eigenvalues larger
than $0$ of either the matrix $L$ or the normalized Laplacian
$D^{-1/2}LD^{-1/2}$ are computed and form the embedding.

\subsection{Diffusion Maps}
\label{sec:isodiffmaplle}
Diffusion Maps \citep{coifman_diffusion_2006} take a distance matrix
as input and calculates the transition probability matrix $P$ of a
diffusion process between the points to approximate the manifold. %
Then the embedding is done by an eigenvalue decompositon of $P$ to
calculate the coordinates of the embedding. %
The algorithm for calculating Diffusion Maps shares some elements with
the way Laplacian Eigenmaps are calculated. %
Both algorithms depart from the same weight matrix, Diffusion Maps
calculate the transition probability on the graph after $t$ time steps
and do the embedding on this probability matrix.

The idea is to simulate a diffusion process between the nodes of the
graph, which is more robust to short-circuiting than the $k$-NNG from
Isomap (see bottom right Figure \ref{fig:knn}). %
Diffusion maps in R are accessible via the
\code{diffusionMap::diffuse()} function, which is available in the
\CRANpkg{diffusionMap} package. %
Additional points can be approximated into an existing embedding using
the Nyström formula \citep{bengio_learning_2004}. %
The implementation in \pkg{dimRed} is based on the
\code{diffusionMap::diffuse} function.
% , which does not contain an
% approximation for unequally sampled manifolds
% \citep{coifman_geometric_2005}. %


\subsection{non-Metric Dimensional Scaling}
\label{sec:nmds}

While Classical Scaling and derived methods (see section
\nameref{sec:classscale}) use eigenvector decomposition to embed the data in
such a way that the given distances are maintained, non-Metric Dimensional
Scaling \citep[nMDS, ][]{kruskal_multidimensional_1964,kruskal_nonmetric_1964}
uses optimization methods to reach the same goal. %
Therefore a stress function,
\begin{equation}
    \label{eq:stress}
    S = \sqrt{\frac{\sum_{i<j} {( d_{ij} - \hat{d}_{ij} )}^2}
                   {\sum_{i<j}d_{ij}^2}},
\end{equation}
is used, and the algorithm tries to embed $y_i$ in such a way that the order of
the $d_{ij}$ is the same as the order of the $\hat d_{ij}$ Because optimization
methods can fit a wide variety of problems, there are very loose limits set to
the form of the error or stress function. %
For instance \citet{mahecha_nonlinear_2007} found that nMDS using geodesic
distances can be almost as powerful as Isomap for embedding biodiversity
patterns. %
Because of the flexibility of nMDS, there is a whole package in R devoted to
Multidimensional Scaling, \code{smacof} \citep{leeuw_multidimensional_2009}. %

Several packages provide implementations for nMDS in R, for example
\CRANpkg{MASS} and \pkg{vegan} with the functions \code{MASS::isoMDS} and
\code{vegan::monoMDS}. Related methods include Sammons Mapping which
con be found as \code{MASS::sammon}. %
The \pkg{dimRed} package wraps \code{vegan::monoMDS}.

\subsection{Force Directed Methods}
\label{sec:graph}

The data $X$ can be considered as a graph with weighted edges, where the weights
are the distances between points. %
Force directed algorithms see the edges of the graphs as springs or the result of
an electric charge of the nodes that result in an attractive or repulsive force
between the nodes, the algorithms then try to minimize the overall energy of the
graph. %
\begin{equation}
    \label{eq:graphenergy}
    E = \sum_{i<j}k_{ij} {( d_{ij} - \hat{d}_{ij} )}^2,
\end{equation}
where $k_{ij}$ is the spring constant for the spring connecting points
$i$ and $j$.

Graph embedding algorithms generally suffer from long running times
(though compared to
other methods presented here they do not scale as badly) and many local
optima. %
This is why a number of methods have been developed that try to deal with some
of the shortcomings, for example, the Kamada-Kawai \citep{kamada_algorithm_1989}, the
Fruchtermann-Reingold \citep{fruchterman_graph_1991}, or the DrL
\citep{martin_dr.l:_2007} algorithms. %

There are a number of graph embedding algorithms included in the
\CRANpkg{igraph} package, they can be accessed using the
\code{igraph::layout\_with\_*} function family. The \pkg{dimRed} package
only wraps the three algorithms mentioned above; there are many others
which are not interesting for dimensionality reduction. %

\subsection{$t$-SNE}
\label{sec:tsne}

Stochastic Neighbor Embedding  \citep[SNE; ][]{hinton_stochastic_2003} is a
technique that minimizes the Kullback-Leibler divergence of scaled
similarities of the points $i$ and $j$ in a high dimensional space,
$p_{ij}$, and a low
dimensional space, $q_{ij}$:

\begin{equation}\label{eq:kldivergence}
    KL(P \| Q) = \sum_{i\neq j} p_{ij}\log\frac{p_{ij}}{q_{ij}}.
\end{equation}
SNE uses a Gaussian kernel (see Equation~\ref{eq:gauss}) to compute similarities in
a high and a low dimensional space. The $t$-Distributed Stochastic Neighborhood
Embedding \citep[$t$-SNE; ][]{van_der_maaten_visualizing_2008} improves on SNE by
using a $t$-Distribution as a kernel in low dimensional space. %
Because of the heavy-tailed $t$-distribution, $t$-SNE maintains local
neighborhoods of the data better and penalizes wrong embeddings of dissimilar
points. %
This property makes it especially suitable to represent clustered data and
complex structures in few dimensions.

The $t$-SNE method has one parameter, perplexity, to tune. This determines the
neighborhood size of the kernels used.

The general runtime of $t$-SNE is $\mathcal{O}(n^2)$, but an efficient
implementation using tree search algorithms that scales as $\mathcal{O}(n\log n)$
exists and can be found in the \CRANpkg{Rtsne} package in R. %
The $t$-SNE implementation in \pkg{dimRed} wraps  the \pkg{Rtsne}
package.

There exist a number of derived techniques for dimensionality reduction, e.g.,\
NeRV \citep{venna_information_2010} and JNE \citep{lee_type_2013}, that improve
results but for which there do not yet exist packages on CRAN implementing them.


% \begin{figure}[tb]
%   \centering
%   \includegraphics[width=.95\textwidth]{plots/kpca_grid}
%   \caption[kPCA data space transformation.]{Transformation of a
%     two-dimensional data space by kPCA\@. It can be observed that kPCA
%     does not extrapolate well outside of the training range}
%   \label{fig:kpca}
% \end{figure}


\subsection{ICA}
\label{sec:ica}

Independent Component Analysis (ICA) interprets the data $X$ as a mixture
of independent signals, e.g.,\ a number of sound sources recorded by
several microphones, and tries to ``un-mix'' them to find the original
signals in the recorded signals. %
ICA is a linear rotation of the data, just as PCA, but instead of
recovering the maximum variance, it recovers statistically independent
components. %
% To do so a pre-whitening is performed on $X$, so that $x_{\cdot i}$
% are uncorrelated and have variance $1$.
A signal matrix $S$ and a mixing matrix $A$ are estimated so that
$X = AS$.

There are a number of algorithms for ICA, the most widely used is
fastICA \citep{hyvarinen_fast_1999} because it provides a fast and robust
way to estimate $A$ and $S$.
% It considers two signals to be
% independent if they are non-gaussian, the algorithm uses a fast
% approximation of the negentropy \citep{comon_independent_1994} measure to make two
% variables independent.
FastICA maximizes a measure for non-Gaussianity called negentropy $J$
\citep{comon_independent_1994}. This is equivalent to minimizing
mutual information between the resulting components. %
Negentropy $J$ is defined as follows:
%mutual information $i$ is defined as:

\begin{align}\label{eq:negentropy}
  H(u) &= - \int f(u) \log f(Y) \, \mathrm{d}u, \\
  J(u) &= H(u_{\text{gauss}}) - H(u),
\end{align}
% $$I(y_1, y_2, \ldots, y_n) = J(\mathbf{y}) - \sum_i J(y_i)$$
where $u = (u_1, \ldots, u_n)^T$ is a random vector with density $f(\cdot)$ and
$u_{\text{gauss}}$ is a Gaussian random variable with the same covariance
structure as $u$. FastICA uses a very efficient approximation to calculate
negentropy. Because ICA can be translated into a simple linear projection, a
forward and an inverse method can be supplied.

There are a number of packages in R that implement algorithms for
ICA, the \pkg{dimRed} package wraps  the
\texttt{fastICA::fastICA()} function from \CRANpkg{fastICA}.


\subsection{DRR}
\label{sec:drr}

Dimensionality Reduction via Regression is a very recent technique extending PCA
\citep{laparra_dimensionality_2015}. %
Starting from a rotated (PCA) solution $X'= XU$, it predicts redundant
information from the remaining components using non-linear regression.
\begin{equation}\label{eq:drr}
    y_{\cdot i} = x'_{\cdot i} - f_i(x'_{\cdot 1}, x'_{\cdot 2}, \ldots, x'_{\cdot i-1})
\end{equation}
with $x_{\cdot i}$ and $y_{\cdot i}$ being the loading of observations on the
$i$-th axis. %
In theory, any kind of regression can be used. the
authors of the original paper choose Kernel Ridge Regression 
\citep[KRR; ][]{saunders_ridge_1998} because it is a flexible nonlinear
regression technique and computational optimizations for a fast
calculation exist.  DRR has another advantage over other techniques
presented here, because it provides an exact forward and inverse function.

The usage of KRR also has the advantage of making the method convex, here we list
it under non-convex methods, because other types of regression may make it non-convex.

Mathematically, functions are limited to map one input to a single output point.
Therefore, DRR reduces to PCA if manifolds are too complex;
but it seems very useful for slightly curved manifolds. The initial rotation is
important, because the result strongly depends on the order of dimensions in
high dimensional space.

DRR is implemented in the package \CRANpkg{DRR}. The package provides
forward and inverse functions which can be used to train on a
subset.


\section{Quality criteria}
\label{sec:quality}

The advantage of unsupervised learning is that one does not need to
specify classes or a target variable for the data under
scrutiny. Instead the chosen algorithm arranges the input data. For
example, arranged into clusters or into a lower dimensional
representation. %
In contrast to a supervised problem, there is no natural way to
directly measure the quality of any output or to compare two methods
by an objective measure like for instance modeling efficiency or
classification error. %
The reason is that every method optimizes a different error function,
and it would be unfair to compare $t$-SNE and PCA by means of either
recovered variance or KL-Divergence. %
One fair measure would be the reconstruction error, i.e.,\
reconstructing the original data from a limited number of dimensions,
but as discussed above not many methods provide forward and inverse
mappings. %

However, there are a series of independent estimators on the quality of a
low-dimensional embedding. %
The \pkg{dimRed} package provides a number of quality measures which have
been proposed in the literature to measure performance of dimensionality
reduction techniques.

\subsection{Co-ranking matrix based measures}
\label{sec:coranking}

The co-ranking matrix \citep{lee_quality_2009} is a way to capture the
changes in ordinal distance. As before, let $d_{ij} = d(x_i,x_j)$
be the distances between $x_i$ and $x_j$, i.e.,\ in high dimensional
space and $\hat d_{ij} = d(y_i,y_j)$ the distances in low dimensional
space, then we can define the rank of $y_j$ with respect to $y_i$

\begin{equation}
  \label{eq:rankmatrix-low-dim}
  \hat r_{ij} = |\{k : \hat d_{ik} < \hat d_{ij} \text{ or } (\hat
  d_{ik} = \hat d_{ij} \text{ and } 1 \leq k < j \leq n)\}|,
\end{equation}
and, analogously, the rank in high-dimensional space as:
\begin{equation}
  \label{eq:rankmatrix-high-dim}
  r_{ij} = |\{k : d_{ik} < d_{ij} \text{ or } (d_{ik} = d_{ij} \text{
    and } 1 \leq k < j \leq n)\}|,
\end{equation}
where the notation $|A|$ denotes the number of elements in a set $A$. %
This means that we simply replace the distances in a distance matrix column
wise by their ranks. %
Therefore $r_{ij}$ is an integer which indicates that $x_i$ is the
$r_{ij}$-th closest neighbor of $x_j$ in the set $X$.

The co-ranking matrix $Q$ then has elements
\begin{equation}
  \label{eq:coranking}
  q_{kl} = |\{(i,j) : \hat r_{ij} = k \text{ and } r_{ij} = l\}|,
\end{equation}
which is the 2d-histogram of the ranks. That is, $q_{ij}$ is an integer which counts
how many points of distance rank $j$ became rank $i$. %
In a perfect DR, this matrix will only have non-zero entries in the diagonal; if
most of the non-zero entries are in the lower triangle, then the DR collapsed
far away points onto each other; if most of the non-zero entries are in the
upper triangle, then the DR teared close points apart. % 
For a detailed description of the properties of the co-ranking matrix the reader
is referred to \citet{lueks_how_2011}. % 

The co-ranking matrix can be computed using function
\code{coRanking::coranking()} and can be visualized using
\code{coRanking::imageplot()}. %
A good embedding should scatter the values around the diagonal of the matrix. If
the values are predominantly in the lower triangle, then the embedding collapses the original
structure causing far away points to be much closer; if the values are
predominantly in the upper triangle the points from the original structure are
torn apart. %
Nevertheless this method requires visual inspection of the matrix. %
For an automated assessment of quality, a scalar value that assigns a quality to
an embedding is needed. %

A number of metrics can be computed from the co-ranking matrix. For example:

\begin{equation}\label{eq:qnx}
    Q_{N\!X}(k) = \frac{1}{kn} \sum_{i=1}^k\sum_{j=1}^k q_{ij},
\end{equation}
which is the number of points that belong to the $k$-th nearest neighbors in
both high- and low-dimensional space, normalized to give a maximum of $1$
\citep{lee_quality_2009}. %
This quantity can be adjusted for random embeddings, giving the Local
Continuity Meta Criterion \citep{chen_local_2006}:
\begin{equation}\label{eq:lcmc}
\text{LCMC}(k) = Q_{N\!X}(k) - \frac{k}{n-1}
\end{equation}
The above measures still depend on $k$, but LCMC has a well defined
maximum at $k_{\max} $. %
Two measures without parameters are then defined:

\begin{align}\label{eq:qlocalglobal}
  Q_{\text{local}}  &= \frac{1}{k_{\max}}     \sum_{k=1}^{k_{\max}}
                      Q_{N\!X}(k) \text{ and} \\
  Q_{\text{global}} &= \frac{1}{n - k_{\max}} \sum_{k=k_{\max}}^{n-1} 
                      Q_{N\!X}(k).
\end{align}
These measure the preservation of local and global distances respectively. The
original authors advised using $Q_{\text{local}}$ over $Q_{\text{global}}$, but
this depends on the application.

LCMC($k$) can be normalized to a maximum of $1$, yielding the
following measure for a quality embedding \citep{lee_type_2013}:
\begin{equation}\label{eq:rnx}
    R_{NX}(k) = \frac{(n-1) Q_{NX}(k) - k}{n-1-k},
\end{equation}
where a value of 0 corresponds to a random embedding and a value of 1
to a perfect embedding into the $k$-ary neighborhood. %
To transform $R_{NX}(k)$ into a parameterless measure, the area under
the curve can be used:
\begin{equation}\label{eq:auclnk}
    \text{AUC}_{\ln k}\left(R_{NX}(k)\right) = \left( \sum_{k=1}^{n-2}
    R_{NX}(k) \right) \Bigg/ \left( \sum_{k=1}^{n-2} 1/k  \right).
\end{equation}

This measure is normalized to one and takes $k$ at a log-scale. %
Therefore it prefers methods that preserve local distances. %

In R, the co-ranking matrix can be calculated using the the
\code{coRanking::coranking} function. %
The \pkg{dimRed} package contains the functions \code{Q\_local},
\code{Q\_global}, \code{Q\_NX}, \code{LCMC}, and \code{R\_NX} to
calculate the above quality measures in addition to \code{AUC\_lnK\_R\_NX}. %

Calculating the co-ranking matrix is a relatively expensive operation
because it requires sorting every row of the distance matrix twice.
It therefore scales with $\mathcal{O}(n^2\log n)$. %
There is also a plotting function \code{plot\_R\_NX}, which plots the
$R_{NX}$ values with log-scaled $K$ and adds the $AUC_{\ln K}$ to the
legend (see Figure \ref{fig:plotexample}). %

There are a number of other measures that can be computed from a co-ranking
matrix, e.g.,\ see \citet{lueks_how_2011,lee_quality_2009}, or
\citet{babaee_assessment_2013}.


\subsection{Cophenetic correlation}
\label{sec:discor}

An old measure originally developed to compare clustering methods in the field
of phylogenetics is cophenetic correlation \citep{sokal_comparison_1962}. %
This method consists simply of the correlation between the upper or lower
triangles of the distance matrices (in dendrograms they are called cophenetic
matrices, hence the name) in a high and low dimensional space. Additionally the
distance measure and correlation method can be varied. %
In the \pkg{dimRed} package this is implemented in the
\code{cophenetic\_correlation} function. %

Some studies use a measure called ``residual variance''
\citep{tenenbaum_global_2000,mahecha_nonlinear_2007}, which is defined as
$$ 1 - r^2(D, \hat D), $$
where $r$ is the Pearson correlation and $D$, $\hat D$ are the distances
matrices consisting of elements $d_{ij}$ and $\hat d_{ij}$ respectively. %


\subsection{Reconstruction error}
\label{sec:reconerror}

The fairest  and most common way to assess the quality of a dimensionality
reduction when the method provides an inverse mapping is 
the reconstruction error.
The \pkg{dimRed} package includes a function to calculate the root mean
squared error which is defined as:
\begin{equation}\label{eq:rmse}
\text{RMSE} = \sqrt{\frac{1}{n} \sum_{i=1}^n {d(x'_i, x_i)}^2}
\end{equation}
with $x'_i = f^{-1}(y_{i})$, $f^{-1}$ being the
function that maps an embedded value back to feature space. %

The \pkg{dimRed} package provides the \code{reconstruction\_rmse} and
\code{reconstruction\_error} functions.



\section{Test data sets}
\label{sec:testdata}

There are a number of test data sets that are often used to showcase a
dimensionality reduction technique. Common ones being the 3d S-curve and the Swiss
roll, among others. %
These data sets have in common that they usually have three dimensions, and well
defined manifolds. %
Real world examples usually have more dimensions and often are much noisier, the
manifolds may not be well sampled and exhibit holes and large pieces may be
missing. %
Additionally, we cannot be sure if we can observe all the relevant variables. %

The \pkg{dimRed} package implements a number of test datasets that are being
used in literature to benchmark methods with the function
\code{dimRed::loadDataSet()}. %
For artificial datasets the number of points and the noise level can be
adjusted, the function also returns the internal coordinates.

\section{The \pkg{dimRed} Package}


The \pkg{dimRed} package collects DR methods readily implemented in R,
implements missing methods and offers means to compare the quality of
embeddings. %
The package is open source and available under the GPL3
license. Released versions of the package are
available through CRAN
(\url{https://cran.r-project.org/package=dimRed})  and development
versions are hosted on GitHub (\url{https://github.com/gdkrmr/dimRed}). %
The \pkg{dimRed} package provides a common interface and convenience functions for a variety of
different DR methods so that it is made easier to use and compare different
methods. An overview of the packages main functions can be found in Table~\ref{tab:interfacefun}. %

\begin{table}[h]
    \centering
    \begin{tabular}{l p{8cm}}
      \toprule
      Function                 & Description \\
      \midrule 
      \code{embed}             & Embed data using a DR method. \\
      \code{quality}           & Calculate a quality score from
                                 the result of \code{embed}. \\
      \code{plot}              & Plot a \code{"dimRedData"} or
                                 \code{"dimRedResult"} object, colors
                                 the points automatically, for
                                 exploring the data. \\
      \code{plot\_R\_NX}       & Compares the quality of various
                                 embeddings. \\
      \code{dimRedMethodList}  & Returns a character vector that
                                 contains all implemented DR
                                 methods. \\
      \code{dimRedQualityList} & Returns a character vector that
                                 contains all implemented quality
                                 measures. \\
      \bottomrule
    \end{tabular}
    \caption[Main interface functions.]{The main interface functions
      of the \pkg{dimRed} package.}\label{tab:interfacefun}
\end{table}

Internally, the package uses S4 classes but for normal usage the user
does not need to have any knowledge on the inner workings of the S4 class
system in R (cf.\ table~\ref{tab:s4classes}). %
The package contains simple conversion functions from and to standard
R-objects like a data.frame or a matrix. %
The \code{"dimRedData"} class provides a container for the data to be
processed. %
The slot \code{data} contains a matrix with dimensions in columns and
observations in rows, the slot \code{meta} may contain a data frame with
additional information, e.g.,\ categories or other information of the data
points. %

\begin{table}[h]
    \centering
    \begin{tabular}{l p{9cm}}
      \toprule
      Class Name          & Function \\
      \midrule
      \code{"dimRedData"}   & Holds the data for a DR.
                            Fed to \code{embed()}.
                            An \code{as.dimRedData()} methods exists for 
                            \code{"data.frame"}, 
                            \code{"matrix"}, and
                            \code{"formula"} exist.\\
      \code{"dimRedMethod"} & Virtual class, ancestor of all DR
                            methods. \\
      \code{"dimRedResult"} & The result of \code{embed()}, 
                            the embedded data. \\
      \bottomrule
    \end{tabular}
    \caption[S4 Classes]{The S4 classes used in the
      \pkg{dimRed} package.}\label{tab:s4classes}

\end{table}

Each embedding method is a class which inherits from \code{"dimRedMethod"} which
means that it contains a function to generate \code{"dimRedResult"} objects and a
list of standard parameters. %
The class \code{"dimRedResult"} contains the data in reduced dimensions, the
original meta information along with the original data, and, if possible,
functions for the forward and inverse mapping. %

From a user-perspective the central function of the package is \code{embed}
which is called in the form \code{embed(data, method, \ldots)}, \code{data} can
take standard R objects such as instances of \code{"data.frame"}, \code{"matrix"}, or \code{"formula"},
as input. %
The \code{method} is given as a character vector. All available methods can be
listed by calling \samp{dimRedMethodList()}. %
Method-specific parameters can be passed through \code{...}; when no method-specific
parameters are given, defaults are chosen. %
The \code{embed} function returns an object of class \code{"dimRedResult"}.

For comparing different embeddings, \pkg{dimRed} contains the function
\code{quality} which relies on the output of \code{embed} and a method name. %
This function returns a scalar quality score; a vector that contains the names
of all quality functions is returned by calling \samp{dimRedQualityList()}. %

For easy visual examination, the package contains \code{plot} methods for
\code{"dimRedData"} and \code{"dimRedResult"} objects in order to plot high
dimensional data using parallel plots and pairwise scatter plots. %
Automatic coloring of data points is done using the available metadata. %

\section{Examples}
\label{sec:examples}

The comparison of different DR methods, choosing the right parameters for a
method, and the inspection of the results is simplified by
\pkg{dimRed}. %
This section contains a number of examples to highlight the usage of the package. %

% Example 1:
To compare methods of dimensionality reduction, first a test data set is loaded
using \code{loadDataSet}, then the \code{embed} function is used for DR
(\code{embed} can also handle standard R types like \code{matrix} and
\code{data.frame}). %
This makes it very simple to apply different methods of DR to the same data
e.g.,\ by defining a character vector of method names and then
iterating over these, say with
\code{lapply}. %
For inspection, \pkg{dimRed} provides methods for the \code{plot} function to
visualize the resulting embedding (Figure~\ref{fig:plotexample} b and d), internal
coordinates of the manifold are represented by color gradients. %
To visualize how well embeddings represent different neighborhood sizes, the
function \code{plot\_R\_NX} is used on a list of embedding results
(Figure~\ref{fig:plotexample}~c). %
The plots in figure~\ref{fig:plotexample} are produced by the following code: %


\begin{figure}[htp]
  \centering
<<"pca_isomap_example",include=FALSE,fig.width=4,fig.height=4>>=
if(Sys.getenv("BNET_BUILD_VIGNETTE") != "") {
  library(dimRed); library(ggplot2); #library(dplyr); library(tidyr)
  ## define which methods to apply
  embed_methods <- c("Isomap", "PCA")
  ## load test data set
  data_set <- loadDataSet("3D S Curve", n = 1000)
  ## apply dimensionality reduction
  data_emb <- lapply(embed_methods, function(x) embed(data_set, x))
  names(data_emb) <- embed_methods
  ## plot data set, embeddings, and quality analysis
  ## plot(data_set, type = "3vars")
  ## lapply(data_emb, plot, type = "2vars")
  ## plot_R_NX(data_emb)

  add_label <- function(label)
    grid::grid.text(label, 0.2, 1, hjust = 0, vjust = 1,
                    gp = grid::gpar(fontface = "bold",
                                    cex = 1.5))
  ## pdf('~/phd/text/dimRedPackage/plots/plot_example.pdf', width = 4, height = 4)
  ## plot the results
  plot(data_set, type = "3vars", angle = 15, mar = c(3, 3, 0, 0), box = FALSE, grid = FALSE, pch = 16)
  add_label("a")
  par(mar = c(4, 4, 0, 0) + 0.1, bty = "n", las = 1)
  plot(data_emb$Isomap, type = "2vars", pch = 16)
  add_label("b")
  plot(data_emb$PCA,    type = "2vars", pch = 16)
  add_label("d")
  ## calculate quality scores
  print(
  plot_R_NX(data_emb) +
  theme(legend.title = element_blank(),
  legend.position = c(0.5, 0.1),
  legend.justification = c(0.5, 0.1))
  )
  add_label("c")
} else {
  # These cannot all be plot(1:10)!!! It's a mistery to me.
  plot(1:10)
  barplot(1:10)
  hist(1:10)
  plot(1:10)
}
@
  \includegraphics[page=1,width=.45\textwidth]{figure/pca_isomap_example-1.pdf}
  \includegraphics[page=1,width=.45\textwidth]{figure/pca_isomap_example-2.pdf}
  \includegraphics[page=1,width=.45\textwidth]{figure/pca_isomap_example-3.pdf}
  \includegraphics[page=1,width=.45\textwidth]{figure/pca_isomap_example-4.pdf}

  \caption[dimRed example]{%
    Comparing PCA and Isomap: %
    (a) An S-shaped manifold, colors represent the internal coordinates of the
    manifold. %
    (b) Isomap embedding, the S-shaped manifold is unfolded. %
    (c) $R_{NX}$ plotted agains neighborhood sizes, Isomap is much better at
    preserving local distances and PCA is better at preserving global Euclidean
    distances. %
    The numbers on the legend are the $\text{AUC}_{1 / K}$.
    (d) PCA projection of the data, the directions of maximum variance are preserved. %
  }\label{fig:plotexample}
\end{figure}

<<eval=FALSE>>=
## define which methods to apply
embed_methods <- c("Isomap", "PCA")
## load test data set
data_set <- loadDataSet("3D S Curve", n = 1000)
## apply dimensionality reduction
data_emb <- lapply(embed_methods, function(x) embed(data_set, x))
names(data_emb) <- embed_methods
## figure \ref{fig:plotexample}a, the data set
plot(data_set, type = "3vars")
## figures \ref{fig:plotexample}b (Isomap) and \ref{fig:plotexample}d (PCA)
lapply(data_emb, plot, type = "2vars")
## figure \ref{fig:plotexample}c, quality analysis
plot_R_NX(data_emb)
@

The function \code{plot\_R\_NX} produces a figure that plots the neighborhood
size ($k$ at a log-scale) against the quality measure $\text{R}_{NX}(k)$ (see
Equation \ref{eq:rnx}). %
This gives an overview of the general behavior of methods: if $\text{R}_{NX}$ is
high for low values of $K$, then local neighborhoods are maintained well; if
$\text{R}_{NX}$ is high for large values of $K$, then global gradients are
maintained well. %
It also provides a way to directly compare methods by plotting more than one
$\text{R}_{NX}$ curve and an overall quality of the embedding by taking the area
under the curve as an indicator for the overall quality of the embedding (see
fig~\ref{eq:auclnk}) which is shown as a number in the legend.

Therefore we can see from Figure~\ref{fig:plotexample}c that $t$-SNE is very good a
maintaining close and medium distances for the given data set, whereas PCA is only
better at maintaining the very large distances. %
The large distances are dominated by the overall bent shape of the S in 3D
space, while the close distances are not affected by this bending. %
This is reflected in the properties recovered by the different methods, the PCA
embedding recovers the S-shape, while $t$-SNE ignores the S-shape and recovers
the inner structure of the manifold.

% Example 2:
Often the quality of an embedding strongly depends on the choice of parameters,
the interface of \pkg{dimRed} can be used to facilitate searching the
parameter space.

Isomap has one parameter $k$ which determines
the number of neighbors used to construct the $k$-NNG\@. %
If this number is too large, then Isomap will resemble an MDS
(Figure~\ref{fig:knn} e), if the number is too small, the resulting embedding
contains holes (Figure~\ref{fig:knn} c). %
The following code finds the optimal value, $k_{\text{max}}$, for $k$ using the
$Q_{\text{local}}$ criterion, the results are visualized in Figure~\ref{fig:knn}
a:

\begin{figure}[htp]
  \centering
<<include=FALSE>>=
if(Sys.getenv("BNET_BUILD_VIGNETTE") != "") {
  library(dimRed)
  library(cccd)
  ## Load data
  ss <- loadDataSet("3D S Curve", n = 500)
  ## Parameter space
  kk <- floor(seq(5, 100, length.out = 40))
  ## Embedding over parameter space
  emb <- lapply(kk, function(x) embed(ss, "Isomap", knn = x))
  ## Quality over embeddings
  qual <- sapply(emb, function(x) quality(x, "Q_local"))
  ## Find best value for K
  ind_max <- which.max(qual)
  k_max <- kk[ind_max]

  add_label <- function(label){
    par(xpd = TRUE)
    b = par("usr")
    text(b[1], b[4], label, adj = c(0, 1), cex = 1.5, font = 2)
    par(xpd = FALSE)
  }

  names(qual) <- kk
}
@

<<"select_k",include=FALSE,fig.width=11,fig.height=5>>=
if (Sys.getenv("BNET_BUILD_VIGNETTE") != "") {
  par(mfrow = c(1, 2),
      mar = c(5, 4, 0, 0) + 0.1,
      oma = c(0, 0, 0, 0))
  plot(kk, qual, type = "l", xlab = "k", ylab = expression(Q[local]), bty = "n")
  abline(v = k_max, col = "red")
  add_label("a")
  plot(ss, type = "3vars", angle = 15, mar = c(3, 3, 0, 0), box = FALSE, grid = FALSE, pch = 16)
  add_label("b")
} else {
  plot(1:10)
  plot(1:10)
}
@

<<"knngraphs",include=FALSE,fig.width=8,fig.height=3>>=
if(Sys.getenv("BNET_BUILD_VIGNETTE") != "") {
par(mfrow = c(1, 3),
    mar = c(5, 4, 0, 0) + 0.1,
    oma = c(0, 0, 0, 0))
add_knn_graph <- function(ind) {
    nn1 <- nng(ss@data, k = kk[ind])
    el <- get.edgelist(nn1)
    segments(x0 = emb[[ind]]@data@data[el[, 1], 1],
             y0 = emb[[ind]]@data@data[el[, 1], 2],
             x1 = emb[[ind]]@data@data[el[, 2], 1],
             y1 = emb[[ind]]@data@data[el[, 2], 2],
             col = "#00000010")
}
plot(emb[[2]]@data@data, type = "n", bty = "n")
add_knn_graph(2)
points(emb[[2]]@data@data, col = dimRed:::colorize(ss@meta),
       pch = 16)
add_label("c")
plot(emb[[ind_max]]@data@data, type = "n", bty = "n")
add_knn_graph(ind_max)
points(emb[[ind_max]]@data@data, col = dimRed:::colorize(ss@meta),
       pch = 16)
add_label("d")
plot(emb[[length(emb)]]@data@data, type = "n", bty = "n")
add_knn_graph(length(emb))
points(emb[[length(emb)]]@data@data, col = dimRed:::colorize(ss@meta),
       pch = 16)
add_label("e")
} else {
plot(1:10)
plot(1:10)
plot(1:10)
}
@
    \includegraphics[width=.95\textwidth]{figure/select_k-1.pdf}
    \includegraphics[width=.95\textwidth]{figure/knngraphs-1.pdf}
    \caption[estimating $k$ using @Q_\text{local}]{%
      Using \pkg{dimRed} and the $Q_\text{local}$ indicator to estimate a
      good value for the parameter $k$ in Isomap. %
      (a) $Q_\text{local}$ for different values of $k$, the vertical red
      line indicates the maximum $k_{\text{max}}$. %
      (b) The original data set, a 2 dimensional manifold bent in an
      S-shape in 3 dimensional space. %
      Bottom row: Embeddings and $k$-NNG for different values of $k$. %
      (c) When $k = 5$, the value for $k$ is too small resulting in holes in the
      embedding, the manifold itself is still unfolded correctly. %
      (d) Choose $k = k_\text{max}$, the best representation of the original
      manifold in two dimensions achievable with Isomap. %
      (e) $k = 100$, too large, the $k$-NNG does not approximate the manifold
      any more. %
    }\label{fig:knn}
\end{figure}

<<eval=FALSE>>=
## Load data
ss <- loadDataSet("3D S Curve", n = 500)
## Parameter space
kk <- floor(seq(5, 100, length.out = 40))
## Embedding over parameter space
emb <- lapply(kk, function(x) embed(ss, "Isomap", knn = x))
## Quality over embeddings
qual <- sapply(emb, function(x) quality(x, "Q_local"))
## Find best value for K
ind_max <- which.max(qual)
k_max <- kk[ind_max]
@

Figure~\ref{fig:knn}a shows how the $Q_{\text{local}}$ criterion changes when
varying the neighborhood size $k$ for Isomap, the gray lines in
Figure~\ref{fig:knn} represent the edges of the $k$-NN Graph. %
If the value for $k$ is too low, the inner structure of the manifold will still
be recovered, but it will be imperfect (Figure~\ref{fig:knn}c, note that the holes
appear in places that are not covered by the edges of the $k$-NN Graph),
therefore the $Q_{\text{local}}$ score is lower than optimal. %
If $k$ is too large, the error of the embedding is much larger due to short
circuiting and we observe a very steep drop in the $Q_{\text{local}}$ score. %
The short circuiting can be observed in Figure~\ref{fig:knn}e with the edges that
cross the gap between the tips and the center of the S-shape. %

% Example 3:

It is also very easy to compare across methods and quality scores. %
The following code produces a matrix of quality scores and methods,
where \code{dimRedMethodList} returns a character vector with all methods. A
visualization of the matrix can be found in Figure~\ref{fig:qualityexample}. %

\begin{figure}[htp]
  \centering
<<"plot_quality",include=FALSE>>=
if(Sys.getenv("BNET_BUILD_VIGNETTE") != "") {
  embed_methods <- dimRedMethodList()
  quality_methods <- c("Q_local", "Q_global", "AUC_lnK_R_NX",
                      "cophenetic_correlation")
  iris_data <- loadDataSet("Iris")
  quality_results <- matrix(
      NA, length(embed_methods), length(quality_methods),
      dimnames = list(embed_methods, quality_methods)
)
embedded_data <- list()

for (e in embed_methods) {
  try(embedded_data[[e]] <- embed(iris_data, e))
  for (q in quality_methods)
    try(quality_results[e,q] <- quality(embedded_data[[e]], q))
}

quality_results <- quality_results[order(rowMeans(quality_results)), ]

palette(c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e"))
col_hsv <- rgb2hsv(col2rgb(palette()))
## col_hsv["v", ] <- col_hsv["v", ] * 3 / 1
palette(hsv(col_hsv["h", ], col_hsv["s", ], col_hsv["v", ]))
par(mar = c(2, 8, 0, 0) + 0.1)
barplot(t(quality_results), beside = TRUE, col = 1:4,
        legend.text = quality_methods, horiz = TRUE, las = 1,
        cex.names = 0.85,
        args.legend = list(x = "topleft", bg = "white", cex = 0.8))
} else {
  plot(1:10)
}
@
  \includegraphics[width=.5\textwidth]{figure/plot_quality-1.pdf}

  \caption[Quality comparision]{%
    A visualization of the \code{quality\_results} matrix. %
    The methods are ordered by mean quality score. %
    The reconstruction error was omitted, because a higher value means
    a worse embedding, while in the present methods a higher score
    means a better embedding. %
    Parameters were not tuned for the example, therefore it should not
    be seen as a general quality assessment of the methods. %
  }\label{fig:qualityexample}
\end{figure}

<<eval=FALSE>>=
embed_methods <- dimRedMethodList()
quality_methods <- c("Q_local", "Q_global", "AUC_lnK_R_NX",
                     "cophenetic_correlation")
scurve <- loadDataSet("3D S Curve", n = 2000)
quality_results <- matrix(
  NA, length(embed_methods), length(quality_methods),
  dimnames = list(embed_methods, quality_methods)
)

embedded_data <- list()
for (e in embed_methods) {
  embedded_data[[e]] <- embed(scurve, e)
  for (q in quality_methods) {
    try(quality_results[e, q] <- quality(embedded_data[[e]], q))
  }
}
@

This example showcases the simplicity with which different methods and quality criteria
can be combined. %
Because of the strong dependencies on parameters it is not advised to apply this
kind of analysis without tuning the parameters for each method separately. %
There is no automatized way to tune parameters in \pkg{dimRed}. %

\section{Conclusion}
\label{sec:conc}

This paper presents the \pkg{dimRed} and \pkg{coRanking} packages and
it provides a brief overview of the methods implemented therein. %
The \pkg{dimRed} package is written in the R language, one of the most popular
languages for data analysis. The package is freely available from CRAN. %
The package is object oriented and completely open source and therefore easily available
and extensible. %
Although most of the DR methods already had implementations in R,
\pkg{dimRed} adds some new methods for dimensionality reduction, and
\pkg{coRanking} adds methods for an independent quality control of DR
methods to the R ecosystem. %
DR is a widely used technique. However, due to the lack of easily usable tools,
choosing the right method for DR is complex and depends upon a variety of factors. %
The \pkg{dimRed} package aims to facilitate experimentation with different
techniques, parameters, and quality measures so that choosing the right method
becomes easier. %
The \pkg{dimRed} package wants to enable the user to objectively compare methods that
rely on very different algorithmic approaches. %
It makes the life of the programmer easier, because all methods are aggregated
in one place and there is a single interface and standardized classes to access
the functionality. %

\section{Acknowledgments}
\label{sec:ack}

We thank Dr.\ G.\ Camps-Valls and an anonymous reviewer for many useful
comments. %
This study was supported by the European Space Agency (ESA) via the Earth System
Data Lab project (\url{http://earthsystemdatacube.org}) and the EU via the H2020
project BACI, grant agreement No 640176. %

\bibliographystyle{abbrvnat}
\bibliography{bibliography}

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_embed.R
\docType{class}
\name{DrL-class}
\alias{DrL-class}
\alias{DrL}
\title{Distributed Recursive Graph Layout}
\description{
An S4 Class implementing Distributed recursive Graph Layout.
}
\details{
DrL uses a complex algorithm to avoid local minima in the graph
embedding which uses several steps.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

DrL can take the following parameters:
\describe{
  \item{ndim}{The number of dimensions, defaults to 2. Can only be 2 or 3}
  \item{knn}{Reduce the graph to keep only the neares neighbors. Defaults to 100.}
  \item{d}{The distance function to determine the weights of the graph edges. Defaults to euclidean distances.}
}
}

\section{Implementation}{

Wraps around \code{\link[igraph]{layout_with_drl}}. The parameters
maxiter, epsilon and kkconst are set to the default values and
cannot be set, this may change in a future release. The DimRed
Package adds an extra sparsity parameter by constructing a knn
graph which also may improve visualization quality.
}

\examples{
\dontrun{
if(requireNamespace(c("igraph", "coRanking"), quietly = TRUE)) {

dat <- loadDataSet("Swiss Roll", n = 200)
emb <- embed(dat, "DrL")
plot(emb, type = "2vars")
}

}

}
\references{
Martin, S., Brown, W.M., Wylie, B.N., 2007. Dr.l: Distributed Recursive
(graph) Layout (No. dRl; 002182MLTPL00). Sandia National Laboratories.
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/l1pca.R
\docType{class}
\name{PCA_L1-class}
\alias{PCA_L1-class}
\alias{PCA_L1}
\title{Principal Component Analysis with L1 error.}
\description{
S4 Class implementing PCA with L1 error.
}
\details{
PCA transforms the data so that the L2 reconstruction error is minimized or
the variance of the projected data is maximized. This is sensitive to
outliers, L1 PCA minimizes the L1 reconstruction error or maximizes the sum
of the L1 norm of the projected observations.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

PCA can take the following parameters:
\describe{
  \item{ndim}{The number of output dimensions.}
  \item{center}{logical, should the data be centered, defaults to \code{TRUE}.}
  \item{scale.}{logical, should the data be scaled, defaults to \code{FALSE}.}
  \item{fun}{character or function, the method to apply, see the \code{pcaL1} package}
  \item{\ldots}{other parameters for \code{fun}}
}
}

\section{Implementation}{


Wraps around the different methods is the \code{pcaL1} package. Because PCA
can be reduced to a simple rotation, forward and backward projection
functions are supplied.
}

\examples{
if(requireNamespace("pcaL1", quietly = TRUE)) {

dat <- loadDataSet("Iris")
emb <- embed(dat, "PCA_L1")

plot(emb, type = "2vars")
plot(inverse(emb, getData(getDimRedData((emb)))), type = "3vars")

}

}
\references{
Park, Y.W., Klabjan, D., 2016. Iteratively Reweighted Least Squares
Algorithms for L1-Norm Principal Component Analysis, in: Data Mining (ICDM),
2016 IEEE 16th International Conference On. IEEE, pp. 430-438.
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dimRedMethod-class.R
\name{dimRedMethodList}
\alias{dimRedMethodList}
\title{dimRedMethodList}
\usage{
dimRedMethodList(filter = FALSE)
}
\arguments{
\item{filter}{filter methods by methods that have their dependencies installed}
}
\value{
a character vector with the names of classes that inherit
    from \code{dimRedMethod}.
}
\description{
Get the names of all methods for dimensionality reduction.
}
\details{
Returns the name of all classes that inherit from
\code{\link{dimRedMethod-class}} to use with \code{\link{embed}}.
}
\examples{
dimRedMethodList()

}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{getData}
\alias{getData}
\title{Method getData}
\usage{
getData(object)
}
\arguments{
\item{object}{The object to be converted.}
}
\description{
Extracts the data slot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{getNDim}
\alias{getNDim}
\title{Method getNDim}
\usage{
getNDim(object, ...)
}
\arguments{
\item{object}{The object to get the dimensions from.}

\item{...}{other arguments.}
}
\description{
Extract the number of embedding dimensions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsne.R
\docType{class}
\name{tSNE-class}
\alias{tSNE-class}
\alias{tSNE}
\title{t-Distributed Stochastic Neighborhood Embedding}
\description{
An S4 Class for t-SNE.
}
\details{
t-SNE is a method that uses Kullback-Leibler divergence between the
distance matrices in high and low-dimensional space to embed the
data. The method is very well suited to visualize complex
structures in low dimensions.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

t-SNE can take the following parameters:
\describe{
  \item{d}{A distance function, defaults to euclidean distances}
  \item{perplexity}{The perplexity parameter, roughly equivalent to neighborhood size.}
  \item{theta}{Approximation for the nearest neighbour search, large values are more inaccurate.}
  \item{ndim}{The number of embedding dimensions.}
}
}

\section{Implementation}{


Wraps around \code{\link[Rtsne]{Rtsne}}, which is very well
documented. Setting \code{theta = 0} does a normal t-SNE, larger
values for \code{theta < 1} use the Barnes-Hut algorithm which
scales much nicer with data size. Larger values for perplexity take
larger neighborhoods into account.
}

\examples{
\dontrun{
dat <- loadDataSet("3D S Curve", n = 300)
emb <- embed(dat, "tSNE", perplexity = 80)
plot(emb, type = "2vars")
}
}
\references{
Maaten, L. van der, 2014. Accelerating t-SNE using Tree-Based
Algorithms. Journal of Machine Learning Research 15, 3221-3245.

van der Maaten, L., Hinton, G., 2008. Visualizing Data using
t-SNE. J. Mach. Learn. Res. 9, 2579-2605.
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastica.R
\docType{class}
\name{FastICA-class}
\alias{FastICA-class}
\alias{FastICA}
\title{Independent Component Analysis}
\description{
An S4 Class implementing the FastICA algorithm for Indepentend
Component Analysis.
}
\details{
ICA is used for blind signal separation of different sources. It is
a linear Projection.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

FastICA can take the following parameters:
\describe{
  \item{ndim}{The number of output dimensions. Defaults to \code{2}}
}
}

\section{Implementation}{

Wraps around \code{\link[fastICA]{fastICA}}. FastICA uses a very
fast approximation for negentropy to estimate statistical
independences between signals. Because it is a simple
rotation/projection, forward and backward functions can be given.
}

\examples{
if(requireNamespace("fastICA", quietly = TRUE)) {

dat <- loadDataSet("3D S Curve")
emb <- embed(dat, "FastICA", ndim = 2)
plot(getData(getDimRedData(emb)))

}
}
\references{
Hyvarinen, A., 1999. Fast and robust fixed-point algorithms for independent
component analysis. IEEE Transactions on Neural Networks 10, 626-634.
https://doi.org/10.1109/72.761722
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{distance_correlation,dimRedResult-method}
\alias{distance_correlation,dimRedResult-method}
\alias{distance_correlation}
\title{Method distance_correlation}
\usage{
\S4method{distance_correlation}{dimRedResult}(object)
}
\arguments{
\item{object}{of class dimRedResult}
}
\description{
Calculate the distance correlation between the distance matrices in
high and low dimensioal space.
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataSets.R
\name{dataSets}
\alias{dataSets}
\alias{loadDataSet}
\alias{dataSetList}
\title{Example Data Sets for dimensionality reduction}
\usage{
loadDataSet(name = dataSetList(), n = 2000, sigma = 0.05)

dataSetList()
}
\arguments{
\item{name}{A character vector that specifies the name of the data
set.}

\item{n}{In generated data sets the number of points to be
generated, else ignored.}

\item{sigma}{In generated data sets the standard deviation of the
noise added, else ignored.}
}
\value{
\code{loadDataSet} an object of class
    \code{\link{dimRedData}}. \code{dataSetList()} return a
    character string with the implemented data sets
}
\description{
A compilation of standard data sets that are often being used to
showcase dimensionality reduction techniques.
}
\details{
The argument \code{name} should be one of
\code{dataSetList()}. Partial matching is possible, see
\code{\link{match.arg}}. Generated data sets contain the internal
coordinates of the manifold in the \code{meta} slot.  Call
\code{dataSetList()} to see what data sets are available.
}
\examples{
## a list of available data sets:
dataSetList()

## Load a data set:
swissRoll <- loadDataSet("Swiss Roll")
\donttest{
if(requireNamespace("scatterplot3d", quietly = TRUE))
  plot(swissRoll, type = "3vars")
}

## Load Iris data set, partial matching:
loadDataSet("I")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nnmf.R
\docType{class}
\name{NNMF-class}
\alias{NNMF-class}
\alias{NNMF}
\title{Non-Negative Matrix Factorization}
\description{
S4 Class implementing NNMF.
}
\details{
NNMF is a method for decomposing a matrix into a smaller
 dimension such that the constraint that the data (and the
 projection) are not negative is taken into account.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

The method can take the following parameters:
\describe{
  \item{ndim}{The number of output dimensions.}
  \item{method}{character, which algorithm should be used. See
   \code{\link[NMF]{nmf}} for possible values. Defaults to
   "brunet"}
  \item{nrun}{integer, the number of times the computations are
   conducted. See \code{\link[NMF]{nmf}}}
  \item{seed}{integer, a value to control the random numbers used.}
  \item{options}{named list, other options to pass to  \code{\link[NMF]{nmf}}}
}
}

\section{Implementation}{


Wraps around \code{\link[NMF]{nmf}}. Note that the estimation uses random
 numbers. To create reproducible results, set the random number seed in the
 function call. Also, in many cases, the computations will be conducted
 in parallel using multiple cores. To disable this, use the option
 \code{.pbackend = NULL}.
}

\examples{
if(requireNamespace(c("NNMF", "MASS"), quietly = TRUE)) {

set.seed(4646)
dat <- loadDataSet("Iris")
emb <- embed(dat, "NNMF")

plot(emb)

# project new values:
nn_proj <- predict(emb, dat[1:7])
plot(nn_proj)

}
}
\references{
Lee, D.D., Seung, H.S., 1999. Learning the parts of objects by non-negative
matrix factorization. Nature 401, 788-791. https://doi.org/10.1038/44565
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nmds.R
\docType{class}
\name{nMDS-class}
\alias{nMDS-class}
\alias{nMDS}
\title{Non-Metric Dimensional Scaling}
\description{
An S4 Class implementing Non-Metric Dimensional Scaling.
}
\details{
A non-linear extension of MDS using monotonic regression
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

nMDS can take the following parameters:
\describe{
  \item{d}{A distance function.}
  \item{ndim}{The number of embedding dimensions.}
}
}

\section{Implementation}{

Wraps around the
\code{\link[vegan]{monoMDS}}. For parameters that are not
available here, the standard configuration is used.
}

\examples{
if(requireNamespace("vegan", quietly = TRUE)) {

dat <- loadDataSet("3D S Curve", n = 300)
emb <- embed(dat, "nMDS")
plot(emb, type = "2vars")

}
}
\references{
Kruskal, J.B., 1964. Nonmetric multidimensional scaling: A numerical method.
Psychometrika 29, 115-129. https://doi.org/10.1007/BF02289694
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{AUC_lnK_R_NX,dimRedResult-method}
\alias{AUC_lnK_R_NX,dimRedResult-method}
\alias{AUC_lnK_R_NX}
\title{Method AUC_lnK_R_NX}
\usage{
\S4method{AUC_lnK_R_NX}{dimRedResult}(object, weight = "inv")
}
\arguments{
\item{object}{of class dimRedResult}

\item{weight}{the weight function used, one of \code{c("inv", "log", "log10")}}
}
\description{
Calculate the Area under the R_NX(ln K), used in Lee et. al. (2015). Note
that despite the name, this does not weight the mean by the logarithm, but by
1/K. If explicit weighting by the logarithm is desired use \code{weight =
"log"} or \code{weight = "log10"}
}
\details{
The naming confusion originated from equation 17 in Lee et al (2015) and the
name of this method may change in the future to avoid confusion.
}
\references{
Lee, J.A., Peluffo-Ordonez, D.H., Verleysen, M., 2015.
  Multi-scale similarities in stochastic neighbour embedding: Reducing
  dimensionality while preserving both local and global structure.
  Neurocomputing 169, 246-261. https://doi.org/10.1016/j.neucom.2014.12.095
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R, R/dimRedData-class.R
\name{as.dimRedData}
\alias{as.dimRedData}
\alias{as.dimRedData,formula-method}
\title{Converts to dimRedData}
\usage{
as.dimRedData(formula, ...)

\S4method{as.dimRedData}{formula}(formula, data)
}
\arguments{
\item{formula}{The formula, left hand side is assigned to the meta slot right
hand side is assigned to the data slot.}

\item{...}{other arguments.}

\item{data}{Will be coerced into a \code{\link{data.frame}} with
\code{\link{as.data.frame}}}
}
\description{
Conversion functions to dimRedData.
}
\section{Methods (by class)}{
\itemize{
\item \code{formula}: Convert a \code{data.frame} to a dimRedData
object using a formula
}}

\examples{
## create a dimRedData object using a formula
as.dimRedData(Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,
              iris)[1:5]

}
\seealso{
Other dimRedData: 
\code{\link{dimRedData-class}}
}
\concept{dimRedData}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoencoder.R
\docType{class}
\name{AutoEncoder-class}
\alias{AutoEncoder-class}
\alias{AutoEncoder}
\title{AutoEncoder}
\description{
An S4 Class implementing an Autoencoder
}
\details{
Autoencoders are neural networks that try to reproduce their input. Consider
this method unstable, as the internals may still be changed.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

Autoencoder can take the following parameters:
\describe{
  \item{ndim}{The number of dimensions for reduction.}
  \item{n_hidden}{The number of neurons in the hidden
      layers, the length specifies the number of layers,
      the length must be impair, the middle number must
      be the same as ndim.}
  \item{activation}{The activation functions for the layers,
      one of "tanh", "sigmoid", "relu", "elu", everything
      else will silently be ignored and there will be no
      activation function for the layer.}
  \item{weight_decay}{the coefficient for weight decay,
      set to 0 if no weight decay desired.}
  \item{learning_rate}{The learning rate for gradient descend}
  \item{graph}{Optional: A list of bits and pieces that define the
     autoencoder in tensorflow, see details.}
  \item{keras_graph}{Optional: A list of keras layers that define
     the encoder and decoder, specifying this, will ignore all
     other topology related variables, see details.}
  \item{batchsize}{If NA, all data will be used for training,
      else only a random subset of size batchsize will be used}
  \item{n_steps}{the number of training steps.}
}
}

\section{Details}{

There are several ways to specify an autoencoder, the simplest is to pass the
number of neurons per layer in \code{n_hidden}, this must be a vector of
integers of impair length and it must be symmetric and the middle number must
be equal to \code{ndim}, For every layer an activation function can be
specified with \code{activation}.

For regularization weight decay can be specified by setting
\code{weight_decay} > 0.

Currently only a gradient descent optimizer is used, the learning rate can be
specified by setting \code{learning_rate}.
The learner can operate on batches if \code{batchsize} is not \code{NA}.
The number of steps the learner uses is specified using \code{n_steps}.
}

\section{Further training a model}{

If the model did not converge in the first training phase or training with
different data is desired, the \code{\link{dimRedResult}} object may be
passed as \code{autoencoder} parameter; In this case all topology related
parameters will be ignored.
}

\section{Using Keras layers}{

The encoder and decoder part can be specified using a list of \pkg{keras}
layers. This requires a list with two entries, \code{encoder} should contain
a LIST of keras layers WITHOUT the \code{\link[keras]{layer_input}}
that will be concatenated in order to form the encoder part.
\code{decoder} should be
defined accordingly, the output of \code{decoder} must have the same number
of dimensions as the input data.
}

\section{Using Tensorflow}{

The model can be entirely defined in \pkg{tensorflow}, it must contain a
list with the following entries:
\describe{
  \item{encoder}{A tensor that defines the encoder.}
  \item{decoder}{A tensor that defines the decoder.}
  \item{network}{A tensor that defines the reconstruction (encoder + decoder).}
  \item{loss}{A tensor that calculates the loss (network + loss function).}
  \item{in_data}{A \code{placeholder} that points to the data input of
    the network AND the encoder.}
  \item{in_decoder}{A \code{placeholder} that points to the input of
    the decoder.}
  \item{session}{A \pkg{tensorflow} \code{Session} object that holds
    the values of the tensors.}
}
}

\section{Implementation}{

Uses \pkg{tensorflow} as a backend, for details an
  problems relating tensorflow, see \url{https://tensorflow.rstudio.com}.
}

\examples{
\dontrun{
dat <- loadDataSet("3D S Curve")

emb <- embed(dat, "AutoEncoder")

# predicting is possible:
samp <- sample(floor(nrow(dat) / 10))
emb2 <- embed(dat[samp])
emb3 <- predict(emb2, dat[-samp])

plot(emb, type = "2vars")
plot(emb2, type = "2vars")
points(getData(emb3))
}

}
\seealso{
Other dimensionality reduction methods: 
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot}
\alias{plot}
\alias{plot.dimRed}
\alias{plot,dimRedData,ANY-method}
\alias{plot.dimRedData}
\alias{plot,dimRedResult,ANY-method}
\alias{plot.dimRedResult}
\title{Plotting of dimRed* objects}
\usage{
plot(x, y, ...)

\S4method{plot}{dimRedData,ANY}(
  x,
  type = "pairs",
  vars = seq_len(ncol(x@data)),
  col = seq_len(min(3, ncol(x@meta))),
  ...
)

\S4method{plot}{dimRedResult,ANY}(
  x,
  type = "pairs",
  vars = seq_len(ncol(x@data@data)),
  col = seq_len(min(3, ncol(x@data@meta))),
  ...
)
}
\arguments{
\item{x}{dimRedResult/dimRedData class, e.g. output of
embedded/loadDataSet}

\item{y}{Ignored}

\item{...}{handed over to the underlying plotting function.}

\item{type}{plot type, one of \code{c("pairs", "parpl", "2vars",
"3vars", "3varsrgl")}}

\item{vars}{the axes of the embedding to use for plotting}

\item{col}{the columns of the meta slot to use for coloring, can be
referenced as the column names or number of x@data}
}
\description{
Plots a object of class dimRedResult and dimRedData. For the
documentation of the plotting function in base see here:
\code{\link{plot.default}}.
}
\details{
Plotting functions for the classes usind in \code{dimRed}. they are
intended to give a quick overview over the results, so they are
somewhat inflexible, e.g. it is hard to modify color scales or
plotting parameters.

If you require more control over plotting, it is better to convert
the object to a \code{data.frame} first and use the standard
functions for plotting.
}
\section{Methods (by class)}{
\itemize{
\item \code{x = dimRedData,y = ANY}: Ploting of dimRedData objects

\item \code{x = dimRedResult,y = ANY}: Ploting of dimRedResult objects.
}}

\examples{
scurve = loadDataSet("3D S Curve")
if(requireNamespace("graphics", quietly = TRUE))
  plot(scurve, type = "pairs", main = "pairs plot of S curve")
if(requireNamespace("MASS", quietly = TRUE))
  plot(scurve, type = "parpl")
if(requireNamespace("graphics", quietly = TRUE))
  plot(scurve, type = "2vars", vars = c("y", "z"))
if(requireNamespace("scatterplot3d", quietly = TRUE))
  plot(scurve, type = "3vars")
if(requireNamespace("rgl", quietly = TRUE))
  plot(scurve, type = "3varsrgl")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{getPars}
\alias{getPars}
\title{Method getPars}
\usage{
getPars(object, ...)
}
\arguments{
\item{object}{The object to be converted.}

\item{...}{other arguments.}
}
\description{
Extracts the pars slot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_embed.R
\docType{class}
\name{FruchtermanReingold-class}
\alias{FruchtermanReingold-class}
\alias{FruchtermanReingold}
\title{Fruchterman Reingold Graph Layout}
\description{
An S4 Class implementing the Fruchterman Reingold Graph Layout
algorithm.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

\describe{
  \item{ndim}{The number of dimensions, defaults to 2. Can only be 2 or 3}
  \item{knn}{Reduce the graph to keep only the neares neighbors. Defaults to 100.}
  \item{d}{The distance function to determine the weights of the graph edges. Defaults to euclidean distances.}
}
}

\section{Implementation}{

Wraps around \code{\link[igraph]{layout_with_fr}}, see there for
details. The Fruchterman Reingold algorithm puts the data into
a circle and puts connected points close to each other.
}

\examples{
if(requireNamespace(c("igraph", "coRanking"), quietly = TRUE)) {

dat <- loadDataSet("Swiss Roll", n = 100)
emb <- embed(dat, "FruchtermanReingold")
plot(emb, type = "2vars")

}

}
\references{
Fruchterman, T.M.J., Reingold, E.M., 1991. Graph drawing by force-directed
placement. Softw: Pract. Exper. 21, 1129-1164.
https://doi.org/10.1002/spe.4380211102
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{quality,dimRedResult-method}
\alias{quality,dimRedResult-method}
\alias{quality}
\alias{quality.dimRedResult}
\alias{dimRedQualityList}
\title{Quality Criteria for dimensionality reduction.}
\usage{
\S4method{quality}{dimRedResult}(.data, .method = dimRedQualityList(), .mute = character(0), ...)

dimRedQualityList(filter = FALSE)
}
\arguments{
\item{.data}{object of class \code{dimRedResult}}

\item{.method}{character vector naming one of the methods}

\item{.mute}{what output from the embedding method should be muted.}

\item{...}{the pameters, internally passed as a list to the
quality method as \code{pars = list(...)}}

\item{filter}{filter methods by installed packages}
}
\value{
a number
}
\description{
A collection of functions to compute quality measures on
\code{\link{dimRedResult}} objects.
}
\section{Methods (by class)}{
\itemize{
\item \code{dimRedResult}: Calculate a quality index from a dimRedResult object.
}}

\section{Implemented methods}{


Method must be one of \code{"\link{Q_local}", "\link{Q_global}",
"\link{mean_R_NX}", "\link{total_correlation}",
"\link{cophenetic_correlation}", "\link{distance_correlation}",
"\link{reconstruction_rmse}"}
}

\section{Rank based criteria}{


\code{Q_local}, \code{Q_global}, and \code{mean_R_NX} are
quality criteria based on the Co-ranking matrix.  \code{Q_local}
and \code{Q_global} determine the local/global quality of the
embedding, while \code{mean_R_NX} determines the quality of the
overall embedding. They are parameter free and return a single
number. The object must include the original data.  The number
returns is in the range [0, 1], higher values mean a better
local/global embedding.
}

\section{Correlation based criteria}{


\code{total_correlation} calculates the sum of the mean squared
correlations of the original axes with the axes in reduced
dimensions, because some methods do not care about correlations
with axes, there is an option to rotate data in reduced space to
maximize this criterium. The number may be greater than one if more
dimensions are summed up.

\code{cophenetic_correlation} calculate the correlation between the
lower triangles of distance matrices, the correlation and distance
methods may be specified. The result is in range [-1, 1].

\code{distance_correlation} measures the independes of samples by
calculating the correlation of distances. For details see
\code{\link[energy]{dcor}}.
}

\section{Reconstruction error}{


\code{reconstruction_rmse} calculates the root mean squared error
of the reconstrucion. \code{object} requires an inverse function.
}

\examples{
\dontrun{
embed_methods <- dimRedMethodList()
quality_methods <- dimRedQualityList()
scurve <- loadDataSet("Iris")

quality_results <- matrix(NA, length(embed_methods), length(quality_methods),
                              dimnames = list(embed_methods, quality_methods))
embedded_data <- list()

for (e in embed_methods) {
  message("embedding: ", e)
  embedded_data[[e]] <- embed(scurve, e, .mute = c("message", "output"))
  for (q in quality_methods) {
    message("  quality: ", q)
    quality_results[e, q] <- tryCatch(
      quality(embedded_data[[e]], q),
      error = function (e) NA
    )
  }
}

print(quality_results)
}
}
\references{
Lueks, W., Mokbel, B., Biehl, M., Hammer, B., 2011. How
    to Evaluate Dimensionality Reduction? - Improving the
    Co-ranking Matrix. arXiv:1110.3917 [cs].

Szekely, G.J., Rizzo, M.L., Bakirov, N.K., 2007. Measuring and
    testing dependence by correlation of distances. Ann. Statist. 35,
    2769-2794. doi:10.1214/009053607000000505

Lee, J.A., Peluffo-Ordonez, D.H., Verleysen, M., 2015. Multi-scale
    similarities in stochastic neighbour embedding: Reducing
    dimensionality while preserving both local and global
    structure. Neurocomputing, 169,
    246-261. doi:10.1016/j.neucom.2014.12.095
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\author{
Guido Kraemer
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{getMeta}
\alias{getMeta}
\title{Method getMeta}
\usage{
getMeta(object, ...)
}
\arguments{
\item{object}{The object to be converted.}

\item{...}{other arguments.}
}
\description{
Extracts the meta slot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hlle.R
\docType{class}
\name{HLLE-class}
\alias{HLLE-class}
\alias{HLLE}
\title{Hessian Locally Linear Embedding}
\description{
An S4 Class implementing Hessian Locally Linear Embedding (HLLE)
}
\details{
HLLE uses local hessians to approximate the curvines and is an
extension to non-convex subsets in lowdimensional space.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

HLLE can take the following parameters:
\describe{
  \item{knn}{neighborhood size}
  \item{ndim}{number of output dimensions}
}
}

\section{Implementation}{

Own implementation, sticks to the algorithm in Donoho and Grimes
(2003). Makes use of sparsity to speed up final embedding.
}

\examples{
if(requireNamespace(c("RSpectra", "Matrix", "RANN"), quietly = TRUE)) {

dat <- loadDataSet("3D S Curve", n = 300)
emb <- embed(dat, "HLLE", knn = 15)
plot(emb, type = "2vars")

}

}
\references{
Donoho, D.L., Grimes, C., 2003. Hessian eigenmaps: Locally linear
embedding techniques for high-dimensional data. PNAS 100,
5591-5596. doi:10.1073/pnas.1031596100
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{R_NX,dimRedResult-method}
\alias{R_NX,dimRedResult-method}
\alias{R_NX}
\title{Method R_NX}
\usage{
\S4method{R_NX}{dimRedResult}(object, ndim = getNDim(object))
}
\arguments{
\item{object}{of class dimRedResult}

\item{ndim}{the number of dimensions to take from the embedded data.}
}
\description{
Calculate the R_NX score from Lee et. al. (2013) which shows the neighborhood
preservation for the Kth nearest neighbors, corrected for random point
distributions and scaled to range [0, 1].
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{reconstruction_rmse,dimRedResult-method}
\alias{reconstruction_rmse,dimRedResult-method}
\alias{reconstruction_rmse}
\title{Method reconstruction_rmse}
\usage{
\S4method{reconstruction_rmse}{dimRedResult}(object)
}
\arguments{
\item{object}{of class dimRedResult}
}
\description{
Calculate the reconstruction root mean squared error a dimensionality reduction, the method must have an inverse mapping.
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rotate.R
\name{maximize_correlation,dimRedResult-method}
\alias{maximize_correlation,dimRedResult-method}
\alias{maximize_correlation}
\title{Maximize Correlation with the Axes}
\usage{
\S4method{maximize_correlation}{dimRedResult}(
  object,
  naxes = ncol(object@data@data),
  cor_method = "pearson"
)
}
\arguments{
\item{object}{A dimRedResult object}

\item{naxes}{the number of axes to optimize for.}

\item{cor_method}{which correlation method to use}
}
\description{
Rotates the data in such a way that the correlation with the first
\code{naxes} axes is maximized.
}
\details{
Methods that do not use eigenvector decomposition, like t-SNE often
do not align the data with axes according to the correlation of
variables with the data. \code{maximize_correlation} uses the
\code{\link[optimx]{optimx}} package to rotate the data in such a
way that the original variables have maximum correlation with the
embedding axes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/isomap.R
\docType{class}
\name{Isomap-class}
\alias{Isomap-class}
\alias{Isomap}
\title{Isomap embedding}
\description{
An S4 Class implementing the Isomap Algorithm
}
\details{
The Isomap algorithm approximates a manifold using geodesic
distances on a k nearest neighbor graph. Then classical scaling is
performed on the resulting distance matrix.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

Isomap can take the following parameters:
\describe{
  \item{knn}{The number of nearest neighbors in the graph. Defaults to 50.}
  \item{ndim}{The number of embedding dimensions, defaults to 2.}
  \item{get_geod}{Should the geodesic distance matrix be kept,
                  if \code{TRUE}, access it as \code{getOtherData(x)$geod}}
}
}

\section{Implementation}{


The dimRed package uses its own implementation of Isomap which also
comes with an out of sample extension (known as landmark
Isomap). The default Isomap algorithm scales computationally not
very well, the implementation here uses \code{\link[RANN]{nn2}} for
a faster search of the nearest neighbors.  If data are too large it
may be useful to fit a subsample of the data and use the
out-of-sample extension for the other points.
}

\examples{
if(requireNamespace(c("RSpectra", "igraph", "RANN"), quietly = TRUE)) {

dat <- loadDataSet("3D S Curve", n = 500)
emb <- embed(dat, "Isomap", knn = 10)
plot(emb)

## or simpler, use embed():
samp <- sample(nrow(dat), size = 200)
emb2 <- embed(dat[samp], "Isomap", .mute = NULL, knn = 10)
emb3 <- predict(emb2, dat[-samp])

plot(emb2, type = "2vars")
plot(emb3, type = "2vars")

}
}
\references{
Tenenbaum, J.B., Silva, V. de, Langford, J.C., 2000. A Global Geometric
Framework for Nonlinear Dimensionality Reduction. Science 290, 2319-2323.
https://doi.org/10.1126/science.290.5500.2319
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{cophenetic_correlation,dimRedResult-method}
\alias{cophenetic_correlation,dimRedResult-method}
\alias{cophenetic_correlation}
\title{Method cophenetic_correlation}
\usage{
\S4method{cophenetic_correlation}{dimRedResult}(object, d = stats::dist, cor_method = "pearson")
}
\arguments{
\item{object}{of class dimRedResult}

\item{d}{the distance function to use.}

\item{cor_method}{The correlation method.}
}
\description{
Calculate the correlation between the distance matrices in high and
low dimensioal space.
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_R_NX}
\alias{plot_R_NX}
\title{plot_R_NX}
\usage{
plot_R_NX(x, ndim = NA, weight = "inv")
}
\arguments{
\item{x}{a list of \code{\link{dimRedResult}} objects. The names of the list
will appear in the legend with the AUC_lnK value.}

\item{ndim}{the number of dimensions, if \code{NA} the original number of
embedding dimensions is used, can be a vector giving the embedding
dimensionality for each single list element of \code{x}.}

\item{weight}{the weight function used for K when calculating the AUC, one of
\code{c("inv", "log", "log10")}}
}
\value{
A ggplot object, the design can be changed by appending
  \code{theme(...)}
}
\description{
Plot the R_NX curve for different embeddings. Takes a list of
\code{\link{dimRedResult}} objects as input.
Also the Area under the curve values are computed for a weighted K
(see \link{AUC_lnK_R_NX} for details) and appear in the legend.
}
\examples{
if(requireNamespace(c("RSpectra", "igraph", "RANN", "ggplot", "tidyr", "scales"), quietly = TRUE)) {
## define which methods to apply
embed_methods <- c("Isomap", "PCA")
## load test data set
data_set <- loadDataSet("3D S Curve", n = 200)
## apply dimensionality reduction
data_emb <- lapply(embed_methods, function(x) embed(data_set, x))
names(data_emb) <- embed_methods
## plot the R_NX curves:
plot_R_NX(data_emb) +
    ggplot2::theme(legend.title = ggplot2::element_blank(),
                   legend.position = c(0.5, 0.1),
                   legend.justification = c(0.5, 0.1))
}
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{mean_R_NX,dimRedResult-method}
\alias{mean_R_NX,dimRedResult-method}
\alias{mean_R_NX}
\title{Method mean_R_NX}
\usage{
\S4method{mean_R_NX}{dimRedResult}(object)
}
\arguments{
\item{object}{of class dimRedResult}
}
\description{
Calculate the mean_R_NX score to assess the quality of a dimensionality reduction.
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_embed.R
\docType{class}
\name{KamadaKawai-class}
\alias{KamadaKawai-class}
\alias{KamadaKawai}
\title{Graph Embedding via the Kamada Kawai Algorithm}
\description{
An S4 Class implementing the Kamada Kawai Algorithm for graph embedding.
}
\details{
Graph embedding algorithms se the data as a graph. Between the
nodes of the graph exist attracting and repelling forces which can
be modeled as electrical fields or springs connecting the
nodes. The graph is then forced into a lower dimensional
representation that tries to represent the forces betweent he nodes
accurately by minimizing the total energy of the attracting and
repelling forces.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

KamadaKawai can take the following parameters:
\describe{
  \item{ndim}{The number of dimensions, defaults to 2. Can only be 2 or 3}
  \item{knn}{Reduce the graph to keep only the neares neighbors. Defaults to 100.}
  \item{d}{The distance function to determine the weights of the graph edges. Defaults to euclidean distances.}
}
}

\section{Implementation}{

Wraps around \code{\link[igraph]{layout_with_kk}}. The parameters
maxiter, epsilon and kkconst are set to the default values and
cannot be set, this may change in a future release. The DimRed
Package adds an extra sparsity parameter by constructing a knn
graph which also may improve visualization quality.
}

\examples{
if(requireNamespace(c("igraph", "coRanking"), quietly = TRUE)) {

dat <- loadDataSet("Swiss Roll", n = 200)
emb <- embed(dat, "KamadaKawai")
plot(emb, type = "2vars")

}
}
\references{
Kamada, T., Kawai, S., 1989. An algorithm for drawing general undirected
graphs. Information Processing Letters 31, 7-15.
https://doi.org/10.1016/0020-0190(89)90102-6
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dimRedResult-class.R
\docType{class}
\name{dimRedResult-class}
\alias{dimRedResult-class}
\alias{dimRedResult}
\alias{predict,dimRedResult-method}
\alias{inverse,dimRedResult-method}
\alias{inverse}
\alias{as.data.frame,dimRedResult-method}
\alias{getPars,dimRedResult-method}
\alias{getNDim,dimRedResult-method}
\alias{print,dimRedResult-method}
\alias{getOrgData,dimRedResult-method}
\alias{getDimRedData,dimRedResult-method}
\alias{ndims,dimRedResult-method}
\alias{getOtherData,dimRedResult-method}
\title{Class "dimRedResult"}
\usage{
\S4method{predict}{dimRedResult}(object, xnew)

\S4method{inverse}{dimRedResult}(object, ynew)

\S4method{as.data.frame}{dimRedResult}(
  x,
  org.data.prefix = "org.",
  meta.prefix = "meta.",
  data.prefix = ""
)

\S4method{getPars}{dimRedResult}(object)

\S4method{getNDim}{dimRedResult}(object)

\S4method{print}{dimRedResult}(x)

\S4method{getOrgData}{dimRedResult}(object)

\S4method{getDimRedData}{dimRedResult}(object)

\S4method{ndims}{dimRedResult}(object)

\S4method{getOtherData}{dimRedResult}(object)
}
\arguments{
\item{object}{Of class \code{dimRedResult}}

\item{xnew}{new data, of type \code{\link{dimRedData}}}

\item{ynew}{embedded data, of type \code{\link{dimRedData}}}

\item{x}{Of class \code{dimRedResult}}

\item{org.data.prefix}{Prefix for the columns of the org.data slot.}

\item{meta.prefix}{Prefix for the columns of \code{x@data@meta}.}

\item{data.prefix}{Prefix for the columns of \code{x@data@data}.}
}
\description{
A class to hold the results of of a dimensionality reduction.
}
\section{Methods (by generic)}{
\itemize{
\item \code{predict}: apply a trained method to new data, does not work
with all methods, will give an error if there is no \code{apply}.
In some cases the apply function may only be an approximation.

\item \code{inverse}: inverse transformation of embedded data, does not
work with all methods, will give an error if there is no \code{inverse}.
In some cases the apply function may only be an approximation.

\item \code{as.data.frame}: convert to \code{data.frame}

\item \code{getPars}: Get the parameters with which the method
was called.

\item \code{getNDim}: Get the number of embedding dimensions.

\item \code{print}: Method for printing.

\item \code{getOrgData}: Get the original data and meta.data

\item \code{getDimRedData}: Get the embedded data

\item \code{ndims}: Extract the number of embedding dimensions.

\item \code{getOtherData}: Get other data produced by the method
}}

\section{Slots}{

\describe{
\item{\code{data}}{Output data of class dimRedData.}

\item{\code{org.data}}{original data, a matrix.}

\item{\code{apply}}{a function to apply the method to out-of-sampledata,
may not exist.}

\item{\code{inverse}}{a function to calculate the original coordinates from
reduced space, may not exist.}

\item{\code{has.org.data}}{logical, if the original data is included in the object.}

\item{\code{has.apply}}{logical, if a forward method is exists.}

\item{\code{has.inverse}}{logical if an inverse method exists.}

\item{\code{method}}{saves the method used.}

\item{\code{pars}}{saves the parameters used.}

\item{\code{other.data}}{other data produced by the method, e.g. a distance matrix.}
}}

\examples{
## Create object by embedding data
iris.pca <- embed(loadDataSet("Iris"), "PCA")

## Convert the result to a data.frame
head(as(iris.pca, "data.frame"))
head(as.data.frame(iris.pca))

## There are no nameclashes to avoid here:
head(as.data.frame(iris.pca,
                   org.data.prefix = "",
                   meta.prefix     = "",
                   data.prefix     = ""))

## Print it more or less nicely:
print(iris.pca)

## Get the embedded data as a dimRedData object:
getDimRedData(iris.pca)

## Get the original data including meta information:
getOrgData(iris.pca)

## Get the number of variables:
ndims(iris.pca)

}
\concept{dimRedResult}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_info.R
\name{getRotationMatrix}
\alias{getRotationMatrix}
\title{getRotationMatrix}
\usage{
getRotationMatrix(x)
}
\arguments{
\item{x}{of type \code{\link{dimRedResult}}}
}
\value{
a matrix
}
\description{
Extract the rotation matrix from \code{\link{dimRedResult}} objects derived from PCA and FastICA
}
\details{
The data has to be pre-processed the same way as the method does, e.g.
centering and/or scaling.
}
\examples{
dat <- loadDataSet("Iris")

pca <- embed(dat, "PCA")
rot_pca <- getRotationMatrix(pca)
scale(getData(dat), TRUE, FALSE) \%*\% rot_pca - getData(getDimRedData(pca))


if(requireNamespace("fastICA", quietly = TRUE)) {
  ica <- embed(dat, "FastICA")
  rot_ica <- getRotationMatrix(ica)
  scale(getData(dat), TRUE, FALSE) \%*\% rot_ica - getData(getDimRedData(ica))
}


}
\concept{convenience functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leim.R
\docType{class}
\name{LaplacianEigenmaps-class}
\alias{LaplacianEigenmaps-class}
\alias{LaplacianEigenmaps}
\title{Laplacian Eigenmaps}
\description{
An S4 Class implementing Laplacian Eigenmaps
}
\details{
Laplacian Eigenmaps use a kernel and were originally developed to
separate non-convex clusters under the name spectral clustering.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

\code{LaplacianEigenmaps} can take the following parameters:
\describe{
  \item{ndim}{the number of output dimensions.}

  \item{sparse}{A character vector specifying hot to make the graph
   sparse, \code{"knn"} means that a K-nearest neighbor graph is
   constructed, \code{"eps"} an epsilon neighborhood graph is
   constructed, else a dense distance matrix is used.}

  \item{knn}{The number of nearest neighbors to use for the knn graph.}
  \item{eps}{The distance for the epsilon neighborhood graph.}

  \item{t}{Parameter for the transformation of the distance matrix
  by \eqn{w=exp(-d^2/t)}, larger values give less weight to
  differences in distance, \code{t == Inf} treats all distances != 0 equally.}
  \item{norm}{logical, should the normed laplacian be used?}
}
}

\section{Implementation}{

Wraps around \code{\link[loe]{spec.emb}}.
}

\examples{
if(requireNamespace(c("loe", "RSpectra", "Matrix"), quietly = TRUE)) {

dat <- loadDataSet("3D S Curve")
emb <- embed(dat, "LaplacianEigenmaps")
plot(emb@data@data)

}
}
\references{
Belkin, M., Niyogi, P., 2003. Laplacian Eigenmaps for
Dimensionality Reduction and Data Representation. Neural
Computation 15, 1373.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{getOrgData}
\alias{getOrgData}
\title{Method getOrgData}
\usage{
getOrgData(object, ...)
}
\arguments{
\item{object}{The object to extract data from.}

\item{...}{other arguments.}
}
\description{
Extract the Original data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dimRedData-class.R
\docType{class}
\name{dimRedData-class}
\alias{dimRedData-class}
\alias{dimRedData}
\alias{as.data.frame,dimRedData-method}
\alias{getData,dimRedData-method}
\alias{getMeta,dimRedData-method}
\alias{nrow,dimRedData-method}
\alias{[,dimRedData,ANY,ANY,ANY-method}
\alias{ndims,dimRedData-method}
\title{Class "dimRedData"}
\usage{
\S4method{as.data.frame}{dimRedData}(x, meta.prefix = "meta.", data.prefix = "")

\S4method{getData}{dimRedData}(object)

\S4method{getMeta}{dimRedData}(object)

\S4method{nrow}{dimRedData}(x)

\S4method{[}{dimRedData,ANY,ANY,ANY}(x, i)

\S4method{ndims}{dimRedData}(object)
}
\arguments{
\item{x}{Of class dimRedData}

\item{meta.prefix}{Prefix for the columns of the meta data names.}

\item{data.prefix}{Prefix for the columns of the variable names.}

\item{object}{Of class dimRedData.}

\item{i}{a valid index for subsetting rows.}
}
\description{
A class to hold data for dimensionality reduction and methods.
}
\details{
The class hast two slots, \code{data} and \code{meta}. The
\code{data} slot contains a \code{numeric matrix} with variables in
columns and observations in rows. The \code{meta} slot may contain
a \code{data.frame} with additional information. Both slots need to
have the same number of rows or the \code{meta} slot needs to
contain an empty \code{data.frame}.

See examples for easy conversion from and to \code{data.frame}.

For plotting functions see \code{\link{plot.dimRedData}}.
}
\section{Methods (by generic)}{
\itemize{
\item \code{as.data.frame}: convert to data.frame

\item \code{getData}: Get the data slot.

\item \code{getMeta}: Get the meta slot.

\item \code{nrow}: Get the number of observations.

\item \code{[}: Subset rows.

\item \code{ndims}: Extract the number of Variables from the data.
}}

\section{Slots}{

\describe{
\item{\code{data}}{of class \code{matrix}, holds the data, observations in
rows, variables in columns}

\item{\code{meta}}{of class \code{data.frame}, holds meta data such as
classes, internal manifold coordinates, or simply additional
data of the data set. Must have the same number of rows as the
\code{data} slot or be an empty data frame.}
}}

\examples{
## Load an example data set:
s3d <- loadDataSet("3D S Curve")

## Create using a constructor:

### without meta information:
dimRedData(iris[, 1:4])

### with meta information:
dimRedData(iris[, 1:4], iris[, 5])

### using slot names:
dimRedData(data = iris[, 1:4], meta = iris[, 5])

## Convert to a dimRedData objects:
Iris <- as(iris[, 1:4], "dimRedData")

## Convert to data.frame:
head(as(s3d, "data.frame"))
head(as.data.frame(s3d))
head(as.data.frame(as(iris[, 1:4], "dimRedData")))

## Extract slots:
head(getData(s3d))
head(getMeta(s3d))

## Get the number of observations:
nrow(s3d)

## Subset:
s3d[1:5, ]

## Shuffle data:
s3 <- s3d[nrow(s3d)]

## Get the number of variables:
ndims(s3d)

}
\seealso{
Other dimRedData: 
\code{\link{as.dimRedData}()}

Other dimRedData: 
\code{\link{as.dimRedData}()}
}
\concept{dimRedData}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/umap.R
\docType{class}
\name{UMAP-class}
\alias{UMAP-class}
\alias{UMAP}
\title{Umap embedding}
\description{
An S4 Class implementing the UMAP algorithm
}
\details{
Uniform Manifold Approximation is a gradient descend based algorithm that
gives results similar to t-SNE, but scales better with the number of points.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{


UMAP can take the follwing parameters:
\describe{
  \item{ndim}{The number of embedding dimensions.}
  \item{knn}{The number of neighbors to be used.}
  \item{d}{The distance metric to use.}
  \item{method}{\code{"naive"} for an R implementation, \code{"python"}
    for the reference implementation.}
}

Other method parameters can also be passed, see
\code{\link[umap]{umap.defaults}} for details. The ones above have been
standardized for the use with \code{dimRed} and will get automatically
translated for \code{\link[umap]{umap}}.
}

\section{Implementation}{


The dimRed package wraps the \code{\link[umap]{umap}} packages which provides
an implementation in pure R and also a wrapper around the original python
package \code{umap-learn} (https://github.com/lmcinnes/umap/). This requires
\code{umap-learn} version 0.4 installed, at the time of writing, there is
already \code{umap-learn} 0.5 but it is not supported by the R package
\code{\link[umap]{umap}}.

The \code{"naive"} implementation is a pure R implementation and considered
experimental at the point of writing this, it is also much slower than the
python implementation.

The \code{"python"} implementation is the reference implementation used by
McInees et. al. (2018). It requires the \code{\link[reticulate]{reticulate}}
package for the interaction with python and the python package
\code{umap-learn} installed (use \code{pip install umap-learn}).
}

\examples{
\dontrun{
dat <- loadDataSet("3D S Curve", n = 300)
emb <- embed(dat, "UMAP", .mute = NULL, knn = 10)
plot(emb, type = "2vars")
}

}
\references{
McInnes, Leland, and John Healy.
"UMAP: Uniform Manifold Approximation and Projection for Dimension Reduction."
https://arxiv.org/abs/1802.03426
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{ndims}
\alias{ndims}
\title{Method ndims}
\usage{
ndims(object, ...)
}
\arguments{
\item{object}{To extract the number of dimensions from.}

\item{...}{Arguments for further methods}
}
\description{
Extract the number of dimensions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kpca.R
\docType{class}
\name{kPCA-class}
\alias{kPCA-class}
\alias{kPCA}
\title{Kernel PCA}
\description{
An S4 Class implementing Kernel PCA
}
\details{
Kernel PCA is a nonlinear extension of PCA using kernel methods.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

Kernel PCA can take the following parameters:
\describe{
  \item{ndim}{the number of output dimensions, defaults to 2}
  \item{kernel}{The kernel function, either as a function or a
  character vector with the name of the kernel. Defaults to
  \code{"rbfdot"}}
  \item{kpar}{A list with the parameters for the kernel function,
    defaults to \code{list(sigma = 0.1)}}
}

The most comprehensive collection of kernel functions can be found in
\code{\link[kernlab]{kpca}}. In case the function does not take any
parameters \code{kpar} has to be an empty list.
}

\section{Implementation}{


Wraps around \code{\link[kernlab]{kpca}}, but provides additionally
forward and backward projections.
}

\examples{
\dontrun{
if(requireNamespace("kernlab", quietly = TRUE)) {

dat <- loadDataSet("3D S Curve")
emb <- embed(dat, "kPCA")
plot(emb, type = "2vars")
}

}
}
\references{
Sch\"olkopf, B., Smola, A., M\"uller, K.-R., 1998. Nonlinear Component Analysis
as a Kernel Eigenvalue Problem. Neural Computation 10, 1299-1319.
https://doi.org/10.1162/089976698300017467
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{installSuggests}
\alias{installSuggests}
\title{getSuggests}
\usage{
installSuggests(...)
}
\arguments{
\item{...}{additional options passed to install.packages.}
}
\description{
Install packages wich are suggested by dimRed.
}
\details{
By default dimRed will not install all the dependencies, because
there are quite a lot and in case some of them are not available
for your platform you will not be able to install dimRed without
problems.

To solve this I provide a function which automatically installes
all the suggested packages.
}
\examples{
\dontrun{
installSuggests()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{Q_global,dimRedResult-method}
\alias{Q_global,dimRedResult-method}
\alias{Q_global}
\title{Method Q_global}
\usage{
\S4method{Q_global}{dimRedResult}(object)
}
\arguments{
\item{object}{of class dimRedResult}
}
\description{
Calculate the Q_global score to assess the quality of a dimensionality reduction.
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{makeKNNgraph}
\alias{makeKNNgraph}
\title{makeKNNgraph}
\usage{
makeKNNgraph(x, k, eps = 0, diag = FALSE)
}
\arguments{
\item{x}{data, a matrix, observations in rows, dimensions in
columns}

\item{k}{the number of nearest neighbors.}

\item{eps}{number, if \code{eps > 0} the KNN search is approximate,
see \code{\link[RANN]{nn2}}}

\item{diag}{logical, if \code{TRUE} every edge of the returned
graph will have an edge with weight \code{0} to itself.}
}
\value{
an object of type \code{\link[igraph]{igraph}} with edge
    weight being the distances.
}
\description{
Create a K-nearest neighbor graph from data x. Uses
\code{\link[RANN]{nn2}} as a fast way to find the neares neighbors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{reconstruction_error,dimRedResult-method}
\alias{reconstruction_error,dimRedResult-method}
\alias{reconstruction_error}
\title{Method reconstruction_error}
\usage{
\S4method{reconstruction_error}{dimRedResult}(object, n = seq_len(ndims(object)), error_fun = "rmse")
}
\arguments{
\item{object}{of class dimRedResult}

\item{n}{a positive integer or vector of integers \code{<= ndims(object)}}

\item{error_fun}{a function or string indicating an error function, if
indication a function it must take to matrices of the same size and return
a scalar.}
}
\value{
a vector of number with the same length as \code{n} with the
}
\description{
Calculate the error using only the first \code{n} dimensions of the embedded
data. \code{error_fun} can either be one of \code{c("rmse", "mae")} to
calculate the root mean square error or the mean absolute error respectively,
or a function that takes to equally sized vectors as input and returns a
single number as output.
}
\examples{
\dontrun{
ir <- loadDataSet("Iris")
ir.drr <- embed(ir, "DRR", ndim = ndims(ir))
ir.pca <- embed(ir, "PCA", ndim = ndims(ir))

rmse <- data.frame(
  rmse_drr = reconstruction_error(ir.drr),
  rmse_pca = reconstruction_error(ir.pca)
)

matplot(rmse, type = "l")
plot(ir)
plot(ir.drr)
plot(ir.pca)
}
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\author{
Guido Kraemer
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{getOtherData}
\alias{getOtherData}
\title{Method getOtherData}
\usage{
getOtherData(object, ...)
}
\arguments{
\item{object}{The object to extract data from.}

\item{...}{other arguments.}
}
\description{
Extract other data produced by a dimRedMethod
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{Q_local,dimRedResult-method}
\alias{Q_local,dimRedResult-method}
\alias{Q_local}
\title{Method Q_local}
\usage{
\S4method{Q_local}{dimRedResult}(object, ndim = getNDim(object))
}
\arguments{
\item{object}{of class dimRedResult.}

\item{ndim}{use the first ndim columns of the embedded data for calculation.}
}
\description{
Calculate the Q_local score to assess the quality of a dimensionality reduction.
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\docType{class}
\name{PCA-class}
\alias{PCA-class}
\alias{PCA}
\title{Principal Component Analysis}
\description{
S4 Class implementing PCA.
}
\details{
PCA transforms the data in orthogonal components so that the first
axis accounts for the larges variance in the data, all the
following axes account for the highest variance under the
constraint that they are orthogonal to the preceding axes.  PCA is
sensitive to the scaling of the variables. PCA is by far the
fastest and simples method of dimensionality reduction and should
probably always be applied as a baseline if other methods are tested.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

PCA can take the following parameters:
\describe{
  \item{ndim}{The number of output dimensions.}
  \item{center}{logical, should the data be centered, defaults to \code{TRUE}.}
  \item{scale.}{logical, should the data be scaled, defaults to \code{FALSE}.}
}
}

\section{Implementation}{


Wraps around \code{\link{prcomp}}. Because PCA can be reduced to a
simple rotation, forward and backward projection functions are
supplied.
}

\examples{
dat <- loadDataSet("Iris")
emb <- embed(dat, "PCA")

plot(emb, type = "2vars")
if(requireNamespace("scatterplot3d", quietly = TRUE))
  plot(inverse(emb, getDimRedData(emb)), type = "3vars")

}
\references{
Pearson, K., 1901. On lines and planes of closest fit to systems of points in
space. Philosophical Magazine 2, 559-572.
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{getDimRedData}
\alias{getDimRedData}
\title{Method getDimRedData}
\usage{
getDimRedData(object, ...)
}
\arguments{
\item{object}{The object to extract data from.}

\item{...}{other arguments.}
}
\description{
Extract dimRedData.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dimRedMethod-class.R
\docType{class}
\name{dimRedMethod-class}
\alias{dimRedMethod-class}
\title{Class "dimRedMethod"}
\description{
A virtual class "dimRedMethod" to serve as a template to implement
methods for dimensionality reduction.
}
\details{
Implementations of dimensionality reductions should inherit from
this class.

The \code{fun} slot should be a function that takes three arguments
\describe{
  \item{data}{An object of class \code{\link{dimRedData}}.}
  \item{pars}{A list with the standard parameters.}
  \item{keep.org.data}{Logical. If the original data should be kept in the output.}
}
and returns an object of class \code{\link{dimRedResult}}.

The \code{stdpars} slot should take a list that contains standard
parameters for the implemented methods.

This way the method can be called by \code{embed(data, "method-name",
...)}, where \code{...} can be used to to change single parameters.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding.}

\item{\code{stdpars}}{A list with the default parameters for the \code{fun} slot.}

\item{\code{requires}}{A vector with all packages R packages that need to be
installed to run the method. In some occasions a method may work without
one of the packages. Does not include Python dependencies such as
Tensorflow. Used to auto skip tests}
}}

\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dimRed.R
\docType{package}
\name{dimRed-package}
\alias{dimRed}
\alias{dimRed-package}
\title{The dimRed package}
\description{
This package simplifies dimensionality reduction in R by
  providing a framework of S4 classes and methods. dimRed collects
  dimensionality reduction methods that are implemented in R and implements
  others. It gives them a common interface and provides plotting
  functions for visualization and functions for quality assessment.

Funding provided by the Department for Biogeochemical Integration,
Empirical Inference of the Earth System Group, at the Max Plack
Institute for Biogeochemistry, Jena.
}
\references{
Lee, J.A., Renard, E., Bernard, G., Dupont, P., Verleysen, M.,
2013. Type 1 and 2 mixtures of Kullback-Leibler divergences as cost
functions in dimensionality reduction based on similarity
preservation. Neurocomputing. 112,
92-107. doi:10.1016/j.neucom.2012.12.036

Lee, J.A., Lee, J.A., Verleysen, M., 2008. Rank-based quality
assessment of nonlinear dimensionality reduction. Proceedings of
ESANN 2008 49-54.

Chen, L., Buja, A., 2006. Local Multidimensional Scaling for
Nonlinear Dimension Reduction, Graph Layout and Proximity Analysis.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://www.guido-kraemer.com/software/dimred/}
  \item Report bugs at \url{https://github.com/gdkrmr/dimRed/issues}
}

}
\author{
\strong{Maintainer}: Guido Kraemer \email{guido.kraemer@uni-leipzig.de}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drr.R
\docType{class}
\name{DRR-class}
\alias{DRR-class}
\alias{DRR}
\title{Dimensionality Reduction via Regression}
\description{
An S4 Class implementing Dimensionality Reduction via Regression (DRR).
}
\details{
DRR is a non-linear extension of PCA that uses Kernel Ridge regression.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

DRR can take the following parameters:
\describe{
  \item{ndim}{The number of dimensions}
  \item{lambda}{The regularization parameter for the ridge
  regression.}
  \item{kernel}{The kernel to use for KRR, defaults to
  \code{"rbfdot"}.}
  \item{kernel.pars}{A list with kernel parameters, elements depend
  on the kernel used, \code{"rbfdot"} uses \code{"sigma"}.}
  \item{pca}{logical, should an initial pca step be performed,
  defaults to \code{TRUE}.}
  \item{pca.center}{logical, should the data be centered before the
  pca step. Defaults to \code{TRUE}.}
  \item{pca.scale}{logical, should the data be scaled before the
  pca ste. Defaults to \code{FALSE}.}
  \item{fastcv}{logical, should \code{\link[CVST]{fastCV}} from the
  CVST package be used instead of normal cross-validation.}
  \item{fastcv.test}{If \code{fastcv = TRUE}, separate test data set for fastcv.}
  \item{cv.folds}{if \code{fastcv = FALSE}, specifies the number of
  folds for crossvalidation.}
  \item{fastkrr.nblocks}{integer, higher values sacrifice numerical
  accuracy for speed and less memory, see below for details.}
  \item{verbose}{logical, should the cross-validation results be
  printed out.}
}
}

\section{Implementation}{

Wraps around \code{\link[DRR]{drr}}, see there for details. DRR is
a non-linear extension of principal components analysis using Kernel
Ridge Regression (KRR, details see \code{\link[CVST]{constructKRRLearner}}
and \code{\link[DRR]{constructFastKRRLearner}}). Non-linear
regression is used to explain more variance than PCA. DRR provides
an out-of-sample extension and a backward projection.

The most expensive computations are matrix inversions therefore the
implementation profits a lot from a multithreaded BLAS library.
The best parameters for each KRR are determined by cross-validaton
over all parameter combinations of \code{lambda} and
\code{kernel.pars}, using less parameter values will speed up
computation time. Calculation of KRR can be accelerated by
increasing \code{fastkrr.nblocks}, it should be smaller than
n^{1/3} up to sacrificing some accuracy, for details see
\code{\link[DRR]{constructFastKRRLearner}}. Another way to speed up
is to use \code{pars$fastcv = TRUE} which might provide a more
efficient way to search the parameter space but may also miss the
global maximum, I have not ran tests on the accuracy of this method.
}

\examples{
\dontrun{
if(requireNamespace(c("kernlab", "DRR"), quietly = TRUE)) {

dat <- loadDataSet("variable Noise Helix", n = 200)[sample(200)]

emb <- embed(dat, "DRR", ndim = 3)

plot(dat, type = "3vars")
plot(emb, type = "3vars")

# We even have function to reconstruct, also working for only the first few dimensions
rec <- inverse(emb, getData(getDimRedData(emb))[, 1, drop = FALSE])
plot(rec, type = "3vars")
}

}

}
\references{
Laparra, V., Malo, J., Camps-Valls, G.,
    2015. Dimensionality Reduction via Regression in Hyperspectral
    Imagery. IEEE Journal of Selected Topics in Signal Processing
    9, 1026-1036. doi:10.1109/JSTSP.2015.2417833
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lle.R
\docType{class}
\name{LLE-class}
\alias{LLE-class}
\alias{LLE}
\title{Locally Linear Embedding}
\description{
An S4 Class implementing Locally Linear Embedding (LLE)
}
\details{
LLE approximates the points in the manifold by linear combination
of its neighbors. These linear combinations are the same inside the
manifold and in highdimensional space.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

LLE can take the following parameters:
\describe{
  \item{knn}{the number of neighbors for the knn graph., defaults to 50.}
  \item{ndim}{the number of embedding dimensions, defaults to 2.}
}
}

\section{Implementation}{

Wraps around \code{\link[lle]{lle}}, only
exposes the parameters \code{k} and \code{m}.
}

\examples{
dat <- loadDataSet("3D S Curve", n = 500)

## directy use the S4 class:
lle <- LLE()
emb <- lle@fun(dat, lle@stdpars)

## using embed():
emb2 <- embed(dat, "LLE", knn = 45)

plot(emb, type = "2vars")
plot(emb2, type = "2vars")

}
\references{
Roweis, S.T., Saul, L.K., 2000. Nonlinear Dimensionality Reduction
by Locally Linear Embedding. Science 290,
2323-2326. doi:10.1126/science.290.5500.2323
}
\seealso{
Other dimensionality reduction methods: \code{\link{AutoEncoder-class}},
  \code{\link{DRR-class}},
  \code{\link{DiffusionMaps-class}},
  \code{\link{DrL-class}}, \code{\link{FastICA-class}},
  \code{\link{FruchtermanReingold-class}},
  \code{\link{HLLE-class}}, \code{\link{Isomap-class}},
  \code{\link{KamadaKawai-class}}, \code{\link{MDS-class}},
  \code{\link{NNMF-class}}, \code{\link{PCA-class}},
  \code{\link{PCA_L1-class}}, \code{\link{UMAP-class}},
  \code{\link{dimRedMethod-class}},
  \code{\link{dimRedMethodList}}, \code{\link{kPCA-class}},
  \code{\link{nMDS-class}}, \code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{print}
\alias{print}
\title{Method print}
\usage{
print(x, ...)
}
\arguments{
\item{x}{The object to be printed.}

\item{...}{Other arguments for printing.}
}
\description{
Imports the print method into the package namespace.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mds.R
\docType{class}
\name{MDS-class}
\alias{MDS-class}
\alias{MDS}
\title{Metric Dimensional Scaling}
\description{
An S4 Class implementing classical scaling (MDS).
}
\details{
MDS tries to maintain distances in high- and low-dimensional space,
it has the advantage over PCA that arbitrary distance functions can
be used, but it is computationally more demanding.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

MDS can take the following parameters:
\describe{
  \item{ndim}{The number of dimensions.}
  \item{d}{The function to calculate the distance matrix from the input coordinates, defaults to euclidean distances.}
}
}

\section{Implementation}{


Wraps around \code{\link[stats]{cmdscale}}. The implementation also
provides an out-of-sample extension which is not completely
optimized yet.
}

\examples{
\dontrun{
dat <- loadDataSet("3D S Curve")
emb <- embed(dat, "MDS")
plot(emb, type = "2vars")

# a "manual" kPCA:
emb2 <- embed(dat, "MDS", d = function(x) exp(stats::dist(x)))
plot(emb2, type = "2vars")

# a "manual", more customizable, and slower Isomap:
emb3 <- embed(dat, "MDS", d = function(x) vegan::isomapdist(vegan::vegdist(x, "manhattan"), k = 20))
plot(emb3)

}
}
\references{
Torgerson, W.S., 1952. Multidimensional scaling: I. Theory and method.
Psychometrika 17, 401-419. https://doi.org/10.1007/BF02288916
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DiffusionMaps-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{total_correlation,dimRedResult-method}
\alias{total_correlation,dimRedResult-method}
\alias{total_correlation}
\title{Method total_correlation}
\usage{
\S4method{total_correlation}{dimRedResult}(
  object,
  naxes = ndims(object),
  cor_method = "pearson",
  is.rotated = FALSE
)
}
\arguments{
\item{object}{of class dimRedResult}

\item{naxes}{the number of axes to use for optimization.}

\item{cor_method}{the correlation method to use.}

\item{is.rotated}{if FALSE the object is rotated.}
}
\description{
Calculate the total correlation of the variables with the axes to
assess the quality of a dimensionality reduction.
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffmap.R
\docType{class}
\name{DiffusionMaps-class}
\alias{DiffusionMaps-class}
\alias{DiffusionMaps}
\title{Diffusion Maps}
\description{
An S4 Class implementing Diffusion Maps
}
\details{
Diffusion Maps uses a diffusion probability matrix to robustly
approximate a manifold.
}
\section{Slots}{

\describe{
\item{\code{fun}}{A function that does the embedding and returns a
dimRedResult object.}

\item{\code{stdpars}}{The standard parameters for the function.}
}}

\section{General usage}{

Dimensionality reduction methods are S4 Classes that either be used
directly, in which case they have to be initialized and a full
list with parameters has to be handed to the \code{@fun()}
slot, or the method name be passed to the embed function and
parameters can be given to the \code{...}, in which case
missing parameters will be replaced by the ones in the
\code{@stdpars}.
}

\section{Parameters}{

Diffusion Maps can take the following parameters:
\describe{
  \item{d}{a function transforming a matrix row wise into a
    distance matrix or \code{dist} object,
    e.g. \code{\link[stats]{dist}}.}
  \item{ndim}{The number of dimensions}
  \item{eps}{The epsilon parameter that determines the
     diffusion weight matrix from a distance matrix \code{d},
     \eqn{exp(-d^2/eps)}, if set to \code{"auto"} it will
     be set to the median distance to the 0.01*n nearest
     neighbor.}
  \item{t}{Time-scale parameter. The recommended value, 0,
     uses multiscale geometry.}
  \item{delta}{Sparsity cut-off for the symmetric graph Laplacian,
    a higher value results in more sparsity and faster calculation.
    The predefined value is 10^-5.}
}
}

\section{Implementation}{

Wraps around \code{\link[diffusionMap]{diffuse}}, see there for
details. It uses the notation of Richards et al. (2009) which is
slightly different from the one in the original paper (Coifman and
Lafon, 2006) and there is no \eqn{\alpha} parameter.
There is also an out-of-sample extension, see examples.
}

\examples{
if(requireNamespace("diffusionMap", quietly = TRUE)) {
dat <- loadDataSet("3D S Curve", n = 300)
emb <- embed(dat, "DiffusionMaps")

plot(emb, type = "2vars")

# predicting is possible:
samp <- sample(floor(nrow(dat) / 10))
emb2 <- embed(dat[samp])
emb3 <- predict(emb2, dat[-samp])

plot(emb2, type = "2vars")
points(getData(emb3))
}
}
\references{
Richards, J.W., Freeman, P.E., Lee, A.B., Schafer,
    C.M., 2009. Exploiting Low-Dimensional Structure in
    Astronomical Spectra. ApJ 691,
    32. doi:10.1088/0004-637X/691/1/32

Coifman, R.R., Lafon, S., 2006. Diffusion maps. Applied and
    Computational Harmonic Analysis 21,
    5-30. doi:10.1016/j.acha.2006.04.006
}
\seealso{
Other dimensionality reduction methods: 
\code{\link{AutoEncoder-class}},
\code{\link{DRR-class}},
\code{\link{DrL-class}},
\code{\link{FastICA-class}},
\code{\link{FruchtermanReingold-class}},
\code{\link{HLLE-class}},
\code{\link{Isomap-class}},
\code{\link{KamadaKawai-class}},
\code{\link{MDS-class}},
\code{\link{NNMF-class}},
\code{\link{PCA-class}},
\code{\link{PCA_L1-class}},
\code{\link{UMAP-class}},
\code{\link{dimRedMethod-class}},
\code{\link{dimRedMethodList}()},
\code{\link{kPCA-class}},
\code{\link{nMDS-class}},
\code{\link{tSNE-class}}
}
\concept{dimensionality reduction methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/embed.R
\name{embed}
\alias{embed}
\alias{embed,formula-method}
\alias{embed,ANY-method}
\alias{embed,dimRedData-method}
\title{dispatches the different methods for dimensionality reduction}
\usage{
embed(.data, ...)

\S4method{embed}{formula}(
  .formula,
  .data,
  .method = dimRedMethodList(),
  .mute = character(0),
  .keep.org.data = TRUE,
  ...
)

\S4method{embed}{ANY}(
  .data,
  .method = dimRedMethodList(),
  .mute = character(0),
  .keep.org.data = TRUE,
  ...
)

\S4method{embed}{dimRedData}(
  .data,
  .method = dimRedMethodList(),
  .mute = character(0),
  .keep.org.data = TRUE,
  ...
)
}
\arguments{
\item{.data}{object of class \code{\link{dimRedData}}, will be converted to
be of class \code{\link{dimRedData}} if necessary; see examples for
details.}

\item{...}{the parameters, internally passed as a list to the dimensionality
reduction method as \code{pars = list(...)}}

\item{.formula}{a formula, see \code{\link{as.dimRedData}}.}

\item{.method}{character vector naming one of the dimensionality reduction
techniques.}

\item{.mute}{a character vector containing the elements you want to mute
(\code{c("message", "output")}), defaults to \code{character(0)}.}

\item{.keep.org.data}{\code{TRUE}/\code{FALSE} keep the original data.}
}
\value{
an object of class \code{\link{dimRedResult}}
}
\description{
wraps around all dimensionality reduction functions.
}
\details{
Method must be one of \code{\link{dimRedMethodList}()}, partial matching
is performed. All parameters start with a dot, to avoid clashes
with partial argument matching (see the R manual section 4.3.2), if
there should ever occur any clashes in the arguments, call the
function with all arguments named, e.g. \code{embed(.data = dat,
.method = "mymethod", .d = "some parameter")}.
}
\section{Methods (by class)}{
\itemize{
\item \code{formula}: embed a data.frame using a formula.

\item \code{ANY}: Embed anything as long as it can be coerced to
\code{\link{dimRedData}}.

\item \code{dimRedData}: Embed a dimRedData object
}}

\examples{
## embed a data.frame using a formula:
as.data.frame(
  embed(Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,
        iris, "PCA")
)

## embed a data.frame and return a data.frame
as.data.frame(embed(iris[, 1:4], "PCA"))

## embed a matrix and return a data.frame
as.data.frame(embed(as.matrix(iris[, 1:4]), "PCA"))

\dontrun{
## embed dimRedData objects
embed_methods <- dimRedMethodList()
quality_methods <- dimRedQualityList()
dataset <- loadDataSet("Iris")

quality_results <- matrix(NA, length(embed_methods), length(quality_methods),
                              dimnames = list(embed_methods, quality_methods))
embedded_data <- list()

for (e in embed_methods) {
  message("embedding: ", e)
  embedded_data[[e]] <- embed(dataset, e, .mute = c("message", "output"))
  for (q in quality_methods) {
    message("  quality: ", q)
    quality_results[e, q] <- tryCatch(
      quality(embedded_data[[e]], q),
      error = function(e) NA
    )
  }
}

print(quality_results)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{as.data.frame}
\alias{as.data.frame}
\title{Converts to data.frame}
\usage{
as.data.frame(x, row.names, optional, ...)
}
\arguments{
\item{x}{The object to be converted}

\item{row.names}{unused in \code{dimRed}}

\item{optional}{unused in \code{dimRed}}

\item{...}{other arguments.}
}
\description{
General conversions of objects created by \code{dimRed} to \code{data.frame}.
See class documentations for details (\code{\link{dimRedData}},
\code{\link{dimRedResult}}). For the documentation of this function in base
package, see here: \code{\link[base]{as.data.frame.default}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixColorSpaces.R
\name{mixColorRamps}
\alias{mixColorRamps}
\alias{mixColor1Ramps}
\alias{mixColor2Ramps}
\alias{mixColor3Ramps}
\title{Mixing color ramps}
\usage{
mixColorRamps(vars, ramps)

mixColor1Ramps(vars, ramps = colorRamp(c("blue", "black", "red")))

mixColor2Ramps(
  vars,
  ramps = list(colorRamp(c("blue", "green")), colorRamp(c("blue", "red")))
)

mixColor3Ramps(
  vars,
  ramps = list(colorRamp(c("#001A00", "#00E600")), colorRamp(c("#00001A", "#0000E6")),
    colorRamp(c("#1A0000", "#E60000")))
)
}
\arguments{
\item{vars}{a list of variables}

\item{ramps}{a list of color ramps, one for each variable.}
}
\description{
mix different color ramps
}
\details{
automatically create colors to represent a varying number of
dimensions.
}
\examples{
cols <- expand.grid(x = seq(0, 1, length.out = 10),
                    y = seq(0, 1, length.out = 10),
                    z = seq(0, 1, length.out = 10))
mixed <- mixColor3Ramps(cols)

\dontrun{
if(requireNamespace("rgl", quietly = TRUE)) {
rgl::plot3d(cols$x, cols$y, cols$z, col = mixed, pch = 15)
}

cols <- expand.grid(x = seq(0, 1, length.out = 10),
                    y = seq(0, 1, length.out = 10))
mixed <- mixColor2Ramps(cols)

if(requireNamespace("graphics", quietly = TRUE)) {
plot(cols$x, cols$y, col = mixed, pch = 15)
}
}
}
